/***************************************************************************
 *   Copyright (C) 2008 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blokkalactiondispatcher.h"
#include "blokkalactiondispatcher.moc"

#include "blokkalmainwindow.h"

#include <blokkalaccount.h>
#include <blokkalblog.h>
#include <blokkalutils.h>
#include <blokkalactiontargetrelay.h>
#include <blokkalbehavioursettings.h>

#include <kxmlguifactory.h>
#include <kmenu.h>
#include <kaction.h>
#include <kactioncollection.h>
#include <klocale.h>
#include <kstandardaction.h>
#include <kdebug.h>
#include <kurl.h>

class BlokkalActionDispatcher::Private {
public:
	Private( BlokkalMainWindow * mainWindow ) :
	mainWindow( mainWindow ),
	accountTitle( 0 ),
	connectAction( 0 ),
	blogSeparator( 0 ),
	blogTitle( 0 ),
	hoveredAction( 0 ),
	menusConnected( FALSE ){}

	BlokkalMainWindow * const mainWindow;

	QAction * accountTitle;
	KAction * connectAction;
	QList<QAction*> accountActionList;
	QAction * blogSeparator;

	QAction * blogTitle;
	KAction * newEntryAction;
	KAction * loadEntryAction;
	KAction * openBlogAction;
	QList<QAction*> blogActionList;
	QMap<QAction*, Blokkal::Blog*> actionBlogMap;
	QAction * hoveredAction;

	bool menusConnected;
};

BlokkalActionDispatcher::BlokkalActionDispatcher( BlokkalMainWindow * mainWindow ):
QObject( mainWindow ),
d( new Private( mainWindow ) )
{
	//account popup actions
	d->connectAction = new KAction( mainWindow->actionCollection() );
	connect( d->connectAction,
	         SIGNAL( triggered( void ) ),
	         SLOT( connectAccount( void ) ) );
	mainWindow->actionCollection()->addAction( "connect_disconnect", d->connectAction );

	//blog popup actions
	d->newEntryAction = KStandardAction::openNew( this,
	                                              SLOT( startNewEntry( void ) ),
	                                              this );
	d->newEntryAction->setShortcut( 0 );
	d->newEntryAction->setText( i18n( "Create New Entry" ) );
	d->newEntryAction->setShortcut( i18n( "Ctrl+N" ) );
	mainWindow->actionCollection()->addAction( "file_new", d->newEntryAction );
	
	d->loadEntryAction = KStandardAction::open( this,
	                                            SLOT( browseEntries( void ) ),
	                                            this );
	d->loadEntryAction->setShortcut( 0 );
	d->loadEntryAction->setText( i18n( "Load Old Entry" ) );
	d->loadEntryAction->setShortcut( i18n( "Ctrl+L" ) );
	mainWindow->actionCollection()->addAction( "browse_entries", d->loadEntryAction );

	d->openBlogAction = KStandardAction::open( this,
	                                           SLOT( openBlog( void ) ),
	                                           this );
	d->openBlogAction->setShortcut( 0 );
	d->openBlogAction->setText( i18n( "Open Blog" ) );
	d->openBlogAction->setShortcut( i18n( "Ctrl+B" ) );
	mainWindow->actionCollection()->addAction( "open_blog", d->openBlogAction );

	slotCurrentBlogChanged( Blokkal::ActionTargetRelay::currentBlog() );
	connect( Blokkal::ActionTargetRelay::self(),
	         SIGNAL( currentBlogChanged( Blokkal::Blog * ) ),
	         SLOT( slotCurrentBlogChanged( Blokkal::Blog * ) ) );
}

BlokkalActionDispatcher::~BlokkalActionDispatcher( void )
{
	delete d;
}

KMenu * BlokkalActionDispatcher::accountMenu( void )
{
	return dynamic_cast<KMenu*>( d->mainWindow->guiFactory()->container( "account_menu", d->mainWindow ) );
}

KMenu * BlokkalActionDispatcher::configureAccountMenu( unsigned int flags )
{
	KMenu * const menu = accountMenu(); 

	if( !menu ) {
		return menu;
	}

	Blokkal::Account * const account = Blokkal::ActionTargetRelay::currentAccount();
	if( !account ) {
		return menu;
	}


	if( d->accountTitle ) {
		menu->removeAction( d->accountTitle );
		delete d->accountTitle;
		d->accountTitle = 0;
	}

	//remove all old blog actions
	const QList<QAction*> oldBlogActions = d->actionBlogMap.keys();
	for( int i = 0; i < oldBlogActions.size(); ++i ) {
		menu->removeAction( oldBlogActions[i] );
	}
	qDeleteAll( oldBlogActions );
	d->actionBlogMap.clear();
	if( d->blogSeparator ) {
		menu->removeAction( d->blogSeparator );
		delete d->blogSeparator;
		d->blogSeparator = 0;
	}

	//insert new blog actions
	const QList<Blokkal::Blog*> blogList = account->blogs();
	if( ( flags & IncludeBlogMenus )
	    && !blogList.isEmpty() )
	{
		if( !d->menusConnected ) {
			connect( accountMenu(),
			         SIGNAL( hovered( QAction * ) ),
			         SLOT( saveHoverAction( QAction * ) ) );
			connect( accountMenu(),
			         SIGNAL( aboutToHide( void ) ),
			         blogMenu(),
			         SLOT( hide( void ) ) );
			connect( blogMenu(),
			         SIGNAL( aboutToHide( void ) ),
			         SLOT( configureDynamicMenues( void ) ) );
			d->menusConnected = TRUE;
		}

		d->blogSeparator = menu->insertSeparator( menu->actions().isEmpty() ? 0 : menu->actions().first() );
		for( int i = 0; i < blogList.size(); ++i ) {
			Blokkal::Blog * const blog = blogList[i];
			QAction * const menuAction = new KAction( blog->title().isEmpty() ? blog->id() : blog->title(),
			                                          menu );
			menu->insertAction( menu->actions().isEmpty() ? 0 : menu->actions().first(),
			                    menuAction );
			menuAction->setMenu( blogMenu() );
			d->actionBlogMap.insert( menuAction, blog );
			menuAction->setIcon( blog->account()->isConnected() ? blog->icon().pixmap(16,16) : Blokkal::Utils::fadePixmap( blog->icon().pixmap(16,16) ) );
		}
	}

	KIcon icon = account->icon();
	icon = account->isConnected() ? icon : KIcon( Blokkal::Utils::fadePixmap( icon.pixmap(16,16) ) );
	d->accountTitle = menu->addTitle( icon, account->id(), menu->actions().isEmpty() ? 0 : menu->actions().first() );
	connect( d->accountTitle,
	         SIGNAL( destroyed( void ) ),
	         SLOT( accountMenuTitleDestroyed( void ) ) );

	if( account->isDisconnected() ) {
		d->connectAction->setText( i18n( "Connect" ) );
		d->connectAction->setIcon( KIcon( "network-connect" ) );
	}
	else {
		d->connectAction->setText( i18n( "Disconnect" ) );
		d->connectAction->setIcon( KIcon( "network-disconnect" ) );
	}
	for( int i = 0; i < d->accountActionList.size(); ++i ) {
		menu->removeAction( d->accountActionList[i] );
		if( i == ( d->accountActionList.size() - 1 ) ) {
			delete d->accountActionList[i]; //the last item  is always the separator, see below
		}
	}
	
	d->accountActionList = account->actionList();
	if( !d->accountActionList.isEmpty() ) {
		QAction * const sep = menu->addSeparator();
		for( int i = 0; i < d->accountActionList.size(); ++i ) {
			menu->addAction( d->accountActionList[i] );
		}
		d->accountActionList.append( sep );
	}

	return menu;
}

KMenu * BlokkalActionDispatcher::blogMenu( void )
{
	return dynamic_cast<KMenu*>( d->mainWindow->guiFactory()->container( "blog_menu", d->mainWindow ) );
}

KMenu * BlokkalActionDispatcher::configureBlogMenu( unsigned int /*flags*/ )
{
	KMenu * const menu = blogMenu();

	if( !menu ) {
		return menu;
	}

	Blokkal::Blog * const blog = Blokkal::ActionTargetRelay::currentBlog();
	if( !blog ) {
		return menu;
	}
	if( d->blogTitle ) {
		menu->removeAction( d->blogTitle );
		delete d->blogTitle;
		d->blogTitle = 0;
	}

	KIcon icon = blog->icon();
	icon = blog->account()->isConnected() ? icon : KIcon( Blokkal::Utils::fadePixmap( icon.pixmap(16,16) ) );
	d->blogTitle = menu->addTitle( icon,
	                               blog->title().isEmpty() ? blog->id() : blog->title(),
	                               menu->actions().isEmpty() ? 0 : menu->actions().first() );
	connect( d->blogTitle,
	         SIGNAL( destroyed( void ) ),
	         SLOT( blogMenuTitleDestroyed( void ) ) );
	
	d->loadEntryAction->setEnabled( blog->account()->isConnected() );
	d->openBlogAction->setEnabled( Blokkal::BehaviourSettings::self()->webPageLoadingPolicy() != Blokkal::BehaviourSettings::EnumWebPageLoadingPolicy::LoadDisabled );	
	
	for( int i = 0; i < d->blogActionList.size(); ++i ) {
		menu->removeAction( d->blogActionList[i] );
		if( i == ( d->blogActionList.size() - 1 ) ) {
			delete d->blogActionList[i]; //the last item  is always the separator, see below
		}
	}
	
	d->blogActionList = blog->actionList();
	if( !d->blogActionList.isEmpty() ) {
		QAction * const sep = menu->addSeparator();
		for( int i = 0; i < d->blogActionList.size(); ++i ) {
			menu->addAction( d->blogActionList[i] );
		}
		d->blogActionList.append( sep );
	}

	return menu;
}

void BlokkalActionDispatcher::connectAccount( void )
{
	Blokkal::Account * const account = Blokkal::ActionTargetRelay::currentAccount();
	if( !account ) {
		return;
	}

	if( account->isDisconnected() ) {
		account->connectAccount();
	}
	else {
		account->disconnectAccount();
	}
}

void BlokkalActionDispatcher::startNewEntry( void )
{
	Blokkal::Blog * const blog = Blokkal::ActionTargetRelay::currentBlog();
	if( !blog ) {
		return;
	}

	blog->startNewEntry();
}

void BlokkalActionDispatcher::browseEntries( void )
{
	Blokkal::Blog * const blog = Blokkal::ActionTargetRelay::currentBlog();
	if( !blog ) {
		return;
	}

	blog->browseEntries();
}

void BlokkalActionDispatcher::openBlog( void )
{
	Blokkal::Blog * const blog = Blokkal::ActionTargetRelay::currentBlog();
	if( !blog ) {
		return;
	}
	
	Blokkal::Utils::openWebpage( blog->url() );
}

void BlokkalActionDispatcher::saveHoverAction( QAction * action )
{
	d->hoveredAction = action;
	if( blogMenu()->isVisible() ) {
		return;
	}

	configureDynamicMenues();
}

void BlokkalActionDispatcher::configureDynamicMenues( void )
{
	if( !d->actionBlogMap.keys().contains( d->hoveredAction ) ) {
		return;
	}

	Blokkal::ActionTargetRelay::setCurrentBlog( d->actionBlogMap[d->hoveredAction] );
	configureBlogMenu();
}

void BlokkalActionDispatcher::blogMenuTitleDestroyed( void )
{
	d->blogTitle = 0;
}

void BlokkalActionDispatcher::accountMenuTitleDestroyed( void )
{
	d->accountTitle = 0;
}

void BlokkalActionDispatcher::slotCurrentBlogChanged( Blokkal::Blog * blog )
{
	d->newEntryAction->setEnabled( blog );
	d->loadEntryAction->setEnabled( blog && blog->account()->isConnected() );
	d->openBlogAction->setEnabled( blog
	                               && ( Blokkal::BehaviourSettings::webPageLoadingPolicy() != Blokkal::BehaviourSettings::EnumWebPageLoadingPolicy::LoadDisabled ) );
}
