/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "browsetabpages.h"
#include "browsetabpages.moc"

#include "entrylist.h"
#include "browseoptionwidgets.h"

#include <blokkalio/blogbrowsingdriver.h>
#include <blokkalio/jobs.h>

#include <kdatepicker.h>
#include <kdatetable.h>
#include <klocale.h>
#include <kdebug.h>

#include <QButtonGroup>
#include <QRadioButton>
#include <QGroupBox>
#include <QLayout>
#include <QTimer>

//---------------
//BrowseTabPage
//---------------
class BrowseTabPage::Private {
public:
	Private( Blokkal::Io::BlogBrowsingDriver * driver ):
	driver( driver ),
	loadEntryJob( 0 ){}

	Blokkal::Io::BlogBrowsingDriver * const driver;
	Blokkal::Io::LoadEntryJob * loadEntryJob;
	EntryList * entryList;
};

BrowseTabPage::BrowseTabPage( Blokkal::Io::BlogBrowsingDriver * driver, QWidget * parent ):
QWidget( parent ),
d( new Private( driver ) )
{
}

BrowseTabPage::~BrowseTabPage( void )
{
	delete d;
}

Blokkal::Io::BlogBrowsingDriver * BrowseTabPage::driver( void ) const
{
	return d->driver;
}

void BrowseTabPage::setLoadEntryJob( Blokkal::Io::LoadEntryJob * job )
{
	d->loadEntryJob = job;
}

Blokkal::Io::LoadEntryJob * BrowseTabPage::loadEntryJob( void ) const
{
	return d->loadEntryJob;
}

void BrowseTabPage::setEntryList( EntryList * entryList )
{
	d->entryList = entryList;
	connect( entryList,
	         SIGNAL( entrySelected( Blokkal::Entry* ) ),
	         SLOT( emitEntrySelected( Blokkal::Entry * ) ) );
}

void BrowseTabPage::emitEntrySelected( Blokkal::Entry * entry )
{
	emit entrySelected( entry );
}

EntryList * BrowseTabPage::entryList( void ) const
{
	return d->entryList;
}

void BrowseTabPage::announceJob( KJob * job, const QString & message )
{
	emit entrySelected( 0 );
	connect( job,
	         SIGNAL( result( KJob * ) ),
	         SLOT( handleFinishedJob( KJob * ) ) );
	emit jobReady( job, message );
}

//----------------
//HistoryTabPage
//----------------
class HistoryTabPage::Private {
public:
	Private( void ) {}

	QMap<QRadioButton*,Blokkal::Ui::BrowseOptionWidget *> buttonWidgetMap;
};

HistoryTabPage::HistoryTabPage( Blokkal::Io::BlogBrowsingDriver * _driver, QWidget * parent ):
BrowseTabPage( _driver, parent ),
d( new Private() )
{
	QVBoxLayout * historyPreviewLayout = new QVBoxLayout( this );
	QGroupBox * modeBox = new QGroupBox( i18n( "Entry Loading Options" ), this );
	modeBox->setLayout( new QVBoxLayout( modeBox ) );
	QButtonGroup * modeGroup = new QButtonGroup( modeBox );
	modeGroup->setExclusive( TRUE );

	//This list is used to guarantee button order.
	//QMap does not seem tu guarantee key order, so this
	//list makes sure everything is in the correct place.
	QList<QRadioButton*> buttonList;
	buttonList.append( new QRadioButton( modeBox ) );
	d->buttonWidgetMap.insert( buttonList.last(), new LoadRecentWidget( driver(), modeBox ) );
	buttonList.append( new QRadioButton( modeBox ) );
	d->buttonWidgetMap.insert( buttonList.last(), new LoadByIdWidget( driver(), modeBox ) );

	//adopt custom widgets
	QList<Blokkal::Ui::BrowseOptionWidget*> customWidgets = driver()->customBrowseOptionWidgets( modeBox );
	for( int i = 0; i < customWidgets.size(); ++i ) {
		buttonList.append( new QRadioButton( modeBox ) );
		d->buttonWidgetMap.insert( buttonList.last(), customWidgets[i] );
	}

	//setup browsewidgets and buttons
	for( QMap<QRadioButton*,Blokkal::Ui::BrowseOptionWidget *>::ConstIterator it = d->buttonWidgetMap.begin();
	     it != d->buttonWidgetMap.end();
	     ++it )
	{
		QHBoxLayout * currentLayout = new QHBoxLayout( 0 );
		currentLayout->setMargin( 0 );
		currentLayout->setSpacing( 0 );
		modeBox->layout()->addItem( currentLayout );
		modeBox->layout()->addWidget( it.key() );
		modeGroup->addButton( it.key() );
		currentLayout->addWidget( it.key() );
		currentLayout->addWidget( it.value() );
		connect( it.value(),
					SIGNAL( reloadRequested( Blokkal::Ui::BrowseOptionWidget * ) ),
					SLOT( refresh( void ) ) );
		connect( it.key(),
					SIGNAL( toggled( bool ) ),
					it.value(),
					SLOT( setEnabled( bool ) ) );
	}

	//setup load recent button
	buttonList[0]->setEnabled( driver()->supportedOperations() & Blokkal::Io::BlogBrowsingDriver::LoadRecentEntries );
	buttonList[0]->setChecked( buttonList[0]->isEnabled() );

	//setup load id button
	buttonList[1]->setEnabled( driver()->supportedOperations() & Blokkal::Io::BlogBrowsingDriver::LoadEntryById );
	buttonList[1]->setChecked( !buttonList[0]->isEnabled() && buttonList[1]->isEnabled() );

	//setting enabled state for widgets
	for( QMap<QRadioButton*,Blokkal::Ui::BrowseOptionWidget *>::ConstIterator it = d->buttonWidgetMap.begin();
	     it != d->buttonWidgetMap.end();
	     ++it )
	{
		it.value()->setEnabled( it.key()->isChecked() );
	}
	
	modeBox->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
	historyPreviewLayout->addWidget( modeBox );
	setEntryList( new EntryList( EntryListModel::DateGroups, this ) );
	historyPreviewLayout->addWidget( entryList() );
}

HistoryTabPage::~HistoryTabPage( void )
{
	delete d;
}

Blokkal::Entry * HistoryTabPage::entry( void ) const
{
	return entryList()->selectedEntry();
}

void HistoryTabPage::refresh( void )
{
	for( QMap<QRadioButton*,Blokkal::Ui::BrowseOptionWidget *>::ConstIterator it = d->buttonWidgetMap.begin();
	     it != d->buttonWidgetMap.end();
	     ++it )
	{
		if( it.key()->isChecked() ) {
			setLoadEntryJob( it.value()->createLoadEntryJob() );
			break;
		}
	}

	if( !loadEntryJob() ) {
		kDebug() << "no loadEntryjob() " << endl;
		return;
	}

	entryList()->clear();
	announceJob( loadEntryJob(), i18n( "Loading entries." ) );
}

void HistoryTabPage::handleFinishedJob( KJob * job )
{
	if( job == loadEntryJob() ) {
		if( !job->error() ) {
			entryList()->setEntries( loadEntryJob()->entries() );

		}
		setLoadEntryJob( 0 );
	}
}

void HistoryTabPage::cancelJobs( void )
{
	if( loadEntryJob() ) {
		loadEntryJob()->kill();
		entryList()->clear();
		setLoadEntryJob( 0 );
	}
}

//---------------
//CalendarTabPage
//---------------
class CalendarTabPage::Private {
public:
	Private( void ):
	loadIndexJob( 0 ) {}

	Blokkal::Io::LoadIndexJob * loadIndexJob;
	
	KDatePicker * datePicker;

	QMap< QDate, QStringList > index;
};

CalendarTabPage::CalendarTabPage( Blokkal::Io::BlogBrowsingDriver * driver, QWidget * parent ):
BrowseTabPage( driver, parent ),
d( new Private() )
{
	QHBoxLayout * datePreviewLayout = new QHBoxLayout( this );
	d->datePicker = new KDatePicker( this );
	connect( d->datePicker,
	         SIGNAL( dateChanged( QDate ) ),
	         SLOT( slotDateChanged( QDate ) ) );
	datePreviewLayout->addWidget( d->datePicker );
	//KDatePicker behaves strangely without a fixed size
	//(resizing when changing the month for the first time, etc...)
	//so the size is fixed here. KDatePicker suggests adding 28 pixels to
	//what is reported by sizeHint()
	d->datePicker->setFixedWidth( d->datePicker->sizeHint().width() + 30 );
	d->datePicker->setMinimumHeight( d->datePicker->sizeHint().height() + 30 );
	setEntryList( new EntryList( EntryListModel::NoGroups, this ) );
	datePreviewLayout->addWidget( entryList() );
	datePreviewLayout->setStretchFactor( entryList(), 1 );
}

CalendarTabPage::~CalendarTabPage( void )
{
	delete d;
}

Blokkal::Entry * CalendarTabPage::entry( void ) const
{
	return entryList()->selectedEntry();
}

void CalendarTabPage::refresh( void )
{
	if( !loadIndex() ) {
		loadCurrentDay();
	}
}

bool CalendarTabPage::loadIndex( void )
{
	const QDate baseDate = d->datePicker->date();
	QDate from( baseDate.year(), baseDate.month(), 1 );
	QDate to( baseDate.year(), baseDate.month(), baseDate.daysInMonth() );
	
	for( QMap< QDate, QStringList >::ConstIterator it = d->index.begin();
	     it != d->index.end();
	     it++ )
	{
		d->datePicker->dateTable()->unsetCustomDatePainting( it.key() );
	}

	d->loadIndexJob = driver()->createLoadIndexJob( from, to );

	announceJob( d->loadIndexJob, i18n( "Loading index." ) );
	
	return d->loadIndexJob;
}

void CalendarTabPage::handleFinishedJob( KJob * job )
{
	if( job == loadEntryJob() ) {
		if( !job->error() ) {
			entryList()->setEntries( loadEntryJob()->entries() );
		}
		setLoadEntryJob( 0 );
	}
	else if( job == d->loadIndexJob ) {
		QMap< QDate, QStringList > index = d->loadIndexJob->index();
		for( QMap< QDate, QStringList >::ConstIterator it = index.begin();
		     it != index.end();
		     it++ )
		{
			if( !it.value().isEmpty() ) {
				d->datePicker->dateTable()->setCustomDatePainting( it.key(), palette().link().color() );
			}
		}
		d->index = index;

		if( d->index.contains( d->datePicker->date() )
			&& !d->index[ d->datePicker->date() ].isEmpty()
			&& !job->error() )
		{
			QTimer::singleShot( 0, this, SLOT( loadCurrentDay( void ) ) );
		}
		
		d->loadIndexJob = 0;
	}
}

void CalendarTabPage::loadCurrentDay( void )
{
	setLoadEntryJob( driver()->createLoadEntryJob( d->datePicker->date() ) );
	if( !loadEntryJob() ) {
		return;
	}
	
	entryList()->clear();

	announceJob( loadEntryJob(), i18n( "Loading entries." )  );
}

void CalendarTabPage::slotDateChanged( QDate date )
{
	entryList()->clear();
	emit entrySelected( 0 );

	if(driver()->supportedOperations() & Blokkal::Io::BlogBrowsingDriver::LoadIndex ) {
		if( d->index.contains( date ) && !d->index[date].isEmpty() ) {
			loadCurrentDay();
		}
	}
	else {
		loadCurrentDay();
	}
}

void CalendarTabPage::cancelJobs( void )
{
	if( loadEntryJob() ) {
		loadEntryJob()->kill();
		entryList()->clear();
		setLoadEntryJob( 0 );
	}

	if( d->loadIndexJob ) {
		d->loadIndexJob->kill();
		entryList()->clear();
		for( QMap< QDate, QStringList >::ConstIterator it = d->index.begin();
		     it != d->index.end();
		     it++ )
		{
			d->datePicker->dateTable()->unsetCustomDatePainting( it.key() );
		}
		d->index = QMap< QDate, QStringList >();
	}
}
