/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "entrylist.h"
#include "entrylist.moc"

#include <blokkalentry.h>

#include <klocale.h>
#include <kdebug.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kicon.h>

#include <QPixmap>
#include <QItemDelegate>
#include <QPainter>
#include <QHeaderView>
#include <QRegExp>
#include <QSortFilterProxyModel>
#include <QDateTime>
#include <QApplication>

//-----------------------
//EntrySelectionModel
//-----------------------
class EntrySelectionModel : public QItemSelectionModel
{
public:
	EntrySelectionModel( QAbstractItemModel * model, EntryList * parent );
	virtual ~EntrySelectionModel( void );

public Q_SLOTS:
	virtual void select( const QModelIndex & index, QItemSelectionModel::SelectionFlags command );
	virtual void select( const QItemSelection & selection, QItemSelectionModel::SelectionFlags command );

private:
	EntryList * const m_signalProxy;
};

EntrySelectionModel::EntrySelectionModel( QAbstractItemModel * model, EntryList * parent ):
QItemSelectionModel( model, parent ),
m_signalProxy( parent )
{
}

EntrySelectionModel::~EntrySelectionModel( void )
{}

void EntrySelectionModel::select( const QModelIndex & index, QItemSelectionModel::SelectionFlags command )
{
	QItemSelectionModel::select( index, command );
}

void EntrySelectionModel::select( const QItemSelection & selection, QItemSelectionModel::SelectionFlags command )
{
	if( selection.indexes().size() != 1 ) {
		return;
	}

	QModelIndex index = selection.indexes().first();
	
	QItemSelectionModel::select( selection, command );
	if( m_signalProxy ) {
		/*if( command.testFlag( Deselect )
		    || ( command.testFlag( Clear ) && !command.testFlag( Select ) ) )
		{
			kDebug() << "emitting entrySelected with entry 0" << endl;
			emit m_signalProxy->entrySelected( 0 );
		}
		else*/ if( command.testFlag( Select ) ) {
			Blokkal::Entry * const entry = dynamic_cast<Blokkal::Entry *>( (QObject*) index.data().toULongLong() );
			kDebug() << "emitting entrySelected with entry " << entry << endl;
			emit m_signalProxy->entrySelected( entry );
		}
		else if( command != NoUpdate ) {
			kDebug() << "unhandled command: " << command << endl;
		}
	}
}

//-------------
//EntryDelegate
//-------------
class EntryDelegate : public QItemDelegate
{
public:
	EntryDelegate( QTreeView * parent );
	virtual ~EntryDelegate( void );

	virtual void paint(QPainter * painter,
	                   const QStyleOptionViewItem & option,
	                   const QModelIndex &index) const;
	virtual QSize sizeHint(const QStyleOptionViewItem & option,
	                       const QModelIndex & index) const;

private:
	QStringList lineList( const QStringList & textList, QFontMetrics fontMetrics, int width  ) const;
	QTreeView * const m_view;
	static const int sm_lineHeight = 1;
	static const int sm_margin = 1;
};

EntryDelegate::EntryDelegate( QTreeView * parent ):
QItemDelegate( parent ),
m_view( parent )
{
}

EntryDelegate::~EntryDelegate( void )
{}

void EntryDelegate::paint( QPainter * painter,
                          const QStyleOptionViewItem & option,
                          const QModelIndex & index ) const
{
	QApplication::style()->drawPrimitive( QStyle::PE_PanelItemViewItem, &option, painter );
	
	const Qt::TextElideMode elideMode = option.textElideMode;
	int alignment = Qt::AlignVCenter;
	if( option.direction == Qt::LeftToRight ) {
		alignment |= Qt::AlignLeft;
	}
	else {
		alignment |= Qt::AlignRight;
	}
	
	QColor color = option.palette.text().color();
	if( option.state & QStyle::State_Selected ) {
		color = option.palette.highlightedText().color();
	}
	painter->save();
	painter->setPen( color );
	
	if( index.data().toDate().isValid() ) {
		const QRect iconRect( option.rect.x(),
		                      option.rect.y(),
		                      option.rect.height(),
		                      option.rect.height() );
		drawDecoration( painter,
		                option,
		                iconRect,
		                ( m_view && m_view->isExpanded( index ) ) ? SmallIcon( "folder-open" ) : SmallIcon( "folder" ) );
		const QRect textRect( iconRect.right(),
		                      option.rect.y(),
		                      option.rect.width() - iconRect.width(),
		                      option.rect.height() );
		painter->drawText( textRect, 
		                   alignment, 
		                   KGlobal::locale()->formatDate( index.data().toDate() ) );
		
		//don't forget to restore painter upon early return!
		painter->restore();
		return;
	}
	
	const Blokkal::Entry * const entry = dynamic_cast<Blokkal::Entry*>( (QObject*) index.data().toULongLong() );
	if( !entry ) {
		kDebug() << "invalid data type encountered!" << endl;
		return;
	}

	const int fontHeight = option.fontMetrics.height();
	//icon, time, subject
	const QRect iconRect( option.rect.x(), option.rect.y(),
	                      fontHeight, fontHeight );
	drawDecoration( painter, option, iconRect, entry->icon().pixmap(16,16) );
	QString timeString = KGlobal::locale()->formatTime( entry->date().time(), TRUE );
	const QRect timeRect( iconRect.right(), option.rect.y(),
	                      option.fontMetrics.width( timeString ) + 10*sm_margin, fontHeight );
	painter->drawText( timeRect, alignment, timeString );
	int remainingWidth = option.rect.width() - ( iconRect.width() + timeRect.width() );
	remainingWidth = remainingWidth < 0 ? 0 : remainingWidth;
	const QRect titleRect( timeRect.right(), option.rect.y(), remainingWidth, fontHeight );
	QStyleOptionViewItem titleOption = option;
	
	painter->save();
	QFont titleFont = option.font;
	titleFont.setBold( TRUE );
	titleFont.setItalic( TRUE );
	painter->setFont( titleFont );
	QString subject = entry->subject();
	if( subject.isEmpty() ) {
		if( !(option.state & QStyle::State_Selected) ) {
			painter->setPen( option.palette.color( QPalette::Disabled, QPalette::Text) );
		}

		subject = i18n( "No Subject" );
	}
	painter->drawText( titleRect,
	                   alignment,
	                   elidedText( painter->fontMetrics(),
	                               titleRect.width(),
	                               elideMode,
	                               subject ) );
	painter->restore();

	//line
	const QRect lineRect( option.rect.x(),
	                      titleRect.y() + titleRect.height() + sm_margin,
	                      option.rect.width(), sm_lineHeight );
	QLinearGradient gradient( lineRect.topLeft(),
	                          lineRect.bottomRight() );

	gradient.setColorAt( 0, color );
	gradient.setColorAt( 1, Qt::transparent );
	painter->save();
	painter->setRenderHint( QPainter::Antialiasing );
	painter->fillRect( lineRect, gradient );
	painter->restore();

	//entry text
	const QRect textRect( option.rect.x(),
	                      lineRect.y() + sm_margin + lineRect.height(),
	                      option.rect.width(),
	                      fontHeight );

	
	alignment = Qt::AlignTop|Qt::TextWordWrap;
	if( option.direction == Qt::LeftToRight ) {
		alignment |= Qt::AlignLeft;
	}
	else {
		alignment |= Qt::AlignRight;
	}
	
	//need to split the string and splice it back together
	//to find out what can be drawn in the first and second line
	const QStringList textList = index.data( EntryListModel::DisplayTextRole ).toString().split( QRegExp( "\\s+" ),
	                                                                                             QString::SkipEmptyParts );
	QStringList lineList = this->lineList( textList, option.fontMetrics, option.rect.width() );
	if( lineList.length() > 1 && lineList.length() != textList.length() ) {
		lineList.removeLast();
	}
	
	QRect lineBoundingRect;
	painter->drawText( textRect,
	                   alignment,
	                   lineList.join( " " ),
	                   &lineBoundingRect );
	
	const QStringList lineList2 = textList.mid( lineList.length() );
	const QRect textRect2( option.rect.x(),
	                       textRect.y() + textRect.height(),
	                       option.rect.width(),
	                       fontHeight );
	painter->drawText( textRect2,
	                   alignment,
	                   elidedText( painter->fontMetrics(),
	                               textRect.width(),
	                               elideMode,
	                               lineList2.join( " " ) ) );
	painter->restore();

}

QSize EntryDelegate::sizeHint( const QStyleOptionViewItem & option,
                               const QModelIndex & index ) const
{
	if( index.data().toDate().isValid() ) {
		return QSize( 1, option.fontMetrics.height() );
	}
	
	
	//need to get access to viewport to determine available width
	const QTreeView * const parentView = dynamic_cast<QTreeView*>(parent());
	if( !parentView ) { //should not happen....
		return QSize( 20, 20 );
	}
	
	//need to determine parentCount to compute total indentation
	int parentCount = 0;
	QModelIndex currentIndex = index.parent();
	while( currentIndex.isValid() ) {
		parentCount++;
		currentIndex = currentIndex.parent();
	}
	const int availableWidth = parentView->viewport()->size().width() - parentView->indentation();
	
	const Blokkal::Entry * const entry = dynamic_cast<Blokkal::Entry*>( (QObject*) index.data().toULongLong() );
	if( entry ) {
		int height = 0;
		const QStringList textList = index.data( EntryListModel::DisplayTextRole ).toString().split( QRegExp( "\\s+" ),
                                                                                                             QString::SkipEmptyParts );
		const QStringList lineList = this->lineList( textList,
		                                             option.fontMetrics,
		                                             availableWidth );
		if( lineList.length() == textList.length() ) {
			height += option.fontMetrics.height() * 2; //one line subject, one line text
		}
		else {
			height += option.fontMetrics.height() * 3; //one line subject, two lines text
		}
		height += sm_lineHeight;
		height += sm_margin * 2; //two around line
		return QSize( availableWidth, height );
	}
	kError() << "encountered invalid data type" << endl;
	//width, height
	return QSize( 20, 20 );
}

QStringList EntryDelegate::lineList( const QStringList & textList, QFontMetrics fontMetrics, int width ) const
{
	QStringList lineList;
	for( int wordCount = 1; wordCount <= textList.length(); ++wordCount ) {
		lineList = textList.mid( 0, wordCount );
		if( fontMetrics.width( lineList.join( " " ) ) >= width ) {
			break;
		}
	}
	return lineList;
}

//---------
//EntryList
//---------
class EntryList::Private {
public:
	Private( EntryListModel * model ):
	model( model ){}

	EntryListModel * const model;
};

EntryList::EntryList( EntryListModel::GroupingMode mode, QWidget * parent ) :
QTreeView( parent ),
d( new Private( new EntryListModel( mode, this ) ) )
{
	setRootIsDecorated( mode != EntryListModel::NoGroups );
	QSortFilterProxyModel * sortModel = new QSortFilterProxyModel( this );
	sortModel->setSourceModel( d->model );
	sortModel->setSortRole( EntryListModel::SortRole );
	setModel( sortModel );
	model()->sort( 0, Qt::DescendingOrder );
	setItemDelegate( new EntryDelegate( this ) );
	setSelectionModel( new EntrySelectionModel( model(), this ) );
	header()->hide();
}

EntryList::~EntryList( void )
{
	delete d;
}

void EntryList::setEntries( const QList< Blokkal::Entry * > & entries )
{
	d->model->setEntries( entries );
	expandAll();
}

void EntryList::clear( void )
{
	d->model->clear();
}

Blokkal::Entry * EntryList::selectedEntry( void ) const
{
	QModelIndexList selection = selectionModel()->selectedIndexes();
	if( selection.size() != 1 ) {
		return 0;
	}

	return dynamic_cast<Blokkal::Entry *>( (QObject*) selection.first().data().toULongLong() );
}

//--------------
//EntryListModel
//--------------
class EntryListModel::Private {
public:
	Private( GroupingMode mode ):
	mode( mode ) {}

	const GroupingMode mode;

	QList< QDate > dateList;
	QMap< QDate, QList<Blokkal::Entry*> > entryMap;
	QMap<Blokkal::Entry*, QString> displayTextMap;
};

//How the model works
//-------------------
//The model implementation uses the internalId()
//of QModelIndex. An id == 0 means, that the index
//represents a date. The actual date is available
//through d->dateList using row() as index.
//An id > 0 marks an index as representation
//of an entry. The id itself is the index of
//of the corresponding date in d->dateList + 1
EntryListModel::EntryListModel( GroupingMode mode, QObject * parent ):
QAbstractItemModel( parent ),
d( new Private( mode ) )
{
}

EntryListModel::~EntryListModel( void )
{
	delete d;
}

QModelIndex EntryListModel::index( int row, int column, const QModelIndex & parent ) const
{
	if ( !hasIndex( row, column, parent ) ) {
		return QModelIndex();
	}

	if( d->dateList.isEmpty() ) {
		return QModelIndex();
	}

	if( !parent.isValid() ) {
		//either date or entry, depends on mode
		switch( d->mode ) {
		case NoGroups:
			//must be entry
			return createIndex( row, column, 1 ); //only one entry in datelist
			break;
		case DateGroups:
			//must be date
			return createIndex( row, column, 0 );
			break;
		}
	}
	
	//always an entry
	return createIndex( row, column, parent.row() + 1 );
}

QModelIndex EntryListModel::parent( const QModelIndex & child ) const
{
	if ( !child.isValid() ) {
		return QModelIndex();
	}

	if( !child.internalId() ) {
		//must be date
		return QModelIndex();
	}
	else {
		//must be entry
		switch( d->mode ) {
		case NoGroups:
			//parent is root
			return QModelIndex();
			break;
		case DateGroups:
			//parent is date
			{
				return createIndex( child.internalId() - 1, 0, 0 ); //date index
			}
			break;
		}		
	}

	return QModelIndex();
}

int EntryListModel::rowCount( const QModelIndex & parent ) const
{
	if( d->dateList.isEmpty() ) {
		return 0;
	}
	
	if( !parent.isValid() ) {
		switch( d->mode ) {
		case NoGroups:
			return d->entryMap[d->dateList.first()].size();
			break;
		case DateGroups:
			return d->dateList.size();
			break;
		}
	}
	else if( !parent.internalId() ) {
		//parent must be a date
		return d->entryMap[d->dateList[parent.row()]].size();
	}

	return 0;
}

QVariant EntryListModel::data( const QModelIndex & index, int role ) const
{
	if ( !index.isValid() ) {
		 return QVariant();
	}

	if( !index.internalId() ) {
		//must be date
		if( role == Qt::DisplayRole || role == SortRole ) {
			return d->dateList[index.row()];
		}
	}
	else {
		//must be entry
		if( role == Qt::DisplayRole ) {
			return (qulonglong) d->entryMap[d->dateList[ index.internalId() - 1]].at( index.row() );
		}
		else if( role == DisplayTextRole ) {
			return d->displayTextMap[d->entryMap[d->dateList[ index.internalId() - 1]].at( index.row() )];
		}
		else if( role == SortRole ) {
			return d->entryMap[d->dateList[ index.internalId() - 1]].at( index.row() )->date().time();
		}
	}
	
	return QVariant();
}

int EntryListModel::columnCount( const QModelIndex & parent ) const
{
	Q_UNUSED( parent );
	return 1;
}

void EntryListModel::setEntries( const QList< Blokkal::Entry * > & entries )
{
	for( QList<Blokkal::Entry*>::ConstIterator it = entries.begin();
	     it != entries.end();
	     ++it )
	{
		const QDate date = (*it)->date().date();
		if( !d->dateList.contains( date ) ) {
			d->dateList.append( date );
		}

		(*it)->ref();
		connect( *it,
		         SIGNAL( entryDestroyed( Blokkal::Entry * ) ),
		         SLOT( slotEntryDestroyed( Blokkal::Entry * ) ) );
		d->entryMap[date].append( *it );
		QString displayText = (*it)->text();
		QRegExp tagRegExp( "<([A-Z]|[a-z]|/)+.*>" ); //this is actuall not quite cporrect, because . matches \n as well
		tagRegExp.setMinimal( TRUE );
		displayText.remove( tagRegExp );
		displayText.remove( '\r' );
		displayText.replace( '\n', ' ' );
		displayText = displayText.left( 600 );
		d->displayTextMap[*it] = displayText;
	}
	reset();
}

void EntryListModel::clear( void )
{
	d->dateList.clear();
	d->displayTextMap.clear();
	const QMap<QDate, QList<Blokkal::Entry*> > entryMap = d->entryMap;
	d->entryMap.clear();

	//deref all entries so they are properly deleted
	for( QMap<QDate, QList<Blokkal::Entry*> >::ConstIterator it = entryMap.begin();
	     it != entryMap.end();
	     ++it )
	{
		const QList<Blokkal::Entry*> entryList = it.value();
		for( int i = 0; i < entryList.size(); ++i ) {
			entryList[i]->deref();
		}
	}
	reset();
}

void EntryListModel::slotEntryDestroyed( Blokkal::Entry * entry )
{
	const QDate date = entry->date().date();
	if( !d->dateList.contains( date ) ) {
		return;
	}

	const int index = d->entryMap[date].indexOf( entry );
	beginRemoveRows( createIndex( d->dateList.indexOf( date ), 0, 0 ), index, index );
	d->entryMap[date].removeAll( entry );
	d->displayTextMap.remove( entry );
	endRemoveRows();
}
