/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "accountchooser.h"
#include "accountchooser.moc"

#include <blokkalaccountmanager.h>
#include <blokkalaccount.h>
#include <blokkalprotocol.h>

#include <klocale.h>
#include <kdebug.h>
#include <kicon.h>


#include <QtGui/QSortFilterProxyModel>

//----------------
//AccountListModel
//----------------
class AccountListModel::Private {
public:
	Private( Blokkal::AccountList list):
	list(list)
	{}
	
	Blokkal::AccountList list;
};

AccountListModel::AccountListModel( AccountChooser * parent ):
QAbstractItemModel(parent),
d( new Private( Blokkal::AccountManager::self()->accounts() ) )
{
	
	connect( Blokkal::AccountManager::self(), SIGNAL( accountRegistered( Blokkal::Account * ) ),
		 this, SLOT( connectAccount( Blokkal::Account * ) ) );
	connect( Blokkal::AccountManager::self(), SIGNAL( accountUnregistered( Blokkal::Account * ) ),
		 this, SLOT( removeAccount( Blokkal::Account * ) ) );
	
	for( Blokkal::AccountList::ConstIterator it = d->list.begin();
	     it != d->list.end();
	     ++it )
	{
		connect( *it,
		         SIGNAL( weightChanged( Blokkal::Account * ) ),
		         SLOT( accountWeightChanged( Blokkal::Account * ) ) );
	}
}

AccountListModel::~AccountListModel(void)
{
	delete d;
}

QModelIndex AccountListModel::index( int row,
                                     int column,
                                     const QModelIndex & parent) const
{
	if ( !hasIndex( row, column, parent ) ) {
		return QModelIndex();
	}

	if ( parent.isValid() ) {
		return QModelIndex();
	}

	return createIndex( row, column );
}

QModelIndex AccountListModel::parent( const QModelIndex & child ) const
{
	Q_UNUSED( child );
	return QModelIndex();
}

int AccountListModel::rowCount( const QModelIndex & parent ) const
{
	if( parent.isValid() ) {
		return 0;
	}

	return Blokkal::AccountManager::self()->accounts().size();
}

int AccountListModel::columnCount( const QModelIndex & parent) const
{
	Q_UNUSED( parent );
	return 2;
}

QVariant AccountListModel::data( const QModelIndex & index,
                                 int role) const
{
	if( !index.isValid() ) {
		return QVariant();
	}

	const int row = index.row();
	if(  row >= d->list.size() ) {
		return QVariant();
	}

	const int column = index.column();
	if( column == 0 ) {
		if( role == Qt::DecorationRole ) {
			return QVariant::fromValue( d->list.at(row)->icon().pixmap(16,16) );
		}
		else if( role == SortRole ) {
			return d->list.at(row)->weight();
		}
		else if ( role == Qt::DisplayRole ) {			
			return d->list.at(row)->protocol()->visibleName();
		}
		else {
			return QVariant();
		}
	}
	else if( column == 1 ) {
		if ( role == Qt::DisplayRole ) {
			return d->list.at(row)->id();
		}
		else {
			return QVariant();
		}
	}
	return QVariant();
}

QVariant AccountListModel::headerData( int section,
                     Qt::Orientation orientation,
                     int role ) const
{
	Q_UNUSED( orientation );
	if( role != Qt::DisplayRole ) {
		return QVariant();
	}

	switch( section ) {
	case 0:
		return i18n("Type");
		break;
	case 1:
		return i18n("Account Name");
		break;
	default:
		break;
	}

	return QVariant();
}

void AccountListModel::accountWeightChanged( Blokkal::Account * account ) {
	const int row = d->list.indexOf( account );
	
	emit dataChanged( createIndex( row, 0 ),
			  createIndex( row, columnCount() - 1 ) );
}

void AccountListModel::connectAccount( Blokkal::Account * account ) {
	if( !account ) {
		return;
	}
	
	connect( account, 
		 SIGNAL( weightChanged( Blokkal::Account * ) ),
		 SLOT( accountWeightChanged( Blokkal::Account * ) ) );
	
	const int row = d->list.size();
	beginInsertRows( QModelIndex(), row, row );
	d->list.append( account );
	endInsertRows();
}

void AccountListModel::removeAccount( Blokkal::Account * account ) {
	if( !account ) {
		return;
	}
	
	const int row = d->list.indexOf( account );
	
	beginRemoveRows( QModelIndex(), row, row );
	d->list.removeAll( account );
	endRemoveRows();
}

Blokkal::Account * AccountListModel::accountAt( int row ) const {
	return d->list.at( row );
}

//----------------
//AccountChooser
//----------------
class AccountChooser::Private {
public:
	Private( AccountListModel * model, QSortFilterProxyModel * sortModel ):
	model( model),
	sortModel( sortModel ){}
	
	AccountListModel * const model;
	QSortFilterProxyModel * const sortModel;
};

AccountChooser::AccountChooser( QWidget * parent ):
QTreeView( parent ),
d( new Private( new AccountListModel( this ),
   new QSortFilterProxyModel( this) ) )
{
	d->sortModel->setSourceModel( d->model );
	d->sortModel->setSortRole( AccountListModel::SortRole );
	setModel( d->sortModel );
	
	setSelectionMode( QAbstractItemView::SingleSelection );
	setSelectionBehavior( QAbstractItemView::SelectRows );
	
	setSortingEnabled( FALSE );
	setRootIsDecorated( FALSE );
	resizeColumnToContents( 0 );

	connect( selectionModel(),  
		 SIGNAL( selectionChanged( const QItemSelection &, const QItemSelection & ) ),
	         SLOT( emitAccountSelectionChanged( void ) ) );
	connect( d->model,
		 SIGNAL( dataChanged( const QModelIndex &, const QModelIndex & ) ),
		 SLOT( resort( void ) ) );
	
	resort();
}


AccountChooser::~AccountChooser( void )
{
	delete d;
}

Blokkal::Account * AccountChooser::selectedAccount( void ) const
{
	return d->model->accountAt( d->sortModel->mapToSource( selectedIndexes().first() ).row() );
}

void AccountChooser::emitAccountSelectionChanged( void )
{
	emit accountSelectionChanged( selectedAccount() );
}

void AccountChooser::resort( void )
{
	model()->sort( 0 );
}
