/***************************************************************************
 *   Copyright (C) 2009 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "rsddatafetcher.h"

#include <QtGui/QWidget>
#include <QtCore/QRegExp>
#include <QtXml/QDomElement>

#include <kio/job.h>
#include <kprogressdialog.h>
#include <klocale.h>

#include <kdebug.h>

class RsdDataFetcher::Private {
public:
	enum FetchType{ 
		NoFetch,
		FetchBlog,
		FetchLinkRsd,
		FetchDefaultRsd,
	};
	
	QByteArray responseBuffer;
	RsdDataFetcher * const fetcher;
	QWidget * const parentWidget;
	KProgressDialog * progressDialog;
	KIO::TransferJob * currentJob;
	unsigned int progress;	
	FetchType fetchType;
	QString currentBlogUrl;
	
	QStringList apiNames;
	QMap<QString,QString> apiLinkMap;
	QString preferredApi;
	QString engineName;
	QString engineLink;
	QString homePageLink;
	
			
 	Private( RsdDataFetcher * fetcher, QWidget * parentWidget ):
	fetcher( fetcher ),
	parentWidget( parentWidget ),
	progressDialog( 0 ),
	currentJob( 0 ),
	progress( 0 ),
	fetchType( NoFetch ){}
	
	void startGetJob( const KUrl & url,
	                  FetchType _fetchType )
	{
		fetchType = _fetchType;
		responseBuffer.clear();
		KIO::TransferJob * job = KIO::get( url );
		job->addMetaData( "ConnectTimeout", "300" );
		currentJob = job;
		connect( job,
		         SIGNAL( data( KIO::Job *, const QByteArray & ) ),
		         fetcher,
		         SLOT( slotDataArrived( KIO::Job *, const QByteArray & ) ) );
	
		connect( job,
		         SIGNAL( finished( KJob * ) ),
		         fetcher,
		         SLOT( slotJobFinished( KJob * ) ) );

		connect( job,
		         SIGNAL( percent ( KJob *, unsigned long ) ),
		         fetcher,
		         SLOT( updateProgress( KJob * ) ) );
		job->start();
	}
	
	void cleanupJob( void ) {
		fetchType = NoFetch;
		responseBuffer.clear();
		currentJob = 0;
	}
	
	void destroyProgressDialog( void ) {
		if( progressDialog ) {
			progressDialog->hide();
			progressDialog->deleteLater();
			progressDialog = 0;
		}
	}
	
	bool readRsdData( void ) {
		//clear old data
		apiNames.clear();
		apiLinkMap.clear();
		preferredApi = QString();
		engineName = QString();
		engineLink = QString();
		homePageLink = QString();
		
		QDomDocument rsdDocument;
		if( !rsdDocument.setContent( responseBuffer, FALSE ) ) {
			return FALSE;
		}
		
		if( rsdDocument.documentElement().firstChild().isNull() ) {
			return FALSE;
		}
		
		//find service node
		QDomNode serviceNode;
		QDomNode currentNode = rsdDocument.documentElement().firstChild();
		while( !currentNode.isNull() ) {
			if( currentNode.nodeName().toLower() == QString::fromLatin1( "service" ) ) {
				serviceNode = currentNode;
				break;
			}
			
			currentNode = currentNode.nextSibling();
		}
		
		if( serviceNode.isNull() ) {
			return FALSE;
		}
		
		//find apis node
		//collect engineName, engineLink and homePgaeLink while we go
		QDomNode apisNode;
		currentNode = serviceNode.firstChild();
		while( !currentNode.isNull() ) {
			//TODO read engineName etc.
			if( currentNode.nodeName().toLower() == QString::fromLatin1( "apis" ) ) {
				apisNode = currentNode;
			}
			else if( currentNode.nodeName().toLower() == QString::fromLatin1( "enginename" ) ) {
				engineName = currentNode.toElement().text();
			}
			else if( currentNode.nodeName().toLower() == QString::fromLatin1( "enginelink" ) ) {
				engineLink = currentNode.toElement().text();
			}
			else if( currentNode.nodeName().toLower() == QString::fromLatin1( "homepagelink" ) ) {
				homePageLink = currentNode.toElement().text();
			}
			
			
			if( !apisNode.isNull()
			    && !engineName.isEmpty()
			    && !engineLink.isEmpty()
			    && !homePageLink.isEmpty() )
			{
				break;
			}
			
			currentNode = currentNode.nextSibling();
		}
		
		if( apisNode.isNull() ) {
			return FALSE;
		}
		
		
		//collect api information
		QDomElement apiElement;
		currentNode = apisNode.firstChild();
		while( !currentNode.isNull() ) {
			if( currentNode.isElement() 
			    && currentNode.nodeName().toLower() == QString::fromLatin1( "api" ) ) 
			{
				apiElement = currentNode.toElement();
				const QString apiName = apiElement.attribute( QString::fromLatin1( "name" ) );
				const QString apiLink = apiElement.attribute( QString::fromLatin1( "apiLink" ) );
				const bool isPreferred = apiElement.attribute( QString::fromLatin1( "preferred" ) ).toLower() == QString::fromLatin1( "true" ) ? TRUE : FALSE;
				if( !apiName.isNull()
				    && !apiLink.isNull()
				    && !apiLinkMap.keys().contains( apiName ) )
				{
					apiNames.append( apiName );
					apiLinkMap.insert(apiName, apiLink);
					if( isPreferred ) {
						preferredApi = apiName;
					}
				}
			}
			
			currentNode = currentNode.nextSibling();
		}
		
		return TRUE;
	}
};

RsdDataFetcher::RsdDataFetcher( QWidget * parent ):
QObject( parent ),
d( new Private( this, parent ) )
{
}

RsdDataFetcher::~RsdDataFetcher( void )
{
	delete d;
}

void RsdDataFetcher::fetchData( const QString & url )
{
	//Q: What happens when an operation is already going on?
	//A: Can't happen because we have a modal progress dialog
	//   blocking the UI
	//
	//2 slot activations may still trigger an new load operation
	//returning when d->progressDialog != 0 to avoid problems.
	if( d->progressDialog ) {
		return;
	}
	
	//reloading stuff we already have is somewhat pointless....
	if( url == currentBlogUrl() ) {
		return;
	}
	
	d->progressDialog = new KProgressDialog( d->parentWidget,
	                                         QString::null,
	                                         QString::null );
	
	d->progress = 0;
	d->progressDialog->setModal( TRUE );
	d->progressDialog->progressBar()->setMinimum( 0 );
	d->progressDialog->progressBar()->setMaximum( 100 );
	d->progressDialog->progressBar()->setValue( 0 );
	d->progressDialog->setAutoClose( FALSE );
	d->progressDialog->setAllowCancel( TRUE );
	d->progressDialog->setMinimumDuration( 100 );
	d->progressDialog->setLabelText( i18n( "Loading blog" ) );
	connect( d->progressDialog,
	         SIGNAL( cancelClicked( void ) ),
	         SLOT( cancelJobs( void ) ) );
	
	d->startGetJob( url, Private::FetchBlog );
	d->currentBlogUrl = url;
}

void RsdDataFetcher::slotDataArrived( KIO::Job * job, const QByteArray & data )
{
	if ( job && data.size() > 0 ) {
		d->responseBuffer.append( data );
	}
}

void RsdDataFetcher::slotJobFinished( KJob * job )
{
	if( !job ) {
		return;
	}
	
	switch( d->fetchType ) {
	case Private::FetchBlog:
	{
		if( job->error() ) {
			d->destroyProgressDialog();
			emit dataReceived( currentBlogUrl(), FALSE );
		}
		else {
			const QString rsdLinkPattern = QString::fromLatin1( "<link\\s*rel=\"EditURI\"\\s*type=\"application/rsd\\+xml\"\\s*title=\"rsd\"\\s*href=\"http://[\\w./?!\\+-=%]+\"\\s*/?>" );
			const QRegExp rsdLinkExp = QRegExp( rsdLinkPattern, Qt::CaseInsensitive );
			const QString bufferString = d->responseBuffer;
			int offset = rsdLinkExp.lastIndexIn( bufferString );
			if( offset >= 0 ) {
				//success, load link rsd
				//extract link
				int length = rsdLinkExp.matchedLength();
				const QString rsdMatch = bufferString.mid( offset, length);
				const QRegExp linkExp = QRegExp( QString::fromLatin1("http://[\\w./?!\\+-=%]+"),
				                                 Qt::CaseInsensitive );
				offset = linkExp.lastIndexIn( rsdMatch );
				length = linkExp.matchedLength();
				const QString rsdUri = rsdMatch.mid( offset, length );
				
				//start rsd job
				d->progressDialog->setLabelText( i18n( "Loading configuration data" ) );				
				d->startGetJob( rsdUri, Private::FetchLinkRsd );
			}
			else {
				//no rsd link, load default rsd
				d->progressDialog->setLabelText( i18n( "Loading default configuration data" ) );
				KUrl url = KUrl( d->currentBlogUrl );
				url.addPath( QString::fromLatin1( "rsd.xml" ) );
				d->startGetJob( url, Private::FetchDefaultRsd );
			}
		}
		break;
	}
	case Private::FetchLinkRsd:
	{
		if( job->error() ) {
			//fail, try to load default rsd
			d->progressDialog->setLabelText( i18n( "Loading default configuration data" ) );
			KUrl url = KUrl( d->currentBlogUrl );
			url.addPath( QString::fromLatin1( "rsd.xml" ) );
			d->startGetJob( url, Private::FetchDefaultRsd );
		}
		else {
			//if success emit success
			const bool success = d->readRsdData();
			d->destroyProgressDialog();
			emit dataReceived( currentBlogUrl(), success );
		}
		break;
	}
	case Private::FetchDefaultRsd:
	{
		if( job->error() ) {
			//fail, emit fail
			d->destroyProgressDialog();
			emit dataReceived( currentBlogUrl(), FALSE );
		}
		else {
			//if success emit success
			const bool success = d->readRsdData();
			d->destroyProgressDialog();
			emit dataReceived( currentBlogUrl(), success );
		}
		break;
	}
	case Private::NoFetch:
	{
		break;
	}
	}
}

void RsdDataFetcher::updateProgress( KJob * job )
{
	if( !job ) {
		return;
	}
	
	d->progress += job->percent()/2;
	d->progressDialog->progressBar()->setValue( d->progress );
}

void RsdDataFetcher::cancelJobs(void )
{
	if( d->currentJob ) {
		d->currentJob->kill();
		d->cleanupJob();
	}
	
	d->destroyProgressDialog();
}

QStringList RsdDataFetcher::apiNames(void ) const
{
	return d->apiNames;
}

QString RsdDataFetcher::apiLink(const QString & apiName) const
{
	return d->apiLinkMap[apiName];
}

bool RsdDataFetcher::isPreferredApi(const QString & apiName) const
{
	return apiName == d->preferredApi;
}

QString RsdDataFetcher::currentBlogUrl(void ) const
{
	return d->currentBlogUrl;
}

QString RsdDataFetcher::preferredApi(void ) const
{
	return d->preferredApi;
}

QString RsdDataFetcher::homePageLink(void ) const
{
	return d->homePageLink;
}

QString RsdDataFetcher::engineName(void ) const
{
	return d->engineName;
}

QString RsdDataFetcher::engineLink(void ) const
{
	return d->engineLink;
}
