/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "queuemanager.h"
#include "queuemanager.moc"

#include "queuestatusview.h"

#include <blokkalentry.h>
#include <blokkalpostentryqueue.h>
#include <blokkalui/view.h>

#include <kpushbutton.h>
#include <klocale.h>
#include <kdialog.h>
#include <kstdguiitem.h>
#include <kguiitem.h>
#include <kmessagebox.h>

#include <qlayout.h>
#include <qtooltip.h>
#include <qlabel.h>

class EntryQueue::QueueManager::Private
{
public:
	Private( void ) {}

	QueueStatusView * statusView;
	KPushButton * editButton;
	KPushButton * deleteButton;
	KPushButton * retryButton;
	QLabel * errorMessageLabel;
};

EntryQueue::QueueManager::QueueManager( QWidget * parent ) :
QWidget( parent ),
d( new Private() )
{
	QHBoxLayout * mainLayout = new QHBoxLayout( this );
	mainLayout->setSpacing( KDialog::spacingHint() );
	mainLayout->setMargin( 0 );
	d->statusView = new QueueStatusView( this );
	mainLayout->addWidget( d->statusView );

	QVBoxLayout * buttonLayout = new QVBoxLayout( 0 );
	buttonLayout->setSpacing( KDialog::spacingHint() );

	d->editButton = new KPushButton( KStandardGuiItem::open(), this );
	d->editButton->setText( i18n( "&Open Entry" ) );
	d->editButton->setToolTip( i18n( "Removes the entry from the queue and opens it for editing." ) );
	buttonLayout->addWidget( d->editButton );
	d->deleteButton = new KPushButton( KStandardGuiItem::del(), this );
	buttonLayout->addWidget( d->deleteButton );
	d->errorMessageLabel = new QLabel( this );
	d->errorMessageLabel->setAlignment( Qt::AlignLeft | Qt::AlignTop );
	d->errorMessageLabel->setWordWrap( TRUE );
	d->errorMessageLabel->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Expanding );
	buttonLayout->addWidget( d->errorMessageLabel );
	d->retryButton = new KPushButton( KStandardGuiItem::reset(), this );
	d->retryButton->setText( i18n( "&Retry" ) );
	d->retryButton->setToolTip( i18n( "Make another attempt to post the entry to the server." ) );
	buttonLayout->addWidget( d->retryButton );
	//buttonLayout->addStretch( 1 );
	mainLayout->addLayout( buttonLayout );

	slotSelectionChanged();

	connect( d->editButton,
	         SIGNAL( clicked( void ) ),
	         SLOT( openSelectedEntry( void ) ) );
	connect( d->deleteButton,
	         SIGNAL( clicked( void ) ),
	         SLOT( deleteSelectedEntry( void ) ) );
	connect( d->retryButton,
	         SIGNAL( clicked( void ) ),
	         SLOT( resetSelectedEntry( void ) ) );
	connect( d->statusView,
	         SIGNAL( selectionChanged( void ) ),
	         SLOT( slotSelectionChanged( void ) ) );

	connect( Blokkal::PostEntryQueue::self(),
	         SIGNAL( entryStatusChanged( Blokkal::Entry*, const Blokkal::PostEntryStatus* ) ),
	         SLOT( slotSelectionChanged( void ) ) );
}

EntryQueue::QueueManager::~QueueManager( void )
{
	delete d;
}

void EntryQueue::QueueManager::openSelectedEntry( void )
{
	QueueStatusLVI * item = dynamic_cast<QueueStatusLVI *>( d->statusView->selectedItem() );
	if( !item ) {
		return;
	}

	Blokkal::Entry * entry = item->entry();

	if( Blokkal::PostEntryQueue::self()->unqueue( entry ) ) {
		entry->view()->showView();
	}
	else {
		KMessageBox::sorry( this,
		                    i18n( "The entry could not be removed from the queue. Maybe the entry is currently beeing posted to the server." ) );
	}
}

void EntryQueue::QueueManager::deleteSelectedEntry( void )
{
	QueueStatusLVI * item = dynamic_cast<QueueStatusLVI *>( d->statusView->selectedItem() );
	if( !item ) {
		return;
	}

	int result = KMessageBox::warningYesNo( this,
	                                        i18n( "Do you really want to delete this entry?" ) );

	if( result != KMessageBox::Yes ) {
		return;
	}
	
	Blokkal::Entry * entry = item->entry();
	if( Blokkal::PostEntryQueue::self()->unqueue( entry ) ) {
		entry->deleteLater();
	}
	else {
		KMessageBox::sorry( this,
		                    i18n( "The entry could not be removed from the queue. Maybe the entry is currently beeing posted to the server." ) );
	}
}

void EntryQueue::QueueManager::resetSelectedEntry( void )
{
	QueueStatusLVI * item = dynamic_cast<QueueStatusLVI *>( d->statusView->selectedItem() );
	if( !item ) {
		return;
	}
	
	Blokkal::PostEntryQueue::self()->clearError( item->entry() );
	slotSelectionChanged();
}

void EntryQueue::QueueManager::slotSelectionChanged( void )
{
	QueueStatusLVI * item = dynamic_cast<QueueStatusLVI *>( d->statusView->selectedItem() );
	const Blokkal::PostEntryStatus * status = 0;
	if( item ) {
		status = Blokkal::PostEntryQueue::self()->entryStatus( item->entry() );
	}
	
	if( status ) {
		bool mayUnqueue = status->status() != Blokkal::PostEntryStatus::StatusPosting;
		d->deleteButton->setEnabled( mayUnqueue );
		d->retryButton->setEnabled( mayUnqueue && status->error() );
		QString statusString;
		QString errorInfo;
		switch (status->status() ) {
		case Blokkal::PostEntryStatus::StatusError:
			statusString = i18n( "Error" );
			errorInfo = i18n( "<br><b>Error Code:</b> %1<br><b>Description:</b> %2",
			                  status->error(),
			                  status->errorString() );
			break;
		case Blokkal::PostEntryStatus::StatusPosting:
			statusString = i18n( "Posting..." );
			break;
		case Blokkal::PostEntryStatus::StatusNormal:
			statusString = i18n( "Wating..." );
			break;
		}
		d->errorMessageLabel->setText( i18n( "<b>Status</b>: %1%2",
		                                     statusString,
		                                     errorInfo ) );
		d->editButton->setEnabled( mayUnqueue );
	}
	else {
		d->editButton->setEnabled( FALSE );
		d->deleteButton->setEnabled( FALSE );
		d->retryButton->setEnabled( FALSE );
		d->errorMessageLabel->clear();
	}
}
