/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "queuestatusmonitor.h"
#include "queuestatusmonitor.moc"

#include <blokkalpostentryqueue.h>
#include <blokkalentry.h>
#include <blokkalblog.h>
#include <blokkalaccount.h>
#include <blokkalui/globalsettings.h>

#include <kdialog.h>
#include <klocale.h>
#include <kstringhandler.h>
#include <knotification.h>

#include <QToolButton>
#include <QLabel>
#include <QLayout>
#include <QToolTip>
#include <QLinkedList>

class EntryQueue::QueueStatusMonitor::Private
{
public:
	Private( void ) {}
	QToolButton * button;
	QLabel * label;
};

EntryQueue::QueueStatusMonitor::QueueStatusMonitor( QWidget * parent ):
QWidget( parent ),
d( new Private() )
{
	QHBoxLayout * layout = new QHBoxLayout( this );
	layout->setSpacing( KDialog::spacingHint() );
	layout->setMargin( 0 );
	d->button = new QToolButton( this );
	d->label = new QLabel( this );

	layout->addWidget( d->label );
	layout->addWidget( d->button );

	slotQueueStatusChanged();
	
	connect( Blokkal::PostEntryQueue::self(),
	         SIGNAL( entryAdded( Blokkal::Entry * ) ),
	         SLOT( slotQueueStatusChanged( void ) ) );
	connect( Blokkal::PostEntryQueue::self(),
	         SIGNAL( entryRemoved( Blokkal::Entry * ) ),
	         SLOT( slotQueueStatusChanged( void ) ) );
	
	connect( Blokkal::PostEntryQueue::self(),
	         SIGNAL( entryFailing( Blokkal::Entry * ) ),
	         SLOT( slotEntryFailing( Blokkal::Entry * ) ) );
	connect( Blokkal::PostEntryQueue::self(),
	         SIGNAL( entryPosted( Blokkal::Entry * ) ),
	         SLOT( slotEntryPosted( Blokkal::Entry * ) ) );

	connect( Blokkal::PostEntryQueue::self(),
	         SIGNAL( entryStatusChanged( Blokkal::Entry*, const Blokkal::PostEntryStatus* ) ),
	         SLOT( slotQueueStatusChanged( void ) ) );
}

EntryQueue::QueueStatusMonitor::~QueueStatusMonitor( void )
{
	delete d;
}

QToolButton * EntryQueue::QueueStatusMonitor::button( void ) const
{
	return d->button;
}

void EntryQueue::QueueStatusMonitor::slotQueueStatusChanged( void )
{
	int problems = 0;
	int queuedEntries = Blokkal::PostEntryQueue::self()->count();

	if( !queuedEntries ) {
		d->label->setText( i18n( "%1/%2", queuedEntries, problems ) );
		d->label->setToolTip( i18n( "No entries in queue" ) );
		return;
	}

	const Blokkal::PostEntryStatus * currentStatus;
	QLinkedList<Blokkal::Entry*> entries = Blokkal::PostEntryQueue::self()->entries();
	for( QLinkedList<Blokkal::Entry*>::ConstIterator it = entries.begin();
	     it != entries.end();
	     it++ )
	{
		currentStatus = Blokkal::PostEntryQueue::self()->entryStatus( *it );
		if( currentStatus && currentStatus->status() == Blokkal::PostEntryStatus::StatusError ) {
			problems++;
		}
	}

	QString toolTip = i18n( "Entries in queue: %1",  queuedEntries );
	if( problems ) {
		toolTip.append( i18n( "\nProblems: %1", problems ) );
	}
	d->label->setToolTip( toolTip );

	d->label->setText( i18n( "%1/%2", queuedEntries, problems ) );
}

void EntryQueue::QueueStatusMonitor::slotEntryFailing( Blokkal::Entry * entry )
{
	const Blokkal::PostEntryStatus * status = Blokkal::PostEntryQueue::self()->entryStatus( entry );
	if( !status ) {
		return;
	}

	QString blogInfo = entry->blog()->title().isEmpty()
		? entry->blog()->id()
		: KStringHandler::rsqueeze( entry->blog()->title(), 30 );
	QString errorString = status->errorString();
	
	KNotification * notification = new KNotification( "entry_failed", Blokkal::Ui::GlobalSettings::self()->mainWidget() );
	if( errorString.isEmpty() ) {
		notification->setText( i18n( "\"%1\"\n failed to post into\n%2 (%3)",
		                             entry->preview(),
		                             blogInfo,
		                             entry->blog()->account()->id() ) );
	}
	else {
		notification->setText( i18n( "\"%1\"\n failed to post into\n%2 (%3)\nReason: %4",
		                             entry->preview(),
		                             blogInfo,
		                             entry->blog()->account()->id(),
		                             errorString ) );
	}

	if( notification ) {
		notification->setFlags( KNotification::Persistent );
		connect( notification,
		         SIGNAL( activated( void ) ),
		         SLOT( performButtonAction( void ) ) );
		notification->sendEvent();
	}
}

void EntryQueue::QueueStatusMonitor::slotEntryPosted( Blokkal::Entry * entry )
{
	QString blogInfo = entry->blog()->title().isEmpty()
		? entry->blog()->id()
		: KStringHandler::rsqueeze( entry->blog()->title(), 30 );
	
	KNotification::event( "entry_posted",
	                      i18n( "\"%1\"\n successfully posted into\n%2 (%3)",
	                            entry->preview(),
	                            blogInfo,
	                            entry->blog()->account()->id() ),
	                      QPixmap(),
	                      Blokkal::Ui::GlobalSettings::self()->mainWidget() );
}

void EntryQueue::QueueStatusMonitor::performButtonAction( void )
{
	if( d->button ) {
		d->button->animateClick();
	}
}
