/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALACCOUNT_H
#define BLOKKALACCOUNT_H

#include <QtCore/QObject>

#include "blokkal_export.h"

class QAction;

class KUrl;
class QDomElement;
class KIcon;

namespace KWallet {
class Wallet;
}

namespace Blokkal  {
class Protocol;
class AccountConfig;
class Blog;
class BlogConfig;
namespace Io {
class CategoryManager;
}

/**
 * @short This class provides the interface for a bloggging account.
 *
 * This class provides the interface for a bloggging account.
 * @author Martin Mller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT Account : public QObject
{
Q_OBJECT
public:
	/**
	 * Enumeration for the connection state.
	 */
	enum ConnectionStatus {
		Connected = 1, ///<Account is connected
		Connecting = 2, ///<Account is connecting
		Disconnected = 3, ///<Account is disconnected
	};

	/**
	 * Enumeration for disconnect reasons
	 */
	enum DisconnectReason {
		UserDisconnect = 1, ///< Disconnected due to user action
		IOErrorDisconnect = 2, ///< Disconnected due to some IO error
		OtherDisconnect = 3, ///< Disconnected for some other reason
		AuthenticationDisconnect = 4, ///Disconnected due to wrong username/password
	};

	/**
	 * Destroys the object.
	 * The configuration will not be removed. If you want to destroy this account
	 * permanently, use removeAccount()
	 */
	virtual ~Account( void );

	/**
	 * Returns the protocol owning this account.
	 * @return the account's protocol
	 */
	Blokkal::Protocol * protocol( void ) const;
		
	/**
	 * Returns the server name this account uses.
	 * The key in the config is "servername".
	 *
	 * @return the server name
	 * @see connectAddress()
	 */
	QString serverName( void ) const;

	/**
	 * Sets the server name.
	 * @param serverName the new server name
	 */
	void setServerName( const QString & serverName );

	/**
	 * Returns the address this account connects to.
	 * @return the address this account connects to
	 */
	KUrl connectAddress( void ) const;

	/**
	 * Sets the connect address.
	 * @param address the connect address
	 */
	void setConnectAddress( const QString & address );
	
	/**
	 * Returns the whether this account uses the default
	 * address or not.
	 *
	 * @return TRUE if the default address is used
	 */
	bool usesDefaultConnectAddress( void ) const;

	/**
	 * Determines whether the connect address for this account is the
	 * default one or not
	 * @param useDefault TRUE if this account uses the default address
	 */
	void setUsesDefaultConnectAddress( bool useDefault );

	/**
	 * Returns whether this account may be connected automatically.
	 * @return whether this account may be connected automatically
	 */
	bool autoConnect( void ) const;

	/**
	 * Sets whether this account may be connected automatically.
	 * @param enable TRUE allow this account to be connected automatically
	 */
	void setAutoConnect( bool enable ) const;

	/**
	 * Returns the unique id of this account.
	 * @return the unique id of this account
	 */
	QString id( void ) const;

	/**
	 * Returns the user name this account uses.
	 * The key in the config is "username".
	 *
	 * @return the user name this account uses
	 */
	QString userName( void ) const;

	/**
	 * Sets the username used by this account.
	 * @param user the username used by this account.
	 */
	void setUserName( const QString & user );

	/**
	 * Returns the icon for this account
	 * @return the icon for this account
	 */
	virtual KIcon icon( void ) const;

	/**
	 * Removes the account from the configuration and deletes this object.
	 * You need to override this method only to delete additional configuration
	 * data.
	 */
	virtual void removeAccount( void );

	/**
	 * Returns the connection state of the account.
	 * @see setConnectionStatus
	 * @return the current connection status
	 */
	Blokkal::Account::ConnectionStatus connectionStatus( void ) const;

	/**
	 * This is equal to connectionStatus() == Connected
	 * @return TRUE if connectionStatus() == Connected
	 */
	bool isConnected( void ) const;

	/**
	 * This is equal to connectionStatus() == Disconnected
	 * @return TRUE if connectionStatus() == Disconnected
	 */
	bool isDisconnected( void ) const;

	/**
	 * Returns a list of currently registered blogs.
	 * @return current blogs
	 */
	QList< Blokkal::Blog * > blogs( void ) const;

	/**
	 * Returns the blog with the internal id @a id or 0
	 * if no such blog has been registered.
	 * @return the blog
	 */
	Blokkal::Blog * blog( const QString & id ) const;

	/**
	 * Returns the data directory, where an account may store its private
	 * data.
	 * @return the account data directory
	 */
	QString dataDirectory( void ) const;

	/**
	 * Returns a list of custom actions for context menus.
	 * The default implementation returns an empty list.
	 * @return a list of custom actions
	 */
	virtual QList<QAction*> actionList( void );

public Q_SLOTS:
	/**
	 * This method is called to prepare any succeeding actions.
	 * You are required to gather any required information
	 * (e.g. blogs accessible for the user ) and emit
	 * connectionStatusChanged(). Do not forget to set the new
	 * connection status with setConnectionStatus()
	 */
	virtual void connectAccount( void ) = 0;

	/**
	 * This method is called to close the current session. The default
	 * implementation just calls setConnectionStatus() and emits
	 * connectionStatusChanged().
	 * If @a reason indicates that the account was not disconnected at
	 * behalf of the user a notification will be generated.
	 * @note It is good practice to override this method, so that any pending
	 * connections and connection attempts are canceled.
	 * @param reason the reaon for the disconnect
	 * @param reasonString to use instead of a generic error message
	 */
	virtual void disconnectAccount( Blokkal::Account::DisconnectReason reason = UserDisconnect,
	                                const QString & reasonString = 0 );

	/**
	 * Restores the blogs
	 */
	void restoreBlogs( void );

	/**
	 * Returns the category manager for this account.
	 * @return the category manager for this account
	 */
	Blokkal::Io::CategoryManager * categoryManager( void ) const;

	/**
	 * @return the weight of the account, to be used for sorting accounts
	 */
	unsigned int weight( void ) const;

	/**
	 * Sets the weight of this account. The weight of an account is to be used
	 * for sorting acccounts.
	 * @param weight the new weight
	 * @see AccountManager::accounts()
	 */
	void setWeight( unsigned int weight );

Q_SIGNALS:
	/**
	 * This signal is emitted when the account object is destroyed
	 */
	void accountDestroyed( Blokkal::Account * account );

	/**
	 * This signals is emitted when the connection status changes.
	 * @param account the account that has a new connection status
	 */
	void connectionStatusChanged( Blokkal::Account * account );

	/**
	 * This signal is emitted, when a blog is registered by the account
	 */
	void blogRegistered( Blokkal::Blog * blog );

	/**
	 * This signal is emitted, when a blog is unregistered from the account.
	 */
	void blogUnregistered( Blokkal::Blog * blog );

	/**
	 * This signal is emitted, when the weight of this account is changed.
	 */
	void weightChanged( Blokkal::Account * account );

protected:
	/**
	 * Creates a new account for protocol @a protocol, with name @a id
	 * @param protocol the procotol of this account
	 * @param id the id of this account
	 */
	Account( Blokkal::Protocol * protocol, const QString & id = 0 );
	
	/**
	 * Sets the value returned by @ref connectionStatus.
	 * Emits @ref connectionStatusChanged().
	 * @param status the new connection status
	 */
	void setConnectionStatus( Blokkal::Account::ConnectionStatus status );

	/**
	 * Returns the account config object for this account.
	 * @return the account config
	 */
	Blokkal::AccountConfig * config( void ) const;
	
	/**
	 * Returns the node of the blog with name @a blogName if it exists.
	 * If it does not exist, a new node for this blog will be created, but not
	 * inserted in the tree.
	 * This method is used by the BlogConfig constructor.
	 * @param id get the node for this blog.
	 * @return the configuration node
	 */
	QDomElement blogNode( const QString & id );
	
	/**
	 * Restores the blog according to the information in @a config.
	 * If you do not want or are unable to restore the blog for some reason
	 * return 0.
	 * @param config restore the blog with this configuration
	 * @return the restored blog or 0 if the blog was not restored
	 * @see Blokkal::Blog::saveProperties()
	 */
	virtual Blokkal::Blog * restoreBlog( Blokkal::BlogConfig * config ) = 0;
	
protected Q_SLOTS:
	/**
	 * Checks that @a blog does not already exists and registers it. If it tries
	 * to take over an already registered blog, @a blog will be deleted.
	 * If a registered blog is deleted it will be unregistered automatically
	 * @param blog the new blog
	 * @return @a blog or 0 if it was not registered
	 * @see unregisterBlog()
	 */
	Blokkal::Blog * registerBlog( Blokkal::Blog * blog );

	/**
	 * Unregisters a blog.
	 * @param blog blog to unregister
	 * @see registerBlog()
	 */
	void unregisterBlog( Blokkal::Blog * blog );

private:
	class Private;
	Private * const d;
	friend class BlogConfig;
};

/**
 * @short This class provides the password handling for an account.
 *
 * This class provides the password handling for an account.
 * You will need to override connectInner() instead of connect().
 * @author Martin Mller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT PasswordedAccount : public Blokkal::Account
{
Q_OBJECT
public:	
	/**
	 * Destroys the object.
	 * The configuration will not be removed. If you want to destroy this account
	 * permanently, use removeAccount()
	 */
	virtual ~PasswordedAccount( void );

	/**
	 * Destroys the object and removes the configuration data.
	 * This method has been overridden to remove the password from
	 * the wallet as well.
	 */
	virtual void removeAccount( void );

public Q_SLOTS:
	/**
	 * This method has been overridden in to fetch the password,
	 * either from the wallet or by asking the user.
	 */
	virtual void connectAccount( void );

	/**
	 * This method is called to close the current session.
	 * This method has been overridden to reset the password if @a reason
	 * was AuthenticationDisconnect.
	 * @param reason the reaon for the disconnect
	 * @param reasonString to use instead of a generic error message
	 */
	virtual void disconnectAccount( Blokkal::Account::DisconnectReason reason = UserDisconnect,
	                                const QString & reasonString = 0 );
	
protected:
	/**
	 * Creates a new account for protocol @a protocol, with name @a id
	 * @param protocol the procotol of this account
	 * @param id the id of this account
	 */		
	PasswordedAccount( Blokkal::Protocol * protocol, const QString & id = 0 );

	/**
	 * Returns a reference to the stored password
	 * @return a reference to the stored password
	 */
	const QString & password( void ) const;

protected Q_SLOTS:
	/**
	 * Place the actual connection code in this method.
	 * This method will be called after the password has been
	 * aquired and the actual connection process can begin.
	 * If the password could not be aquired, this slot will not
	 * be called and disconnectAccount() will be called instead.
	 */
	virtual void connectInner( void ) = 0;

private Q_SLOTS:
	void slotWalletOpened( KWallet::Wallet * wallet );
	
private:
	QString promptUser( bool & success, KWallet::Wallet * wallet );
	
	class Private;
	Private * const d;

};

} //Blokkal




#endif
