/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALACCOUNTMANAGER_H
#define BLOKKALACCOUNTMANAGER_H

#include <QtCore/QObject>

#include "blokkal_export.h"

class QDomElement;

namespace Blokkal {
class Account;
class AccountConfig;
class Protocol;

/**
 * A list of Accounts.
 */
typedef QList< Blokkal::Account * > AccountList;

class AccountManagerPrivate;

/**
 * @short Global account manager class.
 *
 * This is the global account manager class. It maitains information of all accounts and
 * saves the configuration of registered accounts.
 * 
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT AccountManager : public QObject
{
friend class AccountManagerPrivate;
Q_OBJECT
public:
	/**
	 * Returns a pointer to the global account manager. If no account manager
	 * exists yet, one will be created.
	 *
	 * @return global account manager
	 */
	static Blokkal::AccountManager * self( void );

	/**
	 * Returns a list of available accounts.
	 * The list is sorted according to the weight of the account with the
	 * heaviest account being in the last in the list returned.
	 * @return registered accounts
	 * @see Account::setWeight()
	 */
	Blokkal::AccountList accounts( void );

	/**
	 * Checks that @a account does not already exists and registers it. If it tries
	 * to take over an already registered account, @a account will be deleted.
	 *
	 * @param account
	 * account to register
	 * @return account or 0 if it was not registered
	 */
	Blokkal::Account * registerAccount( Blokkal::Account * account );

	/**
	 * Returns the account @a id if such an account has been registered
	 * or 0 if no such account exists.
	 *
	 * @param id query for this account
	 * @return the desired account or 0 if it does not exist
	 */
	Blokkal::Account * account( const QString & id );
	

public Q_SLOTS:
	/**
	 * Loads the account information and creates the accounts.
	 */
	void loadAccounts( void );

	/**
	 * Saves the account information to the account file.
	 */
	void saveAccounts( void );

Q_SIGNALS:
	/**
	 * This signal is emitted whenever an account is registered.
	 *
	 * @param account the registered account
	 */
	void accountRegistered( Blokkal::Account * account );

	/**
	 * This signal is emitted whenever an account has been unregistered.
	 * Usually this means, that the account is about to be deleted, so
	 * you should not store the pointer.
	 *
	 * @param account the unregistered account
	 */
	void accountUnregistered( Blokkal::Account * account );
	
protected Q_SLOTS:
	/**
	 * Unregisters @a account
	 * @param account account to unregister
	 */
	void unregisterAccount( Blokkal::Account * account );

protected:
	/**
	 * Returns the node of the account with name @a id if it exists.
	 * If it does not exist, a new node for this account will be created, but not
	 * inserted in the tree. You should check whether the protocol attribute is set
	 * and set if if necessary.
	 * This method is used by the AccountConfig constructor.
	 * @param id get the node for this account.
	 * @return the configuration node
	 */
	QDomElement accountNode( const QString & id );

private:
	AccountManager( void );
	/**
	 * Destroys the object. Do not destroy the object before deleting every account!
	 */
	~AccountManager( void );
	friend class AccountConfig;
};

}

#endif
