/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALBLOKKALBLOG_H
#define BLOKKALBLOKKALBLOG_H

#include "blokkal_export.h"

#include <QtCore/QObject>

class QAction;

class KIcon;
class KUrl;

namespace Blokkal {
class Account;
class Entry;
class BlogConfig;

namespace Ui {
class EditEntryWidget;
}

namespace Io {
class BlogBrowsingDriver;
}

/**
 * @short An abstraction of a blog.
 *
 * An Abstraction of a blog.
 * This class is the base for all protocol
 * implementations of a blog.
 * Blogs are always children of an account object.
 * 
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT Blog : public QObject
{
Q_OBJECT
public:
	/**
	 * Constructs a new Blog object.
	 * @param parentAccount the account this blog belongs to
	 * @param id the internal identifier of this blog
	 */
	Blog( Blokkal::Account * parentAccount, const QString & id );
	
	/**
	 * Destroys the object.
	 * The configuration will not be removed.
	 * To delete this blog permanently, use removeBlog()
	 */
	virtual ~Blog( void );

	/**
	 * This enumeration is used to identify the standard features supported by this entry.
	 * The Blokkal::Ui::EditEntryWidget uses these values to determine which inputs
	 * to show/enable.
	 * @see supportedFeatures()
	 */
	enum EntryFeatures {
		FeatureDate = 0x01, ///< Entry can have a date
		FeatureSubject = 0x02, ///< Entry can have a subject line
		FeatureText = 0x04, ///< Entry can have a body text
		FeatureCategory = 0x08, ///< Entry can have a category
		FeatureAddCategories = 0x10 | FeatureCategory, ///< Categories may be added while editing the entry
		FeatureRemoveCategories = 0x20 | FeatureCategory, ///< Categories may be removed while editing the entry
		FeatureMultipleCategories = 0x40 | FeatureCategory, ///< Entry may be assigned to more than one category
	};

	/**
	 * Returns an or'ed list of Blokkal::Blog::EntryFeatures values to indicate
	 * the possible options of entry editing.
	 * The default implementation returns FeatureDate|FeatureSubject|FeatureText.
	 * @return an or'ed list of Blokkal::Blog::EntryFeatures values
	 */
	virtual unsigned int supportedFeatures( void ) const;	
	
	/**
	 * This methid returns a string describing the type of the blog.
	 * This type name is intended to be shown to the user.
	 * The default implementation returns i18n("Blog").
	 * @return a very short description of the blog type(preferrably one word)
	 */
	virtual QString type( void ) const;

	/**
	 * This method returns an icon, representing the type of the blog.
	 * The default implementation returns the account icon.
	 * @return an icon representing the type of the blog
	 */
	virtual KIcon icon( void ) const;

	/**
	 * This method returns the title of the blog. This string is intended
	 * to be shown to the user.
	 * @return the name or title of the blog.
	 */
	QString title( void ) const;

	/**
	 * Sets the title of the blog object.
	 * @param title the new title
	 */
	void setTitle( const QString & title );

	/**
	 * This method returns the internal identifier of the blog
	 * @return the internal identifier
	 */
	QString id( void ) const;

	/**
	 * This method returns the url of the blog.
	 * @return the url of the blog.
	 */
	virtual KUrl url( void ) const = 0;

	/**
	 * This method returns the account this blog belongs to
	 * @return the owning account
	 */
	Blokkal::Account * account( void ) const;

	/**
	 * This method creates a widget to edit an entry.
	 * @param entry the entry the widget is to edit, if 0 a new entry is created
	 * @param parent the parent widget
	 */
	Blokkal::Ui::EditEntryWidget * createEditEntryWidget( Blokkal::Entry * entry, QWidget * parent );

	/**
	 * Creates a new @ref Blokkal::Entry object.
	 * @return a new Blokkal::Entry object.
	 */
	virtual Blokkal::Entry * createEntry( void ) = 0;

	/**
	 * @return the BlogBrowsingDriver of this blog
	 */
	virtual Blokkal::Io::BlogBrowsingDriver * blogBrowsingDriver( void ) const = 0;

	/**
	 * Removes the blog from the configuration and deletes this object.
	 */
	void removeBlog( void );

	/**
	 * Saves the properties of this blog, so that it can be restored before
	 * the parent account connects to the server. Standard properties are saved
	 * automatically. If you store your data in the config() object, you
	 * do not need to reimplement this method.
	 * The default implementation does nothing.
	 */
	virtual void saveProperties( void );
	
	/**
	 * Returns a list of custom actions for context menus.
	 * The default implementation returns an empty list.
	 * @return a list of custom actions
	 */
	virtual QList<QAction*> actionList( void );

public Q_SLOTS:
	/**
	 * Creates a new entry and displays a view to edit the new entry.
	 */
	void startNewEntry( void );

	/**
	 * Displays a view to browse past entries.
	 */
	void browseEntries( void );

Q_SIGNALS:
	/**
	 * This signal is emitted, when the properties of a blog have
	 * been changed.
	 */
	void propertiesChanged( Blokkal::Blog * blog );

	/**
	 * This signal is emitted when the blog object is destroyed
	 */
	void blogDestroyed( Blokkal::Blog * blog );

protected:
	/**
	 * Returns the blog config object for this account.
	 * @return the blog config
	 */
	Blokkal::BlogConfig * config( void ) const;

private:
	class Private;
	Private * const d;
};

}

#endif
