/***************************************************************************
 *   Copyright (C) 2007 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blokkalblogmodel.h"
#include "blokkalblogmodel.moc"

#include "blokkalaccountmanager.h"
#include "blokkalaccount.h"
#include "blokkalblog.h"
#include "blokkalio/categorymanager.h"

#include <kdebug.h>

class Blokkal::BlogModel::Private {
public:
	Private( void ){}
};

Blokkal::BlogModel::BlogModel( QObject * parent ):
TreeModel( 0, parent ),
d( new Private() )
{
	Blokkal::AccountManager * manager = Blokkal::AccountManager::self();
	Blokkal::AccountList accounts = manager->accounts();
	for( Blokkal::AccountList::ConstIterator it = accounts.begin();
	     it != accounts.end();
	     ++it )
	{
		addAccount( *it );
	}

	connect( manager,
	         SIGNAL( accountRegistered( Blokkal::Account* ) ),
	         SLOT( addAccount( Blokkal::Account * ) ) );
	connect( manager,
	         SIGNAL( accountUnregistered( Blokkal::Account* ) ),
	         SLOT( removeAccount( Blokkal::Account* ) ) );	
}

Blokkal::BlogModel::~BlogModel( void )
{
	delete d;
}

Blokkal::BlogModel::TreeItemNode * Blokkal::BlogModel::findAccountNode( Blokkal::Account * account )
{
	TreeItemNode * node;
	for( int i = 0; i != root()->children.size(); ++i ) {
		node = root()->children.at(i);
		if( ((Blokkal::Account*)node->data.toULongLong() ) == account ) {
			return node;
		}
	}

	return 0;
}

Blokkal::BlogModel::TreeItemNode * Blokkal::BlogModel::findBlogNode( Blokkal::Blog * blog )
{
	TreeItemNode * accountNode = findAccountNode( blog->account() );
	if( !accountNode ) {
		kError() << "unable to find account node for blog " << blog->id() << endl;
		return 0;
	}

	TreeItemNode * node;
	for( int i = 0; i != accountNode->children.size(); ++i ) {
		node = accountNode->children.at(i);
		if( ((Blokkal::Blog*)node->data.toULongLong() ) == blog ) {
			return node;
		}
	}

	return 0;
}

void Blokkal::BlogModel::addAccount( Blokkal::Account * account )
{
	//new nodes are always appended
	beginInsertRows( QModelIndex(), root()->children.size(), root()->children.size() );
	new TreeItemNode( root(), account );
	endInsertRows();
	
	connect( account,
	         SIGNAL( blogRegistered( Blokkal::Blog * ) ),
	         SLOT( addBlog( Blokkal::Blog * ) ) );
	connect( account,
				SIGNAL( blogUnregistered( Blokkal::Blog * ) ),
				SLOT( removeBlog( Blokkal::Blog * ) ) );	
	connect( account,
	         SIGNAL( connectionStatusChanged( Blokkal::Account* ) ),
	         SLOT( emitDataChanged( Blokkal::Account* ) ) );
	connect( account,
	         SIGNAL( weightChanged( Blokkal::Account * ) ),
	         SLOT( emitDataChanged( Blokkal::Account * ) ) );

	QList<Blokkal::Blog*> blogs = account->blogs();	
	for( QList<Blokkal::Blog*>::ConstIterator it = blogs.begin();
			it != blogs.end();
			++it )
	{
		addBlog( *it );
	}
}

void Blokkal::BlogModel::addBlog( Blokkal::Blog * blog )
{
	if( !blog ) {
		return;
	}

	TreeItemNode * node = findAccountNode( blog->account() );
	if( !node ) {
		kError() << "unable to find account for blog" << endl;
		return;
	}

	beginInsertRows( createIndex( node->parent->children.indexOf( node ), 0, node ),
	                 node->children.size(),
	                 node->children.size() );
	new TreeItemNode( node, blog );
	endInsertRows();
}

void Blokkal::BlogModel::removeAccount( Blokkal::Account * account )
{
	if( !account ) {
		return;
	}
	
	QList<Blokkal::Blog*> blogs = account->blogs();
	for( QList<Blokkal::Blog*>::ConstIterator it = blogs.begin();
			it != blogs.end();
			++it )
	{
		removeBlog( *it );
	}

	TreeItemNode * node = findAccountNode( account );
	if( !node ) {
		kError() << "unable to find node for account" << endl;
		return;
	}
	
	int index = node->parent->children.indexOf( node );
	beginRemoveRows( QModelIndex(),
	                 index,
	                 index );
	disconnect( account,
	            SIGNAL( blogRegistered( Blokkal::Blog * ) ),
	            this,
	            SLOT( addBlog( Blokkal::Blog * ) ) );
	disconnect( account,
	            SIGNAL( connectionStatusChanged( Blokkal::Account* ) ),
	            this,
	            SLOT( emitDataChanged( Blokkal::Account* ) ) );
	disconnect( account,
		    SIGNAL( weightChanged( Blokkal::Account* ) ),
	            this,
	            SLOT( emitDataChanged( Blokkal::Account* ) ) );
	delete node->parent->children.takeAt( index );
	endRemoveRows();
}

void Blokkal::BlogModel::removeBlog( Blokkal::Blog * blog )
{
	if( !blog ) {
		return;
	}

	TreeItemNode * node = findBlogNode( blog );
	if( !node ) {
		kError() << "unable to find node for blog" << endl;
		return;
	}

	int index = node->parent->children.indexOf( node );
	beginRemoveRows( createIndex( root()->children.indexOf( node->parent ),
	                              0,
	                              node->parent ),
	                 index,
	                 index );
	delete node->parent->children.takeAt( index );
	endRemoveRows();
}

QVariant Blokkal::BlogModel::data( const QModelIndex & index, int role ) const
{
	if( role != SortRole ) {
		return TreeModel::data( index, role );
	}
	
	const Blokkal::Account * const account = dynamic_cast<Blokkal::Account*>( (QObject*) index.data().toULongLong() );
	if( account ) {
		return account->weight();
	}

	const Blokkal::Blog * const blog = dynamic_cast<Blokkal::Blog*>( (QObject*) index.data().toULongLong() );
	if( blog ) {
		return blog->title().isEmpty() ? blog->id() : blog->title();
	}

	return QVariant();
}

void Blokkal::BlogModel::emitDataChanged( Blokkal::Account * account )
{
	TreeItemNode * accountNode = findAccountNode( account );
	if( accountNode ) {
		const QModelIndex accountIndex = createIndex( accountNode->parent->children.indexOf( accountNode ),
		                                              0,
		                                              accountNode );
		if( accountNode->children.isEmpty() ) {
			emit dataChanged( accountIndex, accountIndex );
		}
		else {
			const QModelIndex blogIndex = createIndex( accountNode->children.size() - 1 ,
			                                           0,
			                                           accountNode->children.last() );
			emit dataChanged( accountIndex, blogIndex );
		}
	}
}

//------------------
//BlogCategoryModel
//------------------
Blokkal::BlogCategoryModel::BlogCategoryModel( QObject * parent ):
BlogModel( parent )
{
	Blokkal::AccountManager * manager = Blokkal::AccountManager::self();
	Blokkal::AccountList accounts = manager->accounts();
	for( Blokkal::AccountList::ConstIterator ait = accounts.begin();
	     ait != accounts.end();
	     ++ait )
	{
		BlogModel::TreeItemNode * accountNode = findAccountNode( *ait );
		if( !accountNode) {
			kError() << "unable to find node for account: " << (*ait)->id() << endl;
			continue;
		}
		
		Blokkal::Account * account = dynamic_cast<Blokkal::Account*>( (QObject*)accountNode->data.toULongLong() );
		if( !account ) {
			kError() << "account node does not have an account as data" << endl;
			continue;
		}
			
		QList<Blokkal::Blog*> blogs = account->blogs();
		for( QList<Blokkal::Blog*>::ConstIterator bit = blogs.begin();
				bit != blogs.end();
				++bit )
		{
			addCategories( *bit );
		}
	}	
}

Blokkal::BlogCategoryModel::~BlogCategoryModel( void )
{
}

void Blokkal::BlogCategoryModel::addBlog( Blokkal::Blog * blog )
{
	BlogModel::addBlog( blog );
	addCategories( blog );
}

void Blokkal::BlogCategoryModel::removeBlog( Blokkal::Blog * blog )
{
	removeCategories( blog );
	BlogModel::removeBlog( blog );
}

void Blokkal::BlogCategoryModel::addCategory( const QString & blogId, Blokkal::Io::Category * category, Blokkal::Io::CategoryManager * manager )
{
	Blokkal::Blog * blog = findBlog( blogId, manager->account() );
	if( !blog ) {
		kError() << "cannot find blog with id " << blogId << endl;
		return;
	}

	TreeItemNode * node = findBlogNode( blog );
	if( !node ) {
		kError() << "cannot find node for blog " << blogId << endl;
		return;
	}

	beginInsertRows( createIndex( node->parent->children.indexOf( node ), 0, node->parent ),
	                 node->children.size(),
	                 node->children.size() );
	new TreeItemNode( node, category );
	endInsertRows();
}

void Blokkal::BlogCategoryModel::removeCategory( const QString & blogId, Blokkal::Io::Category * category, Blokkal::Io::CategoryManager * manager )
{
	Blokkal::Blog * blog = findBlog( blogId, manager->account() );
	if( !blog ) {
		kError() << "cannot find blog with id " << blogId << endl;
		return;
	}

	TreeItemNode * node = findBlogNode( blog );
	if( !node ) {
		kError() << "cannot find node for blog " << blogId << endl;
		return;
	}
	
	TreeItemNode * categoryNode;
	for( int i = 0; i != node->children.size(); ++i ) {
		categoryNode = node->children.at(i);
		if( ((Blokkal::Io::Category*)categoryNode->data.toULongLong() ) == category ) {
			break;
		}
		categoryNode = 0;
	}

	if( !categoryNode ) {
		return;
	}

	int index = node->children.indexOf( categoryNode );
	beginRemoveRows( createIndex( index, 0, node ),
	                 index,
	                 index );
	delete node->children.takeAt( index );
	endRemoveRows();
}

Blokkal::Blog * Blokkal::BlogCategoryModel::findBlog( const QString & blogId, const Blokkal::Account * account )
{
	Blokkal::Blog * blog = 0;
	QList<Blokkal::Blog*> blogs = account->blogs();
	for( QList<Blokkal::Blog*>::ConstIterator it = blogs.begin();
			it != blogs.end();
			++it )
	{
		if( (*it)->id() == blogId ) {
			blog = *it;
			break;
		}
	}

	return blog;
}

void Blokkal::BlogCategoryModel::addCategories( Blokkal::Blog * blog )
{
	BlogModel::TreeItemNode * blogNode = findBlogNode( blog );
	if( !blogNode) {
		kError() << "unable to find node for blog: " << blog->id() << endl;
		return;
	}

	Blokkal::Io::CategoryManager * manager = blog->account()->categoryManager();
	Blokkal::Io::CategoryList categories = manager->categories( blog->id() );
	for( Blokkal::Io::CategoryList::ConstIterator it = categories.begin();
			it != categories.end();
			++it )
	{
		addCategory( blog->id(), *it, manager );
	}

	connect( manager,
	         SIGNAL( categoryAdded( const QString&, Blokkal::Io::Category*, Blokkal::Io::CategoryManager* ) ),
	         SLOT( addCategory( const QString&, Blokkal::Io::Category*, Blokkal::Io::CategoryManager* ) ) );
	connect( manager,
	         SIGNAL( categoryRemoved( const QString&, Blokkal::Io::Category*, Blokkal::Io::CategoryManager* ) ),
	         SLOT( removeCategory( const QString&, Blokkal::Io::Category*, Blokkal::Io::CategoryManager* ) ) );
}

void Blokkal::BlogCategoryModel::removeCategories( Blokkal::Blog * blog )
{
	BlogModel::TreeItemNode * blogNode = findBlogNode( blog );
	if( !blogNode) {
		kError() << "unable to find node for blog: " << blog->id() << endl;
		return;
	}

	Blokkal::Io::CategoryManager * manager = blog->account()->categoryManager();
	Blokkal::Io::CategoryList categories = manager->categories( blog->id() );
	for( Blokkal::Io::CategoryList::ConstIterator it = categories.begin();
			it != categories.end();
			++it )
	{
		removeCategory( blog->id(), *it, manager );
	}	
}

QVariant Blokkal::BlogCategoryModel::data( const QModelIndex & index, int role ) const
{
	if( role != SortRole ) {
		return BlogModel::data( index, role );
	}

	const Blokkal::Io::Category * const category = dynamic_cast<Blokkal::Io::Category*>( (QObject*) index.data().toULongLong() );
	if( category ) {
		return category->name().isEmpty() ? category->id() : category->name() ;
	}

	return BlogModel::data( index, role );
}
