/***************************************************************************
 *   Copyright (C) 2007 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALBLOGMODEL_H
#define BLOKKALBLOGMODEL_H

#include "blokkaltreemodel.h"

#include "blokkal_export.h"

namespace Blokkal {
class Blog;
class Account;

/**
 * @short Wraps the account/blog structure for a view
 * 
 * This class is supposed to be used together with a BlogDelegate.
 * It wraps the pointers to the accounts and blogs into a QVariant.
 * The delegate will then handle the screen representation.
 */
class BLOKKAL_EXPORT BlogModel : public Blokkal::TreeModel
{
Q_OBJECT
public:
	/**
	 * Constructor
	 * @a parent parent object
	 */
	BlogModel( QObject * parent = 0 );
	~BlogModel( void );

	/**
	 * Enumeration for custom roles
	 */
	enum CustomRoles {
		SortRole = Qt::UserRole + 1 ///< Role to access data for sorting the items @see data()
	};

	/**
	 * Overridden to provide sortable data for use with a QProxySortModel or similar
	 * facilities.
	 * This method returns Account::weight() for accounts and Blog::title() or Blog::id()
	 * for blogs.
	 * @see SortRole
	 */
	virtual QVariant data( const QModelIndex & index,
	                       int role = Qt::DisplayRole) const;

protected:
	TreeItemNode * findAccountNode( Blokkal::Account * account );
	TreeItemNode * findBlogNode( Blokkal::Blog * blog );
	
protected Q_SLOTS:
	/**
	 * This slot is called when a new account has been created.
	 * @param account the new account
	 */
	virtual void addAccount( Blokkal::Account * account );

	/**
	 * This slot is called when a new blog has been created.
	 * @param blog the new blog
	 */
	virtual void addBlog( Blokkal::Blog * blog );

	/**
	 * This slot is called when an account is about to be deleted.
	 * @param account the account to remove
	 */
	virtual void removeAccount( Blokkal::Account * account );

	/**
	 * This slot is called when a blog is about to be deleted.
	 * @param blog the blog to remove
	 */
	virtual void removeBlog( Blokkal::Blog * blog );

	/**
	 * This slot can be used to notify the view that an account has
	 * changed in a way that requires an update of the screen representation
	 */
	void emitDataChanged( Blokkal::Account * account );

private:
	class Private;
	Private * const d;
};

namespace Io {
class CategoryManager;
class Category;
}

/**
 * @short a BlogModel that incorporates the categories of a blog
 *
 * This class works exactly like a BlogMode with the litte tiwst that it will
 * make the categories of a blog visible to the view.
 * This class is supposed to be used with a BlogDelegate.
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT BlogCategoryModel : public Blokkal::BlogModel
{
Q_OBJECT
public:
	/**
	 * Constructor
	 * @a parent parent object
	 */
	BlogCategoryModel( QObject * parent = 0 );
	~BlogCategoryModel( void );

	/**
	 * Overridden to provide sortable data for use with a QProxySortModel or similar
	 * facilities.
	 * This method returns Category::name() or Category::id() unless the object in question
	 * is not a category. The base class implementation will be called in that case.
	 * @see BlogModel::SortRole
	 */
	virtual QVariant data( const QModelIndex & index,
	                       int role = Qt::DisplayRole) const;

protected:
	Blokkal::Blog * findBlog( const QString & blogId, const Blokkal::Account * account );
	void addCategories( Blokkal::Blog * blog );
	void removeCategories( Blokkal::Blog * blog );
	
protected Q_SLOTS:
	/**
	 * This slot is called when a new blog has been created.
	 * @param blog the new blog
	 */
	virtual void addBlog( Blokkal::Blog * blog );

	/**
	 * This slot is called when a blog is about to be deleted.
	 * @param blog the blog to remove
	 */
	virtual void removeBlog( Blokkal::Blog * blog );

	virtual void addCategory( const QString & blogId,
	                          Blokkal::Io::Category * category,
	                          Blokkal::Io::CategoryManager * manager );
	virtual void removeCategory( const QString & blogId,
	                             Blokkal::Io::Category * category,
	                             Blokkal::Io::CategoryManager * manager );

};

}

#endif
