/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALBLOKKALCONFIGBASE_H
#define BLOKKALBLOKKALCONFIGBASE_H

#include "blokkal_export.h"
#include <QtCore/QString>

class QDomElement;

namespace Blokkal {

/**
 * @short Encapsulates a XML configuration element.
 *
 * Encapsulates a XML configuration node.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT ConfigBase
{
public:
	/**
	 * Destructor
	 */
	virtual ~ConfigBase( void );

	/**
	 * Removes the encapsulated configuration node from the parent node.
	 * Do not use this object after calling this method.
	 */
	void remove( void );
	
	/**
	 * Writes an entry to the config.
	 * @param name name of the property
	 * @param value new value of the property
	 * @see writeComplex()
	 */
	void writeEntry( const QString & name, const QString & value );

	/**
	 * Writes an entry to the config.
	 * @param name name of the property
	 * @param value new value of the property
	 */
	void writeEntry( const QString & name, bool value );

	/**
	 * Writes a complex entity to the config.
	 * If you need to store large properties, use this
	 * method rather than writeEntry().
	 * @param name of the complex
	 * @param value of the complex
	 */
	void writeComplex( const QString & name, const QString & value );

	/**
	 * Reads an entry stored with writeComplex.
	 * @param name of the complex
	 * @param defaultValue the default value to use if the complex does not exist
	 */
	QString readComplex( const QString & name, const QString & defaultValue = QString::null );
	
	/**
	 * Reads an entry from the config.
	 *
	 * @param name name of the property
	 * @param defaultValue defaultValue of the property. Used if no key with the given name can be found.
	 * @return the value of the property
	 */
	QString readEntry( const QString & name, const QString & defaultValue = QString::null ) const;

	/**
	 * Reads an entry from the config.
	 *
	 * @param name name of the property
	 * @param defaultValue defaultValue of the property. Used if no key with the given name can be found.
	 * @return the value of the property
	 */
	bool readBoolEntry( const QString & name, bool defaultValue = FALSE ) const;

	/**
	 * Sets the value of a property entry.
	 * @param name name of the property entry
	 * @param value the new value of the property entry
	 */
	void writeEntry( const QString & name, int value );

	/**
	 * Returns the value of the property entry <i>name</i> or <i>defaultValue</i>
	 * if the proprty entry does not exist or cannot be interpreted as an int.
	 * @param name the property to read
	 * @param defaultValue the value to use if the property entry does not exist
	 */
	int readIntEntry( const QString & name, int defaultValue = 0 ) const;

	/**
	 * Sets the value of a property entry.
	 * @param name name of the property entry
	 * @param value the new value of the property entry
	 */
	void writeEntry( const QString & name, unsigned int value );

	/**
	 * Returns the value of the property entry <i>name</i> or <i>defaultValue</i>
	 * if the proprty entry does not exist or cannot be interpreted as an unsigned int.
	 * @param name the property to read
	 * @param defaultValue the value to use if the property entry does not exist
	 */
	unsigned int readUIntEntry( const QString & name, unsigned int defaultValue = 0 ) const;
	
	/**
	 * Sets the value of a property entry.
	 * @param name name of the property entry
	 * @param value the new value of the property entry
	 */
	void writeEntry( const QString & name, unsigned long value );

	/**
	 * Returns the value of the property entry <i>name</i> or <i>defaultValue</i>
	 * if the proprty entry does not exist or cannot be interpreted as a ulong.
	 * @param name the property to read
	 * @param defaultValue the value to use if the property entry does not exist
	 */
	unsigned long readULongEntry( const QString & name, unsigned long defaultValue = FALSE ) const;
	
protected:
	/**
	 * Creates a new ConfigBase with a null node as internal representation.
	 * @see setNode()
	 */
	ConfigBase( void );
	
	/**
	 * Creates a new ConfigBase
	 * @param configNode the node that this object will use to store the data
	 */
	ConfigBase( QDomElement configNode );

	/**
	 * Sets the internal node to @a configNode.
	 */
	void setNode( QDomElement configNode );
	
	/**
	 * Returns the encapsulated element.
	 * @return the encapsulated element
	 */
	QDomElement node( void ) const;

	/**
	 * Returns the first property node with this property name
	 * or a null node, if it cannot find any.
	 * @param name the name of the property
	 * @return the first node with this property name or a null node if such a node cannot be found
	 */
	QDomElement findPropertyNode( const QString & name ) const;

	/**
	 * Creates a new property node.
	 * @param name the property name
	 * @return the new property node
	 */
	QDomElement createPropertyNode( const QString & name );

	/**
	 * Returns the first complex node with this complex name
	 * or a null node, if it cannot find any.
	 * @param name the name of the complex
	 * @return the first node with this complex name or a null node if such a node cannot be found
	 */
	QDomElement findComplexNode( const QString & name ) const;

	/**
	 * Creates a new complex node.
	 * @param name the complex name
	 * @return the new complex node
	 */
	QDomElement createComplexNode( const QString & name );

private:
	class Private;
	Private * d;
};

}

#endif
