/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALBLOKKALENTRY_H
#define BLOKKALBLOKKALENTRY_H

#include "blokkalconfigbase.h"
#include "blokkal_export.h"
#include "blokkalui/entryextensionwidget.h"

#include <kurl.h>

#include <QtCore/QObject>

class KIcon;

namespace Blokkal {
class Blog;
namespace Ui {
class View;
}
namespace Io {
class EntryJob;
}
/**
 * @short This class is a base for all protocol implementations of a blog entry.
 *
 * This class is a base for all protocol implementations of a blog entry.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT Entry : public QObject, protected Blokkal::ConfigBase
{
Q_OBJECT
public:
	/**
	 * Enumeration for Job types.
	 */
	enum JobType {
		PostJob = 1, ///< Id for a job that posts or edits an entry
		DeleteJob = 2, ///< Id for a job that deletes an entry
		UserJob = 10 ///< First type available for custom purposes
	};
		
	/**
	 * Destroys the entry object.
	 * If a view does still exist, it will be deleted.
	 * @note Do not delete the entry directly, use ref() and deref() instead. Also, the entry
	 * will delete itself when its view has been destroyed and it is not waiting for calls to deref().
	 */
	virtual ~Entry( void );

	/**
	 * Returns the id of this entry.
	 * @return the id
	 */
	QString id( void ) const;

	/**
	 * Returns a pointer to the parent blog object.
	 * @return the parent blog
	 */
	Blokkal::Blog * blog( void ) const;

	/**
	 * This method returns the text of the entry
	 * @return the text of the entry
	 */
	virtual QString text( void ) const;

	/**
	 * This method returns the subject of the entry.
	 * @return the subject
	 */
	virtual QString subject( void ) const;

	/**
	 * This method returns a prieview of the event.
	 * The default implementation returns @ref subject() unless the
	 * returned string is empty. In that case the first 100 characters of @ref text()
	 * are taken. If the event text is empty as well, i18n( "&gt;No Subject>" )
	 * is returned.
	 */
	virtual QString preview( void ) const;

	/**
	 * This method returns the icon of the entry.
	 * @return the entry icon
	 */
	virtual KIcon icon( void ) const;

	/**
	 * This method returns the date of the entry.
	 * @return the date
	 */
	QDateTime date( void ) const;

	/**
	 * This method sets the date of the entry.
	 * @param date the new date
	 */
	void setDate( const QDateTime & date );

	/**
	 * This method sets the subject of the entry.
	 * @param subject the new subject
	 */
	void setSubject( const QString & subject);

	/**
	 * This method sets the text of the entry.
	 * @param text the new text
	 */
	void setText( const QString & text );

	/**
	 * Creates and returns a job of @a type
	 * @return a job of @a type
	 */
	virtual Blokkal::Io::EntryJob * createJob( Blokkal::Entry::JobType type ) = 0;

	/**
	 * Returns the view for this entry.
	 */
	Blokkal::Ui::View * view( void );

	/**
	 * Returns the name of the file this entry has been loaded from or the name of the
	 * last file this entry has been saved to. This method returns QString::null
	 * if this entry has never been saved to or loaded from a file.
	 * This method also returns QString::null if the file this entry has been
	 * saved to has been removed or cannot be found for some other reason if
	 * @a check is TRUE
	 * @param check if TRUE, checks if the file really exists
	 * @return the filename
	 */
	QString fileName( bool check = FALSE ) const;

	/**
	 * Saves this entry to the @a fileName.
	 * @param fileName the name of the target file
	 * @return FALSE if the document could not be saved
	 */
	bool saveAs( const QString & fileName );

	/**
	 * Returns the widget associated with @a extension.
	 * The default implementation returns 0 for all values of @a extension
	 * @param extension the identifier for the widget to create
	 * @param parent the parent widget
	 * @return the widget associated with @a extension.
	 */
	virtual Blokkal::Ui::EntryExtensionWidget * createExtensionWidget( Ui::EntryExtensionWidget::ExtensionWidget extension, Blokkal::Ui::EditEntryWidget * parent );

	/**
	 * Loads an entry from the file @a fileName.
	 * If an error occured, this method returns 0.
	 * This method attempts to find the correct blog of this entry according
	 * to the information found in the file. If this fails, a dialog will ask
	 * the user to specify the correct blog.
	 * The entry object will be created by the parent blog, that was picked either
	 * by the user or according to the information stored in the file.
	 * @param fileName name of the file to read the entry from
	 * @param canceled if this parameter is not 0, it will be set to TRUE if the user cancels the process and to FALSE otherwise
	 * @param neverAsk nerver ask the user to specify the correct blog
	 * @return the entry or 0 if an error occured.
	 */
	static Blokkal::Entry * open( const QString & fileName, bool * canceled = 0, bool neverAsk = FALSE );

	/**
	 * Returns the ids of the categories this entry belongs to.
	 * @return ids of the categories this entry belongs to
	 */
	QStringList categories( void ) const;

	/**
	 * Sets the ids of the categories this entry belongs to.
	 * @param ids ids of the categories this entry belongs to
	 */
	void setCategories( const QStringList & ids );

	/**
	 * Merges the content of @a entry into this entry.
	 * This method copies the basic attributes of an entry:
	 * date, subject, text, categories.
	 * This method will not copy the id or the categories of @a entry.
	 * If you want to iverride this method, make sure you call this
	 * implementation to make sure the basic properties are merged properly.
	 * @param entry source entry
	 */
	virtual void merge( const Blokkal::Entry * entry );

	/**
	 * This method returns TRUE if it has been marked with setDirty()
	 * to be in a state where it needs to be saved to a file or
	 * to be posted to the server
	 * @return TRUE if contents is dirty
	 */
	bool isDirty( void ) const;

	/**
	 * Marks this entry as dirty.
	 * @see isDirty()
	 * @param dirty TRUE to mark this entry as dirty
	 */
	void setDirty( bool dirty );

	/**
	 * Returns the url of this entry
	 * This method is used by BrowseEntryView.
	 * @see Blog::supportedBrowseModes()
	 * @return the Url of this entry
	 */
	KUrl url( void ) const;

public Q_SLOTS:
	/**
	 * Increases the reference counter. Do not forget to call deref() for every ref()
	 */
	void ref( void );

	/**
	 * Decresases te reference counter. If it reaches 0 and no view is active for this entry,
	 * this entry is deleted.
	 */
	void deref( void );
	
Q_SIGNALS:
	/**
	 * This signal is emitted, when an entry is beeing deleted.
	 * @param entry the entry about to be destroyed
	 */
	void entryDestroyed( Blokkal::Entry * entry );

protected:
	/**
	 * Creates a new Entry object.
	 * @a id specifies the id of this entry. QString::null is considered an invalid
	 * id, given to entries that have not yet been posted to the server.
	 * This is the only check performed on this value, so you are free to
	 * use anything as an id for an entry.
	 * @param parentBlog the blog this entry belongs to
	 * @param id the id of the entry
	 * @param url url of this entry
	 */
	Entry( Blog * parentBlog, const QString & id = QString::null, const KUrl & url = KUrl() );

	/**
	 * Sets the filename, this method is used internally when an entry is being saved to
	 * or loaded from a file.
	 * @param fileName the file name
	 */
	void setFileName( const QString & fileName ) const;

	/**
	 * This method is called right before this entry is saved to disk.
	 * You do not need to reimplement this method if you store all
	 * relevant properties with writeEntry() and readEntry().
	 * The default implementation does nothing.
	 */
	virtual void saveProperties( void );

	/**
	 * This method is called right after the entry has been
	 * loaded and restored from disk. You do not need to reimplement
	 * this method if you access all relevant data with readEntry().
	 * The default implementation does nothing.
	 */
	virtual void readProperties( void );

private:
	class Private;
	Private * const d;
};

}

#endif
