/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALL_IOBLOGBROWSINGDRIVER_H
#define BLOKKALL_IOBLOGBROWSINGDRIVER_H

#include "../blokkal_export.h"

#include <QtCore/QObject>
#include <QtCore/QDate>

namespace Blokkal {
class Blog;
class Account;
namespace Ui {
class BrowseEntryDialog;
class BrowseOptionWidget;
}
namespace Io {
class LoadEntryJob;
class LoadIndexJob;

/**
 * @short Abstract base class for all BlogBrowsingDriver used in Blokkal
 *
 * A BlogBrowsingDriver creates jobs to load entries and the
 * index of of a blog. If a blog does not support a certain operation
 * the create*()-methods return 0.
 * To help the GUI determine which operations are supported, you must
 * reimplement the supportedOperations() method.
 */
class BLOKKAL_EXPORT BlogBrowsingDriver : public QObject {
Q_OBJECT
public:
	/**
	 * Enumeration for the different modes used to browse old entries of this blog
	 */
	enum BrowseMode {
		BrowseEdit = 1, ///< Browse blog in order to edit old entries
		BrowseLink = 2, ///< Browse blog in order to aquire a link to old entries
	};
	
	/**
	 * Constructor
	 * @param parent parent blog
	 */
	BlogBrowsingDriver( Blokkal::Blog * parent );
	virtual ~BlogBrowsingDriver( void );

	/**
	 * @return the blog this driver belongs to
	 */
	Blokkal::Blog * blog( void ) const;

	/**
	 * Enumerations to identify certain blogoperations
	 */
	enum BlogOperations {
		LoadIndex = 0x0001, ///< the index of a blog may be loaded
		LoadEntryById = 0x0002, ///< an entry may be loaded by id
		LoadEntriesByDate = 0x0004, ///< entries may be loaded by date
		LoadRecentEntries = 0x0008, ///< a list of recent entries may be loaded
	};

	/**
	 * If your protocol does only support a limited
	 * number of entries to be loaded by this mode, overridde this method to
	 * return the limit.
	 * If your protocol does not support this type of operation at all you do
	 * not need to reimplement this method.
	 * The default implementation returns 0, which means no limit
	 */
	virtual unsigned int recentLimit( void ) const;

	/**
	 * This method returns an or'ed list of BlogOperations values.
	 * The default implementation returns 0.
	 * @return returns an or'ed list of BlogOperations values
	 */
	virtual unsigned int supportedOperations( void ) const;

	/**
	 * Returns the browse modes supported by this blog.
	 * The default implementation returns BrowseEdit|BrowseLink
	 * @return the browse modes supported by this blog
	 */
	virtual unsigned int supportedBrowseModes( void ) const;

	/**
	 * Returns a BrowseEntryDialog for @a mode.
	 * This method tries to reuse dialog to minimise network traffic.
	 */
	Blokkal::Ui::BrowseEntryDialog * browseBlogDialog( BrowseMode mode );

	/**
	 * Closes and deletes all BrowseEntryDialog objects.
	 * This method is entended to be used on session shutdown to make sure that
	 * all dialogs are deleted before the last window is destructed.
	 */
	void killDialogs( void );

	/**
	 * Creates and returns a job that loads all entries in @a idList.
	 * The default implementation returns 0.
	 * @return a job that loads all entries in @a idList
	 */
	virtual Blokkal::Io::LoadEntryJob * createLoadEntryJob( const QStringList & idList );

	/**
	 * Creates and returns a job that loads all entries posted on @a date.
	 * The default implementation returns 0.
	 * @return a job that loads all entries posted on @a date
	 */
	virtual Blokkal::Io::LoadEntryJob * createLoadEntryJob( const QDate & date );

	/**
	 * Creates and returns a job that loads the last @a n entries in the blog.
	 * The default implementation returns 0.
	 * @see recentLimit()
	 * @return a job that loads the last @a n entries in the blog
	 */
	virtual Blokkal::Io::LoadEntryJob * createLoadEntryJob( unsigned int n );

	/**
	 * Creates and returns a job that loads the index of the blog between
	 * @a from and @a to.
	 * If @a from is invalid, the index starts with the date the blog was created on.
	 * If @a to is invalid, the index ends with the current date.
	 * The default implementation returns 0.
	 * @return job that loads the index of the blog between @a from and @a to
	 */
	virtual Blokkal::Io::LoadIndexJob * createLoadIndexJob( const QDate & from = QDate(), const QDate & to = QDate() );

	/**
	 * If your protocol provides special browse options,
	 * especially ones that are not covered by BlogOperations,
	 * you can return widgets for them in this method.
	 * The default implementation returns an empty list.
	 * @param parent parent widget for the BrowseOptionWidget
	 */
	virtual QList<Blokkal::Ui::BrowseOptionWidget*> customBrowseOptionWidgets( QWidget * parent );
	
private Q_SLOTS:
	void slotBrowseViewDestroyed( QObject * object );
	void slotBrowseViewFinished( void );
	void slotAccountStatusChanged( Blokkal::Account * account );
	
private:
	class Private;
	Private * const d;
};

}

}

#endif
