/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_IOCATEGORYMANAGER_H
#define BLOKKAL_IOCATEGORYMANAGER_H

#include "../blokkal_export.h"

#include <QObject>
#include <QList>

namespace Blokkal {
class Account;
namespace Io {
class Category;

/**
 * A list of Blokkal::Io::Category objects
 */
typedef QList<Blokkal::Io::Category*> CategoryList;

/**
 * @short Manages categories.
 *
 * Manages categories.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT CategoryManager : public QObject
{
Q_OBJECT
public:
	/**
	 * Creates a new CategoryManager for @a account
	 * @param account parent account
	 */
	CategoryManager( Blokkal::Account * account );

	/**
	 * Destructor
	 */
   ~CategoryManager( void );
		
	/**
	 * Removes all categories that not occur in @a categories
	 * @param categories the new category list
	 * @param blogId id of the blog the categories are valid for
	 */
	void setValidCategories( const QString & blogId, QStringList categories );

	/**
	 * Returns the categories for @a blogId. If the id is invalid the
	 * returned list is empty
	 * @param blogId get categories for this blog
	 * @return the categories for @a blogId
	 */
	Blokkal::Io::CategoryList categories( const QString & blogId ) const;

	/**
	 * Returns the category with @a categoryId in the blog with @a blogId
	 * If no category with that id exists a new category object is inserted into the list
	 * and then returned. If additionally no blog with the given id exists, a new list for that
	 * blog is created first.
	 * @param blogId id of the blog the category belongs to
	 * @param categoryId id of the category
	 */
	Blokkal::Io::Category * category( const QString & blogId, const QString & categoryId );

	/**
	 * Returns the account this manager belongs to.
	 * @return the account this manager belongs to
	 */
	Blokkal::Account * account( void ) const;

signals:
	/**
	 * This signal is emitted when a new category for @a blogId has been added.
	 */
	void categoryAdded( const QString & blogId,
	                    Blokkal::Io::Category * category,
	                    Blokkal::Io::CategoryManager * manager = 0 );

	/**
	 * This signal is emitted when a category of @a blogId has been destroyed.
	 */
	void categoryRemoved( const QString & blogId,
	                      Blokkal::Io::Category * category,
	                      Blokkal::Io::CategoryManager * manager = 0 );
	
private:
	class Private;
	Private * d;

	friend class Category;
};

/**
 * @short Representation of a category
 *
 * This class encapsulates the properties of one category.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT Category : public QObject {
Q_OBJECT
public:
	/**
	 * Returns the id of this category.
	 * @return the id of this category
	 */
	QString id( void ) const;

	/**
	 * Returns the visible name of this category.
	 * @return the visible name of this category
	 */
	QString name( void ) const;

	/**
	 * Sets the visible name of this category.
	 * @param name the new visible name of this category
	 */
	void setName( const QString & name );

	/**
	 * Returns the html url of this category.
	 * @return the html url of this category
	 */
	QString htmlUrl( void ) const;
	/**
	 * Sets the html url of this category
	 * @param htmlUrl the url of this category
	 */
	void setHtmlUrl( const QString & htmlUrl );
	
	/**
	 * Returns the rss url of this category
	 * @return the rss url of this category
	 */
	QString rssUrl( void ) const;

	/**
	 * Sets the rss url of this category
	 * @param rssUrl the rss url of this category
	 */
	void setRssUrl( const QString & rssUrl );

signals:
	/**
	 * This signal is emitted when a property of @a category has been altered
	 */
	void updated( Blokkal::Io::Category * category );

private:
	Category( Blokkal::Io::CategoryManager * manager, const QString & blogId, const QString & id, const QString & name = 0, const QString & htmlUrl = 0, const QString &rssUrl = 0 );
	~Category( void );
	void destroy( void );
	
	class Private;
	Private * d;

	friend class CategoryManager;
};

}

}

#endif
