/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALL_IOJOBS_H
#define BLOKKALL_IOJOBS_H

#include "../blokkal_export.h"
#include <kcompositejob.h>

#include <QtCore/QDate>

namespace Blokkal {
class Blog;
class Entry;
namespace Io {
/**
 * Enum for error that are not "hard" IO errors
 */
enum Error {
	UnknownError = -1, ///< Unknown error
	AuthenticationError = -2, ///< Authentication failed
	MalformedReplyError = -3, ///< The reply received form the server was malformed
	UserDefinedError = -100, ///< First error code for custom error codes
};

/**
 * @short Base class for all jobs used in Blokkal
 *
 * Base class for all jobs used in Blokkal
 */
class BLOKKAL_EXPORT Job : public KCompositeJob {
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param parent parent object
	 */
	Job( QObject * parent = 0 );
	virtual ~Job( void );

	/**
	 * Starts all subjobs
	 */
	virtual void start( void );

	virtual bool addSubjob( KJob * job );

protected Q_SLOTS:
	/**
	 * This slot is called when a subjob finishes.
	 */
	virtual void slotResult( KJob * job );

	/**
	 * Emits the total progress of all remaining jobs.
	 */
	virtual void slotEmitPercent( void );

protected:
	/**
	 * This method is called when all subjobs have finished
	 * and the result() signal is about to be emitted.
	 * Override this method in subclasses if you need to do special things after
	 * the job has finished.
	 * The default implementation does nothing.
	 */
	virtual void jobFinished( void );

	/**
	 * This method is called when a subjob has finished.
	 * The default implementation does nothing.
	 */
	virtual void subjobFinished( KJob * job );

	/**
	 * Calls kill() on all subjobs.
	 * @return TRUE if kill() of all subjobs returned TRUE, FALSE otherwise
	 */
	virtual bool doKill( void );
};

/**
 * @short Base class for jobs that operate on entries
 *
 * Base class for jobs that operate on entries
 */
class BLOKKAL_EXPORT EntryJob : public Blokkal::Io::Job {
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param entry parent entry
	 */
	EntryJob( Blokkal::Entry * entry );
	virtual ~EntryJob( void );
	
	/**
	 * @return the entry this job belongs to
	 */
	Blokkal::Entry * entry( void ) const;
	
private:
	class Private;
	Private * const d;
	
};

/**
 * @short Base class for jobs that operate on blogs
 *
 * Base class for jobs that operate on blogs
 */
class BLOKKAL_EXPORT BlogJob : public Blokkal::Io::Job {
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param blog parent blog
	 */
	BlogJob( Blokkal::Blog * blog );
	virtual ~BlogJob( void );

	/**
	 * @return the blog this job belongs to
	 */
	Blokkal::Blog * blog( void ) const;

private:
	class Private;
	Private * const d;	
};

/**
 * @short Abstract base class for jobs that load entries
 *
 * Abstract base class for jobs that load entries
 */
class BLOKKAL_EXPORT LoadEntryJob : public Blokkal::Io::BlogJob {
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param blog parent blog
	 */
	LoadEntryJob( Blokkal::Blog * blog );
	virtual ~LoadEntryJob( void );

	/**
	 * @return the loaded entries
	 */
	virtual QList<Blokkal::Entry *> entries( void ) const = 0;
};

/**
 * @short Abstract base class for jobs that load a blog index
 *
 * Abstract base class for jobs that load a blog index
 */
class BLOKKAL_EXPORT LoadIndexJob : public Blokkal::Io::BlogJob {
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param blog parent blog
	 */
	LoadIndexJob( Blokkal::Blog * blog );
	virtual ~LoadIndexJob( void );

	/**
	 * @return the loaded index
	 */
	virtual QMap<QDate, QStringList > index( void ) const = 0;
};


}
}

#endif
