/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "xmlrpcjob.h"
#include "xmlrpcjob.moc"

#include "util.h"

#include <kdebug.h>
#include <kcodecs.h>
#include <klocale.h>
#include <kurl.h>
#include <kio/job.h>

#include <QDomElement>
#include <QVariant>
#include <QDateTime>

class Blokkal::Io::XmlRpcJob::Private {
public:
	Private( void ) :
	responseBuffer( QByteArray() ),
	response( QVariant() ),
	responseType( ResponseError ){}

	QByteArray responseBuffer;
	QVariant response;
	ResponseType responseType;
};

Blokkal::Io::XmlRpcJob::XmlRpcJob( const KUrl & url,
                                   const QString & methodName,
                                   const QList< QVariant > & params,
                                   unsigned int workArounds,
                                   bool showProgressInfo ) :
Job(),
d( new Private() )
{
	QByteArray postData;

	//generate xml-rpc call data
	QDomDocument callDocument;
	callDocument.appendChild( callDocument.createProcessingInstruction( "xml", "version=\"1.0\"" ) );

	QDomElement methodElement = callDocument.createElement( "methodCall" );
	callDocument.appendChild( methodElement );

	QDomElement currentElement = callDocument.createElement( "methodName" );
	currentElement.appendChild( callDocument.createTextNode( methodName ) );
	methodElement.appendChild( currentElement );

	QDomElement paramsElement = callDocument.createElement( "params" );
	methodElement.appendChild( paramsElement );
	for( QList< QVariant >::ConstIterator it = params.begin();
	     it != params.end();
	     ++it )
	{
		QDomElement paramElement = callDocument.createElement( "param" );
		paramsElement.appendChild( paramElement );

		encodeData( paramElement, *it, workArounds );
	}

	QTextStream stream( &postData, QIODevice::WriteOnly );
	callDocument.save( stream, 3 );

	//kDebug() << postData << endl;

	KIO::TransferJob * job = KIO::http_post( KUrl( url ),
	                                         postData,
	                                         showProgressInfo ? KIO::DefaultFlags : KIO::HideProgressInfo );

	job->addMetaData( "UserAgent", "Blokkal/IO XML-RPC Job" );
	job->addMetaData( "content-type", "Content-Type: text/xml; charset=utf-8" );
	job->addMetaData( "ConnectTimeout", "300" );

	connect( job,
	         SIGNAL( data( KIO::Job *, const QByteArray & ) ),
	         SLOT( slotDataArrived( KIO::Job *, const QByteArray & ) ) );
	addSubjob( job );
}

Blokkal::Io::XmlRpcJob::~XmlRpcJob( void )
{
	delete d;
}

void Blokkal::Io::XmlRpcJob::slotDataArrived( KIO::Job * job, const QByteArray & data )
{
	if( subjobs().isEmpty() || job != subjobs().first() ) {
		return;
	}
	
	d->responseBuffer.append( data );
}

void Blokkal::Io::XmlRpcJob::jobFinished( void )
{
	//kDebug() << d->responseBuffer<< endl;
	if( error() ) {
		d->responseType = ResponseError;
		return;
	}
	
	QString errorText;
	QDomDocument responseDocument;
	if( responseDocument.setContent( d->responseBuffer, &errorText ) ) {
		QDomNode rootNode = responseDocument.firstChild();
		for( ; !rootNode.isNull(); rootNode = rootNode.nextSibling() ) {
			if( rootNode.isElement() ) {
				break;
			}
		}
		
		if( rootNode.firstChild().nodeName() == "params" ) {
			//handle response
			QDomNode valueNode = rootNode.firstChild().firstChild().firstChild();
			d->response = decodeData( valueNode.toElement() );
			d->responseType = ResponseSuccess;
		}
		else if( rootNode.firstChild().nodeName() == "fault" ) {
			//handle fault
			setError( -1 );
			d->response = decodeData( rootNode.firstChild().firstChild().toElement() );
			if( d->response.type() == QVariant::Map ) {
				setErrorText( ( ( d->response.toMap() )["faultString"] ).toString() );
			}
			d->responseType = ResponseFault;
		}
		else {
			setError( -1 );
			setErrorText( i18n( "Could not parse response." ) );
			d->responseType = ResponseError;
		}
	}
	else {
		setError( -1 );
		setErrorText( errorText );
		d->responseType = ResponseError;
	}
	
}

const QVariant & Blokkal::Io::XmlRpcJob::response( void ) const
{
	return d->response;
}

Blokkal::Io::XmlRpcJob::ResponseType Blokkal::Io::XmlRpcJob::responseType( void ) const
{
	return d->responseType;
}

void Blokkal::Io::XmlRpcJob::encodeData( QDomElement & containerElement, QVariant data, unsigned int workArounds )
{
	QDomDocument callDocument = containerElement.ownerDocument();
	QDomElement valueElement = callDocument.createElement( "value" );
	containerElement.appendChild( valueElement );
	QDomElement currentElement;
	switch ( data.type() )  {
	case QVariant::Int:
		currentElement = callDocument.createElement( "int" );
		valueElement.appendChild( currentElement );
		currentElement.appendChild( callDocument.createTextNode( data.toString() ) );
		break;
	case QVariant::Bool:
		currentElement = callDocument.createElement( "boolean" );
		valueElement.appendChild( currentElement );
		currentElement.appendChild( callDocument.createTextNode( data.toBool() ? "1" : "0" ) );
		break;
	case QVariant::String:
		currentElement = callDocument.createElement( "string" );
		valueElement.appendChild( currentElement );
		currentElement.appendChild( callDocument.createTextNode( data.toString() ) );
		break;
	case QVariant::Double:
		currentElement = callDocument.createElement( "double" );
		valueElement.appendChild( currentElement );
		currentElement.appendChild( callDocument.createTextNode( data.toString() ) );
		break;
	case QVariant::DateTime:
		currentElement = callDocument.createElement( "dateTime.iso8601" );
		valueElement.appendChild( currentElement );
		if( workArounds & WordPressDateWorkAround  ) {
			currentElement.appendChild( callDocument.createTextNode( data.toDateTime().toString( Qt::ISODate ).remove( '-' ) ) );
		}
		else {
			currentElement.appendChild( callDocument.createTextNode( data.toDateTime().toString( Qt::ISODate ) ) );
		}
		break;
	case QVariant::ByteArray:
		currentElement = callDocument.createElement( "base64" );
		valueElement.appendChild( currentElement );
		currentElement.appendChild( callDocument.createTextNode( KCodecs::base64Encode( data.toByteArray() ) ) );
		break;
	case QVariant::Map:
		{
			currentElement = callDocument.createElement( "struct" );
			valueElement.appendChild( currentElement );
			const QMap<QString, QVariant> mapData = data.toMap();
			for( QMap<QString, QVariant>::ConstIterator it = mapData.begin();
					it != mapData.end();
					++it )
			{
				QDomElement memberElement = callDocument.createElement( "member" );
				currentElement.appendChild( memberElement );
				QDomElement nameElement = callDocument.createElement( "name" );
				memberElement.appendChild( nameElement );
				nameElement.appendChild( callDocument.createTextNode( it.key() ) );
				encodeData( memberElement, it.value(), workArounds );
			}
		}
		break;
	case QVariant::List:
	case QVariant::StringList:
		{
			currentElement = callDocument.createElement( "array" );
			valueElement.appendChild( currentElement );
			QDomElement dataElement = callDocument.createElement( "data" );
			currentElement.appendChild( dataElement );
			const QList<QVariant> arrayData = data.toList();
			for( QList<QVariant>::ConstIterator it = arrayData.begin();
					it != arrayData.end();
					++it )
			{
				encodeData( dataElement, *it, workArounds );
			}
		}
		break;
	default:
		kError() << "encountered invalid parameter type: " << data.type() << endl;
		break;
	}
}

QVariant Blokkal::Io::XmlRpcJob::decodeData( const QDomElement & valueElement )
{
	Q_ASSERT( valueElement.nodeName() == "value" );

	QDomElement typeElement = valueElement.firstChild().toElement();
	QString type = typeElement.nodeName();
	if( type == "int" || type == "i4" ) {
		return QVariant( typeElement.text().toInt() );
	}

	if ( type == "boolean" ) {
		if ( typeElement.text().toLower() == "true" || typeElement.text() == "1" ) {
			return QVariant( TRUE );
		}
		else {
			return QVariant( FALSE );
		}
	}

	if( type == "double" ) {
		return QVariant( typeElement.text().toDouble() );
	}

	if ( type == "dateTime.iso8601" ) {
		if( typeElement.text().contains( '-' ) ) {
			return QVariant( QDateTime::fromString( typeElement.text(), Qt::ISODate ) );
		}
		else {
			return QVariant( QDateTime::fromString( typeElement.text().insert( 6, '-' ).insert( 4, '-' ), Qt::ISODate ) );
		}
	}

	if ( type == "base64" ) {
		return QVariant( KCodecs::base64Decode( typeElement.text().toLatin1() ) );
	}

	if( type == "struct" ) {
		QMap<QString, QVariant> map;
		QDomNode memberNode = typeElement.firstChild();
		while ( !memberNode.isNull() ) {
			const QString name = memberNode.namedItem( "name" ).toElement().text();
			const QVariant value = decodeData( memberNode.namedItem( "value" ).toElement() );
			map[ name ] = value;
			memberNode = memberNode.nextSibling();
		}
		return QVariant( map );
	}

	if ( type == "array" ) {
		QList<QVariant> list;
		QDomNode valueNode = typeElement.firstChild().firstChild();
		while ( !valueNode.isNull() ) {
			list.append( decodeData( valueNode.toElement() ));
			valueNode = valueNode.nextSibling();
		}
		return QVariant( list );
	}

	//no recognised type specified
	//maybe none specified and first child is a text node
	//type string is default
	if( type == "string"  ||
			!valueElement.firstChild().toText().isNull() )
	{
		return QVariant( Util::decodeUtf8Data( valueElement.text().toUtf8() ) );
	}

	//a type name was specified but we could not recognise it
	kError() << "encountered invalid type: " << type << endl;
	kError() << "returning invalid QVariant" << endl;
	return QVariant();
}
