/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_IOXMLRPCJOB_H
#define BLOKKAL_IOXMLRPCJOB_H

#include "jobs.h"

#include "../blokkal_export.h"

class QDomElement;

class KUrl;
namespace KIO {
class Job;
}

namespace Blokkal {

namespace Io {

/**
 * @short This class implements an XML-RPC call.
 *
 * Basically, this is a wrapper around a KIO::TransferJob (HTTP-POST),
 * that does all the dirty work for you.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT XmlRpcJob : public Blokkal::Io::Job
{
Q_OBJECT
public:
	/**
	 * Creates a new XmlRpcJob.
	 * The job will be started immediately.
	 * The job will destroy itself after it has finished.
	 * If you want to abort the job, delete this object.
	 * @param url the url to connect to
	 * @param methodName the name of the method to call
	 * @param params parameters for the method call
	 * @param workArounds enabled workarounds
	 * @param showProgressInfo true to show progress information
	 */
	XmlRpcJob( const KUrl & url,
	           const QString & methodName,
	           const QList<QVariant> & params,
	           unsigned int workArounds = 0,
	           bool showProgressInfo = FALSE );

	/**
	 * Destructor
	 */
	virtual ~XmlRpcJob( void );

	/**
	 * Enumeration for workarounds for some buggy server
	 * side implementations
	 */
	enum WorkArounds {
		WordPressDateWorkAround = 0x01, ///< Encode date in WordPress variant of ISO 8601
	};

	/**
	 * Returns the content of the XMl-RPX response.
	 * Check responseType() first.
	 * @return the content of the XMl-RPX response
	 */
	const QVariant & response( void ) const;

	/**
	 * Enumerations for possible response types
	 */
	enum ResponseType {
		ResponseSuccess = 1, ///< A valid XML-RPX response indicating a success was received
		ResponseFault = 2, ///< A valid XML-RPX response indicating an error was received
		ResponseError = 3, ///< No valid XML-RPX response was received, either due to a malformed response, an IO error or similar
	};

	/**
	 * Returns the type of the response
	 * @return the type of the response
	 */
	Blokkal::Io::XmlRpcJob::ResponseType responseType( void ) const;

	/**
	 * Encodes @a data into @a containerElement.
	 * Example: If @a containerElement is a "<param>" element and
	 * @a data is a QString, then the result will be
	 * \code
	 * <param>
	 *   <value>SomeString</value>
	 * </param>
	 * \endcode
	 * This method works recursively, so it will encode structures
	 * like QMap or QList automatically.
	 * @param containerElement put the date in new child nodes of this element
	 * @param data the data to encode
	 * @param workArounds or'ed list of WorkArounds
	 */
	static void encodeData( QDomElement & containerElement,
	                        QVariant data,
	                        unsigned int workArounds = 0 );

	/**
	 * Returns a QVariant representation of the Xml-Rpc
	 * data in @a valueElement
	 * @param valueElement read the date from this element
	 * @return the QVariant representation
	 */
	static QVariant decodeData( const QDomElement & valueElement );

protected Q_SLOTS:
	/**
	 * Stores the returned data in the response buffer.
	 * You should not need to override this method.
	 * @param job the worker job
	 * @param data the new data
	*/
	virtual void slotDataArrived( KIO::Job *job, const QByteArray & data );

protected:
	/**
	 * This method is called when all subjobs have finished
	 * and the result() signal is about to be emitted.
	 * Override this method in subclasses if you need to do special things after
	 * the job has finished.
	 * The default implementation does nothing.
	 */
	virtual void jobFinished( void );
	
private:
	class Private;
	Private * const d;
};

}

}

#endif
