/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALBLOKKALPLUGIN_H
#define BLOKKALBLOKKALPLUGIN_H

#include "blokkal_export.h"

#include <kxmlguiclient.h>
#include <QtCore/QObject>

class KPluginInfo;
class KIcon;
namespace Blokkal {
/**
 * @brief Base class for blokkal plugins.
 * 
 * All plugins must implement this class.
 *
 * For a general tutorial about plugins and applications using plugins
 * see
 * http://developer.kde.org/documentation/tutorials/developing-a-plugin-structure/index.html
 * for a nice tutorial.
 * 
 * You must pass the instance of your plugin to the constructor
 * of this class. The instance is used to load the ui.rc file
 * for your plugin. Simply call the constructor from your implementation
 * like this:
 * \code
 * Plugin( KGenericFactory<MyPlugin>::instance(), parent, name )
 * \endcode
 *
 * You will also need to create a desktop file to make your plugin
 * available to Blokkal:
 * \code
 * [Desktop Entry]
 * Encoding=UTF-8
 * Type=Service
 * Icon=blokkal
 * ServiceTypes=Blokkal/Plugin
 * X-KDE-Library=blokkal_myplugin
 * X-KDE-PluginInfo-Author=Your Name
 * X-KDE-PluginInfo-Email=someone@somewhere.org
 * X-KDE-PluginInfo-Name=blokkal_myplugin
 * X-KDE-PluginInfo-Version=0.0.1
 * X-KDE-PluginInfo-Website=
 * X-KDE-PluginInfo-Category=Plugins
 * X-KDE-PluginInfo-Depends=
 * X-KDE-PluginInfo-License=GPL
 * X-KDE-PluginInfo-EnabledByDefault=false
 * Name=MyPlugin
 * Comment=A short description of what your plugin actually does.
 * \endcode
 * If you want to create a Blokkal::Protocol or Blokkal::UI::EditEntryViewPlugin
 * the value of ServiceTypes must be Blokkal/Protocol or Blokkal/EditEntryViewPlugin
 * respectively.
 *
 * @note You will not get the application name set in the about data of your plugin,
 * even if you specify the Name-key in the desktop file. If you provide any GUI
 * elements in your plugin (which is almost certainly the case for protocols and
 * view plugins ) you can load your plugin like this:
 * \code
 * static const KAboutData aboutData("blokkal_myplugin", I18N_NOOP("MyPlugin") , "0.0.1" );
 * K_EXPORT_COMPONENT_FACTORY( blokkal_myplugin, KGenericFactory<MyPluginClass>( &aboutData ) )
 * \endcode
 *
 * In conclusion, do the following steps to create a plugin:
 *
 * Create the desktop file.
 *
 * Implement the plugin class:<br>
 * Header:
 * \code
 * #include <blokkalplugin.h>
 * class MyPlugin : public Blokkal::Plugin {
 * Q_OBJECT
 * public:
 *   MyPlugin( QObject * parent, const QVariantList & args );
 *   virtual ~MyPlugin( void );
 * };
 * \endcode
 *
 * Source:
 * \code
 * #include "myplugin.h"
 * #include "myplugin.moc"
 *
 * #include <kpluginfactory.h>
 * 
 * static const KAboutData aboutData("blokkal_myplugin", I18N_NOOP("MyPlugin") , "0.0.1" );
 * K_EXPORT_COMPONENT_FACTORY( blokkal_myplugin, KGenericFactory<MyPlugin>( &aboutData ) )
 * K_PLUGIN_FACTORY( MyPluginFactory, registerPlugin<MyPlugin>(); )
 * K_EXPORT_PLUGIN( MyPluginFactory("blokkal_myplugin") )
 * 
 *
 * MyPlugin::MyPlugin( QObject * parent, const QStringList & args ) :
 * Blokkal::Protocol( MyPluginFactory::componentData(), parent )
 * {}
 *
 * MyPlugin::~MyPlugin( void )
 * {}
 * \endcode
 * To build the module you must define the build target this way:
 * \code
 * kde4_add_plugin(blokkal_myplugin ${blokkal_myplugin_PART_SRCS})
 * target_link_libraries(blokkal_myplugin  ${KDE4_KDECORE_LIBS} blokkal )
 * install(TARGETS blokkal_myplugin DESTINATION ${PLUGIN_INSTALL_DIR} )
 * \endcode
 *
 * To install everything in the right place your Makefile.am should
 * also contain these lines:
 * \code
 * install( FILES  blokkal_mypluginui.rc DESTINATION ${DATA_INSTALL_DIR}/blokkal_myplugin )
 * install( FILES  blokkal_myplugin.desktop DESTINATION ${SERVICES_INSTALL_DIR} )
 * \endcode
 *
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT Plugin : public QObject, public KXMLGUIClient
{
Q_OBJECT
public:
	/**
	 * Creates a new Plugin object.
	 * @param parent owning object
	 * @param instance the instance of this plugin
	 */
	Plugin( const  KComponentData & instance, QObject * parent = 0 );

	/**
	 * Destructor
	 */
   virtual ~Plugin( void );

	/**
	 * Returns the internal plugin name.
	 * @return the plugin name
	 */
	QString pluginName( void ) const;

	/**
	 * Returns the plugin info for this plugin.
	 * @return the plugin info
	 */
	KPluginInfo pluginInfo( void ) const;

	/**
	 * Returns the icon for this plugin.
	 * @return the icon for this plugin
	 */
	KIcon icon( void ) const;

	/**
	 * Returns the user visible name of the plugin.
	 * @return the visible name
	 */
	QString visibleName( void ) const;

public Q_SLOTS:
	/**
	 * This slot is called prior to deleting the plugin object.
	 * If you need to do some cleanup when shutting down, do it here.
	 * The default implementation just emits readyToUnload().
	 */
	virtual void prepareToUnload( void );

Q_SIGNALS:
	/**
	 * This signal is emitted after a call prepareToUnload().
	 * The plugin object may be safely deleted after this signal.
	 * @param plugin the plugin that has just shut down
	 */
	void readyToUnload( Blokkal::Plugin * plugin );
};

}

#endif
