/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKPLUGINMANAGER_H
#define BLOKPLUGINMANAGER_H

#include "blokkal_export.h"

#include <kplugininfo.h>

#include <QtCore/QObject>

namespace Blokkal {
class Plugin;
class Protocol;
namespace Ui {
class EditEntryViewFactory;
class BrowseEntryViewFactory;
}
class PluginManagerPrivate;
/**
 * @brief Global plugin manager class.
 * 
 * Plugins can be loaded and accessed through this class.
 * The plugin manager will ref() kapp, so you must call
 * shutdown() to unload all plugins in order to quit.
 * 
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT PluginManager : public QObject
{
friend class PluginManagerPrivate;
Q_OBJECT
public:
	/**
	 * This method returns a pointer to the global plugin manager object.
	 * If no plugin manager exists yet, one will be created.
	 *
	 * @return pointer to the global plugin manager
	 */
	static PluginManager * self( void );

	/**
	 * This method returns a list of the KPluginInfo objects for all available Protocols plugins.
	 * @return list of available protocols
	 */
	KPluginInfo::List availableProtocols( void ) const;

	/**
	 * This method returns a list of the KPluginInfo objects for all available EditEntryViewFactory plugins.
	 * @return list of available edit entry view plugins
	 */
	KPluginInfo::List availableEditEntryViewFactorys( void ) const;

		/**
	 * This method returns a list of the KPluginInfo objects for all available BrowseEntryViewFactory plugins.
	 * @return list of available edit entry view plugins
	 */
	KPluginInfo::List availableBrowseEntryViewFactorys( void ) const;

	/**
	 * This method returns a list of the KPluginInfo objects for all available plugins.
	 * @return list of available protocols
	 */
	KPluginInfo::List availablePlugins( void ) const;
	
	/**
	 * Returns a pointer to the protocol plugin with name @a pluginName.
	 * If the plugin is not loaded yet, it will be loaded. If loading the plugin
	 * fails 0 is returned.
	 * @param pluginName name of the desired protocol
	 * @return pointer to the protocol plugin
	 */
	Blokkal::Protocol * protocol( const QString & pluginName );

	/**
	 * Returns a pointer to the Blokkal::Ui::EditEntryViewFactory with name <i>pluginName</i>.
	 * If the plugin is not loaded yet, it will be loaded now.
	 * If loading the plugin fails 0 is returned.
	 * If @a pluginName is empty, the default view plugin will be loaded.
	 * @param pluginName name of the desired plugin
	 * @return pointer to the plugin
	 */
	Blokkal::Ui::EditEntryViewFactory * editEntryViewFactory( const QString & pluginName = QString::null );

	/**
	 * Returns a pointer to the Blokkal::Ui::EditEntryViewFactory with name @a pluginName.
	 * If the plugin is not loaded yet, it will be loaded now.
	 * If loading the plugin fails 0 is returned.
	 * If @a pluginName is empty, the default view plugin will be loaded.
	 * @param pluginName name of the desired plugin
	 * @return pointer to the plugin
	 */
	Blokkal::Ui::BrowseEntryViewFactory * browseEntryViewFactory( const QString & pluginName = QString::null );

	/**
	 * Returns the requested plugin. If the plugin is not
	 * loaded yet, it will be loaded. If loading the plugin
	 * fails 0 is returned.
	 * @param pluginName name of the plugin
	 * @return the plugin
	 */
	Blokkal::Plugin * plugin( const QString & pluginName );

	/**
	 * Returns the plugin info for a plugin with name @a pluginName.
	 * @param pluginName name of the plugin
	 * @return the plugin info for the plugin
	 */
	KPluginInfo pluginInfo( const QString & pluginName ) const;

	/**
	 * Returns the plugin info for a plugin.
	 * @param plugin the plugin
	 * @return the plugin info for the plugin
	 */
	KPluginInfo pluginInfo( const Plugin * plugin ) const;

public Q_SLOTS:
	/**
	 * Unloads the plugin with @a pluginName. If no plugin
	 * with this name is loaded, nothing happens.
	 * @param pluginName the internal name of the plugin to unload
	 */
	void unloadPlugin( const QString & pluginName );

	/**
	 * Unloads all plugins
	 */
	void unloadAllPlugins( void );

	/**
	 * This method is similar to unloadAllPlugins() but it will also
	 * deref() from kapp, thus alowing the application to quit.
	 */
	void shutdown( void );

Q_SIGNALS:
	/**
	 * This signal is emitted when a plugin has been loaded
	 */
	void pluginLoaded( Blokkal::Plugin * plugin );
	
protected Q_SLOTS:
	/**
	 * This slot is called when a plugin signals that it is ready to be unloaded.
	 * @param plugin the plugin ready to unload
	 */
	void deletePlugin( Blokkal::Plugin * plugin );

	/**
	 * This slot is called when a plugin has been deleted.
	 * @param plugin the destroyed plugin
	 */
	void removePlugin( QObject * plugin );

	/**
	 * Returns the plugin with the requested name.
	 * If it is not yet loaded it is instantiated according
	 * to the given type.
	 * 
	 * @param pluginName name of the plugin
	 */
	Blokkal::Plugin * loadPlugin( const QString & pluginName );
	
private:
	PluginManager( void );
	~PluginManager( void );
};

}

#endif
