/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALBLOKKALPOSTENTRYQUEUE_H
#define BLOKKALBLOKKALPOSTENTRYQUEUE_H

#include <QtCore/QObject>
#include "blokkal_export.h"

template <class T> class QLinkedList;

class KJob;

namespace Blokkal {
class Entry;
class Account;
class PostEntryStatus;
namespace Io {
class EntryJob;
}

class PostEntryQueuePrivate;
/**
 * @short Handles posting of entries.
 *
 * This class manages entries that need to be posted into their blogs.
 * It will post the entries when their accounts go online.
 * @note This class does not guarantee that the entries will be posted in the correct order.
 * Entries that belong to different blogs will be posted independently from each other.
 * Entries that belong to the same blog will be posted in the same order they are queued
 * as long as no errors occur.
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT PostEntryQueue : public QObject
{
friend class PostEntryQueuePrivate;	
Q_OBJECT
public:
	/**
	 * Returns a pointer to the global PostEntryQueue.
	 * If none exists yet, one will be created.
	 * @return the global PostEntryQueue object
	 */
	static Blokkal::PostEntryQueue * self( void );

	/**
	 * Returns the number of currently queued entries.
	 * @return the number of currently queued entries
	 */
	int count( void ) const;

	/**
	 * Returns true if no entries are queed.
	 * @return true if no entries are queed
	 */
	bool isEmpty( void ) const;

	/**
	 * Returns a list of currently queued entries.
	 * @return a list of currently queued entries
	 */
	QLinkedList<Blokkal::Entry *> entries( void ) const;

	/**
	 * Return a pointer to the PostEntryStatus object for the entry.
	 * @param entry get the status for this entry
	 * @return the status for the entry, or 0
	 */
	const Blokkal::PostEntryStatus * entryStatus( Blokkal::Entry * entry ) const;

	/**
	 * Test whether the queue is currently processing entries.
	 * This does not mean, that any entries are beeing posted, just that
	 * the queue will do so when all other conditions are met.
	 * @return TRUE when the queue is currently processing entries
	 */
	bool isProcessing( void ) const;

public Q_SLOTS:
	/**
	 * Call this slot before shutting down the plugin manager.
	 */
	void stopSyncing( void );

	/**
	 * Restores the last saved state.
	 */
	void restoreState( void );

	/**
	 * Queues an entry for posting.
	 * This object will call ref() on @a entry.
	 * The entry will be deref()ed when it has been succesfully posted.
	 * @param entry entry to queue
	 * @see unqueue()
	 */
	void queue( Blokkal::Entry * entry );

	/**
	 * Removes an entry from the queue.
	 * If the entry is currently beeing posted to the server
	 * it cannot be unqued and this method will return FALSE
	 * unless @a force is set to true. In this case the post operation
	 * is aborted.
	 * If the entry was not queued in the first place this method
	 * will return FALSE.
	 * @note You must call deref() on the returned entry when you no longer need it.
	 * @param entry entry to remove
	 * @param force remove entry, even when it is currently beeing posted
	 * @return TRUE if the entry was successfully unqueued
	 */
	bool unqueue( Blokkal::Entry * entry, bool force = FALSE );

	/**
	 * Resets the status if the entry is marked as failing.
	 * Another attempt to post the entry will be made.
	 * If the entry is unknown or not failing nothing happens.
	 * @param entry reset the status of this entry.
	 */
	void clearError( Blokkal::Entry * entry );

	/**
	 * Suspends queue processing. Ongoing post operations are not stopped.
	 */
	void suspendProcessing( void );

	/**
	 * Resumes queue processing.
	 */
	void resumeProcessing( void );
	
Q_SIGNALS:
	/**
	 * This signal is emitted after an entry has been added
	 * to the queue and before any attempt is made to post
	 * the entry.
	 */
	void entryAdded( Blokkal::Entry * addedEntry );

	/**
	 * This signal is emitted after an entry has been removed
	 * from the queue.All references to PostEntryStatus objects
	 * for this entry are invalid now.
	 * @see entryPosted()
	 */
	void entryRemoved( Blokkal::Entry * removedEntry );

	/**
	 * This signal is emitted after an entry has been removed
	 * from the queue after it has succesfully been posted to the server.
	 * The entry will be destroyed after this signal has been emitted.
	 * @see entryRemoved()
	 */
	void entryPosted( Blokkal::Entry * postedEntry );

	/**
	 * This signal is emitted when an entry is failing to post.
	 * @param failingEntry the entry failing to post
	 */
	void entryFailing( Blokkal::Entry * failingEntry );

	/**
	 * This signal is emitted when the status of an entry changes.
	 * @param entry the changed entry
	 * @param status the new status
	 */
	void entryStatusChanged( Blokkal::Entry * entry, const Blokkal::PostEntryStatus * status );

private Q_SLOTS:
	/**
	 * Searches for entries that may be posted and starts
	 * the post operation.
	 */
	void processQueue( void );

	/**
	 * Called when a job finished.
	 */
	void slotJobFinished( KJob * job );

	/**
	 * Called when an account changes connection status
	 */
	void slotConnectionsChanged( Blokkal::Account * account );

	/**
	 * Called when a new account is created
	 */
	void slotAccountAdded( Blokkal::Account * account );

	/**
	 * Called when an entry is destroyed.
	 */
	void slotEntryDestroyed( Blokkal::Entry * entry );

private:
	PostEntryQueue( void );
	~PostEntryQueue( void );
};

/**
 * @short This class describes the status of an entry manager by Blokkal::PostEntryQueue.
 *
 * This class describes the status of an entry manager by Blokkal::PostEntryQueue.
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT PostEntryStatus
{
public:
	/**
	 * Creates a new PostEntryStatus
	 */
	PostEntryStatus( void );

	/**
	 * Destructor
	 */
	~PostEntryStatus( void );

	/**
	 * This enum describes the status of the entry
	 */
	enum Status {
		StatusNormal = 1, ///< For normal queued entries
		StatusPosting = 2, ///< This entry is beeing posted right now
		StatusError = 3, ///< Some error occured
	};

	/**
	 * Returns the error code of the last post attempt.
	 * @return the error code of the last post attempt
	 */
	int error( void ) const;

	/**
	 * Returns the error string of the last post attempt.
	 * @return the error string of the last post attempt
	 */
	QString errorString( void ) const;

	/**
	 * This method returns the status of the entry.
	 * @return the status of the entry
	 */
	Blokkal::PostEntryStatus::Status status( void ) const;

	/**
	 * If the entry is currently beeing posted, this method
	 * returns a pointer to the handler conducting the operation.
	 * @return the job
	 */
	const Blokkal::Io::EntryJob * job( void ) const;
	
private:
	class Private;
	Private * const d;

	friend class PostEntryQueue;
};

}

#endif
