/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Martin Mller                            *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKPROTOCOL_H
#define BLOKPROTOCOL_H

#include "blokkalplugin.h"

namespace Blokkal {
class Account;

namespace Ui {
class EditAccountWidget;
class FormattingBackend;
class EditEntryWidget;
}

/**
 * @brief Base class for protocol plugins.
 * 
 * Protocol plugins must implement this class.
 * Protocols are basically factory classes for Account objects
 * which in turn are factory classes for Blog objects etc.
 * Thus, if you delete a protocol all its accounts, blogs and entries
 * will be destroyed.
 * Do not delete a protocol plugin without calling prepareToUnload().
 * @author Martin Mller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT Protocol : public Plugin
{
Q_OBJECT
public:
	/**
	 * Creates a new Protocol object
	 * @param parent owning object
	 * @param instance instance of the plugin
	 * @see Blokkal::Plugin
	 */
	Protocol( const KComponentData & instance, QObject * parent = 0 );

	/**
	 * Destructor
	 */
	virtual ~Protocol( void );

	/**
	 * This method returns a pointer to an @ref Blokkal::Ui::EditAccountWidget
	 * used for editing the account @a account.
	 * @param account the account this widget should edit. If 0, this widget is used to create a new account.
	 * @param parent The designated parent widget.
	 * @return a widget for editing the properties of an account.
	 */
	virtual Blokkal::Ui::EditAccountWidget * createEditAccountWidget( Blokkal::Account * account = 0, QWidget * parent = 0 ) = 0;

	/**
	 * Creates an @ref Blokkal::Account object.
	 * @param accountName the name of the new account
	 * @return a new account object
	 */
	virtual Blokkal::Account * createAccount( const QString & accountName = 0 ) = 0;

	/**
	 * Creates a custom Blokkal::Ui::FormattingBackend.
	 * The default implementation just returns an instance of Blokkal::Ui::FormattingBackend.
	 * @param widget parent EditEntryWidget.
	 * @return a custom Blokkal::Ui::FormattingBackend
	 */
	virtual Blokkal::Ui::FormattingBackend * createFormattingBackend( Blokkal::Ui::EditEntryWidget * widget );
	
public Q_SLOTS:
	/**
	 * Disconnects all accounts of this protocol type.
	 */
	virtual void prepareToUnload( void );

protected Q_SLOTS:
	/**
	 * This slot is called when an account changes its
	 * connection status after prepareToUnload() has been called.
	 * When all accounts are disconnected, this slot will emit
	 * the readyToUnload() signal.
	 */
	virtual void checkAccountStates( void );
};

}

#endif
