/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALUIEDITENTRYWIDGET_H
#define BLOKKALUIEDITENTRYWIDGET_H

#include <QtGui/QWidget>

#include "entryextensionwidget.h"
#include "../blokkal_export.h"

class KAction;
class KActionCollection;

namespace Blokkal {
class Blog;
class Entry;
namespace Ui {
class HtmlEntryTextEdit;
class FormattingBackend;
/**
 * @short Widgets used to edit entries.
 *
 * This widget provides inputs for basic entry properties such as
 * date, subject, body text and categories. The constructor calls
 * Blog::supportedFeatures() to determine which properties are actually
 * supported by the entry and hides/disables the corresponding widgets
 * if necessary.
 * The functionality of this widget may be extended by two ways: either with
 * Blokkal::Ui::FormattingBackend or with Blokkal::Ui::EntryExtensionWidget.
 * The latter are returned by Entry::createExtensionWidget().
 * The constructor calls that method and adds all returned widgets to the appropriate
 * locations in the layout.
 * The Blokkal::Ui::FormattingBackend returned by Blokkal::Protocol
 * is used to add special markup tags (or other actions)
 * to the GUI of the owning view. The Blokkal::Ui::FormattingBackend will
 * be registered as a child KDEXMLGUIClient of this widget.
 * @note The internal entry is not destroyed, when the widget is destroyed.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT EditEntryWidget : public QWidget
{
Q_OBJECT
public:
	/**
	 * Creates a new EditEntryWidget.
	 * @note setInstance() will be called in this constructor
	 * with the instance of the protocol plugin.
	 * @param entry the entry to edit
	 * @param parentWidget the parentWidget
	 */
	EditEntryWidget( Blokkal::Entry * entry, QWidget * parentWidget = 0 );
	
	/**
	 * Destroys the widget
	 */
	~EditEntryWidget( void );

	/**
	 * Applies all changes to the Entry object and returns it.
	 * @return the modified @ref Blokkal::Entry object
	 */
	Blokkal::Entry * apply( void );

	/**
	 * Returns the entry without updating any values to the
	 * contents of the input widgets or altering it otherwise.
	 * @return the entry
	 */
	Blokkal::Entry * entry( void ) const;

	/**
	 * Returns the specified extension widget or 0 if no such widget exists.
	 * @param extension the identifier of the wxtension widget to return
	 * @return the specified extension widget
	 */
	Blokkal::Ui::EntryExtensionWidget * extensionWidget( EntryExtensionWidget::ExtensionWidget extension ) const;

	/**
	 * This method is used to tell the view whether
	 * the entry in this widget may be posted or not
	 * @return TRUE if the entry may be posted
	 */
	bool isContentLegal( void );

	/**
	 * @return the FormattingBackend of this widget
	 */
	Blokkal::Ui::FormattingBackend * formattingBackend( void ) const;

	/**
	 * Hides the features specified by @a features, which is
	 * an or'ed list of Blog::EntryFeatures values.
	 * All other features will be shown.
	 * @a features features to be shown
	 */
	void hideFeature( unsigned int features );
	
	QString caption( void ) const;
	void setCaption( const QString & caption );	

public Q_SLOTS:
	/**
	 * Resets the contents of all inputs widgets to the values of the
	 * Entry object. This method does also call the rollBack() methods of
	 * all EntryExtensionWidgets.
	 */
	void rollBack( void );
	
Q_SIGNALS:
	/**
	 * This signal is emitted whenever the content of the widget has been
	 * modified somehow. This means, that apply() might result in a modified
	 * entry.
	 */
	void contentsChanged( void );

	/**
	 * This signal is emitted whenever the content of this widget has been
	 * modified in a fassion that affects the caption.
	 */
	void captionChanged( const QString & newCaption );

	/**
	 * This signal is emitted when the user has entered a new category.
	 * If your entry does not specify that it supports category adding
	 * this signal will no be emitted.
	 * When this signal is emitted it is up to you to initiate the
	 * necessary actions.
	 * @a newCategoryString may contain any string.
	 */
	void newCategory( const QString & newCategoryString );

	/**
	 * This signal is emitted when the user has selected a category
	 * to be removed.
	 * If your entry does not specify that it supports category removal
	 * this signal will no be emitted.
	 * When this signal is emitted it is up to you to initiate the
	 * necessary actions.
	 */
	void destroyCategory( const QString & removeCategoryId );

protected:
	/**
	 * @return the HtmlEntryTextEdit of this widget
	 */
	Blokkal::Ui::HtmlEntryTextEdit * entryTextEdit( void ) const;

private Q_SLOTS:
	void updateEntryDateTime( void );
	void emitContentsChanged( void );
	void slotDateChanged( void );
	void emitNewCategory( void );
	void emitCategoryRemoved( const QString & categoryId );

	void updateSplitterVisibility( void );

private:
	friend class View;
	class Private;
	Private * const d;
};

}
}

#endif
