/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UIENTRYEXTENSIONWIDGET_H
#define BLOKKAL_UIENTRYEXTENSIONWIDGET_H

#include "../blokkal_export.h"

#include <QtGui/QWidget>

namespace Blokkal {
class Entry;
namespace Ui {
class EditEntryWidget;

/**
 * @short Base class for extensions to the EditEntryWidget.
 *
 * Widget derived from this class can be used to extend the functionality
 * of the EditEntryWidget to cope with special features of the protocol
 * you are implementing.
 * Make sure you set up your Entry object correctly. After that the extensions will
 * be added automatically.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT EntryExtensionWidget : public QWidget
{
Q_OBJECT
public:
	/**
	 * This enumeration is used to identify the extension slots in the EditEntryWidget.
	 * If you specify that your entry supports an extension widget, you must reimplement
	 * Blokkal::Entry::createExtensionWidget() appropriately.
	 * It is entirely up to you how many or what kind of entry manipulation
	 * widgets you provide with an extension widget.
	 * @see Blokkal::Entry::createExtensionWidget()
	 */	
	enum ExtensionWidget {
		ExtensionWidget0 = 0x01,
		ExtensionWidget1 = 0x02,
		ExtensionWidget2 = 0x04,
		ExtensionWidget3 = 0x08,
		ExtensionWidget4 = 0x10
	};
	
	/**
	 * Constructor
	 * @param entry the entry this widget will manipulate
	 * @param parent the parent widget, in this case the EditEntryWidget
	 */
	EntryExtensionWidget( Entry * entry, Blokkal::Ui::EditEntryWidget * parent );

	/**
	 * Destructor
	 */
    virtual ~EntryExtensionWidget( void );

Q_SIGNALS:
	void visibilityChanged( void );

protected:
	/**
	 * Returns a pointer to the Entry object passed to the constructor
	 * @return a pointer to the Entry object passed to the constructor
	 */
	Blokkal::Entry * entry( void ) const;

	/**
	 * Returns a pointer to the owning EditEntryWidget
	 * @return a pointer to the owning EditEntryWidget
	 */
	const Blokkal::Ui::EditEntryWidget * editEntryWidget( void ) const;

	virtual bool event( QEvent * event );

public Q_SLOTS:
	/**
	 * Reimplement this slot to set the content of your widget
	 * to reflect the data in the Entry object
	 */
	virtual void rollBack( void ) = 0;

	/**
	 * Reimplement this slot to update the Entry object to
	 * the data in the widget
	 */
	virtual void apply( void ) = 0;

Q_SIGNALS:
	/**
	 * This signals is emitted, whenever the content of this widget is
	 * changed by the user input.
	 */
	void contentChanged( Blokkal::Ui::EntryExtensionWidget * widget );

protected:
	/**
	 * Reimplement this method if you need to do something
	 * after the EditEntryWidget has been constructed, like
	 * connecting to slots or signals of other extension widgets.
	 * This method is called once after all other extension widgets have
	 * been constructed.
	 * The default implementation does nothing.
	 * @see EditEntryWidget::extensionWidget()
	 */
	virtual void postInit( void );
	
protected Q_SLOTS:
	/**
	 * Emits the contentChanged() signal.
	 */
	virtual void emitContentChanged( void );
	
private:
	friend class EditEntryWidget;
	class Private;
	Private * d;
};

}

}

#endif
