/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UIFORMATTINGBACKEND_H
#define BLOKKAL_UIFORMATTINGBACKEND_H

#include "../blokkal_export.h"

#include <kxmlguiclient.h>

#include <QtCore/QObject>

namespace Blokkal {

namespace Ui {
class EditEntryWidget;
class HtmlEntryTextEdit;
/**
 * @short Entry point for custom formating options.
 *
 * This class can be used by plugins to add custom
 * (HTML) tags to the standard options offered by EditEntryWidget.
 * This class is a child of KXMLGUIClient, so you may merge
 * your actions into the menu of the entry view.
 * Use
 * \code
 * setXMLFile( "youruifile.rc" );
 * \endcode
 * in the constructor to do so.
 * A sample ui file looks like this:
 * \code
 * <!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
 * <kpartgui name="blokkal_yourprotocol_edit" version="1">
 * <MenuBar>
 * <Menu name="edit"><text>&amp;Edit</text>
 * <Action name="edit_yourprotocol_customaction1" />
 * <Action name="edit_yourprotocol_customaction2" />
 * </Menu>
 * </MenuBar>
 * <ToolBar name="editToolBar"><Text>Edit Toolbar</Text>
 * <Action name="edit_yourprotocol_customaction3" />
 * </ToolBar>
 * \endcode
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT FormattingBackend : public QObject, public KXMLGUIClient
{
Q_OBJECT
public:
	/**
	 * Constructor.
	 * Use editEntryWidget() to access the EditEntryWidget
	 * and do not cast parent widget, as the parent widget may change
	 * after the backend has been constructed.
	 * @param parent owning EditEntryWidget
	 */
	FormattingBackend( Blokkal::Ui::EditEntryWidget * parent );

	/**
	 * Destructor
	 */
	virtual ~FormattingBackend( void );

	/**
	 * If your protocol provides tags, that will be formatted
	 * in a special way by the server you may emulate that behaviour
	 * in this method. @a richText will be used for preview purposes
	 * only, so you may do whatever necessary for the correct appearance.
	 * The default implementation just passed @a richText as return value.
	 * @note You must pass @a richText as the return value of this method.
	 * @param richText rich text string you may apply special formatting to
	 * @return a reference to @a richText
	 */
	virtual QString & formatForPreview( QString & richText ) const;

	/**
	 * Returns a pointer to the owning widget.
	 * @return a pointer to the owning widget
	 */
	Blokkal::Ui::EditEntryWidget * editEntryWidget( void ) const;

protected:
	/**
	 * Inserts text at the current cursor position.
	 * @note Calling this method in the constructor will do nothing.
	 * @param text text to insert
	 */
	void insertTextAtCursor( const QString & text );

	/**
	 * If some text is selected, the selected text is framed
	 * by @a front and  @a tail.
	 * If no text is selected the current cursor position is used instead.
	 * @note Calling this method in the constructor will do nothing.
	 * @param front text to place in front of the selection
	 * @param tail text to place after the selection
	 */
	void frameSelection( const QString & front, const QString & tail );

	/**
	 * Makes links, that are not clickable clickable
	 */
	QString & formatLinks( QString & richText ) const;

	/**
	 * Replaces "\n" with "<br/>"
	 */
	QString & formatLineBreaks( QString & richText ) const;

private:
	/**
	 * This method is used by EditEntryWidget to set the
	 * text edit this backend is actually communicating with.
	 * I don't want subclasses to be able to tinker with that
	 * object directly.
	 */
	void setTextEdit( HtmlEntryTextEdit * textEdit );
		
	friend class EditEntryWidget;
	class Private;
	Private * const d;
};

}

}

#endif
