/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UiPROVIDERCOMBOBOX_H
#define BLOKKAL_UiPROVIDERCOMBOBOX_H

#include "../blokkal_export.h"

#include <kcombobox.h>

class QIcon;

namespace Blokkal {

namespace Ui {

class ProviderInformation;
/**
 * List of ProviderInformation objects
 */
typedef QList<Blokkal::Ui::ProviderInformation> ProviderList;

/**
 * @short KComboBox for selection of provider setups
 *
 * This class is intended to simplify presentation of various
 * provider setups.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT ProviderComboBox : public KComboBox
{
Q_OBJECT
public:
	/**
	 * Creates a new ProviderComboBox.
	 * @param parent parent widget
	 */
	ProviderComboBox( QWidget * parent = 0 );
	
	/**
	 * Destructor
	 */
	virtual ~ProviderComboBox( void );

	/**
	 * Replaces the current provider list with @a providers
	 * @param providers the new provider list
	 * @note An item for "Custom" is always added to the provider list.
	 */
	void setProviders( const ProviderList & providers );

	/**
	 * Adds the providers in @a providers to the current provider list.
	 * @param providers the new providers
	 * @note The "Custom" item is always at the end of the list that is presented to the user
	 */
	void addProviders( const ProviderList & providers );

	/**
	 * Returns the id string of the currently selected provider.
	 * @return the id string of the currently selected provider
	 */
	QString currentProvider( void ) const;

public Q_SLOTS:
	/**
	 * Sets the currently selected provider to the one which
	 * id matches @a id.
	 * If no such provider exists the "Custom"-Element is selected
	 * @param id id of the provider to select
	 */
	void setCurrentProvider( const QString & id );

Q_SIGNALS:
	/**
	 * This signal is emitted when a new provider is selected.
	 * @param id id of the newly selected provider
	 */
	void providerSelected( const QString & id );

private Q_SLOTS:
	void emitProviderSelected( int index );

private:
	class Private;
	Private * const d;

};

/**
 * @short Encapsulates provider information
 *
 * This class encapsulates provider information.
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT ProviderInformation {
public:
	/**
	 * Creates a new ProviderInformation object
	 * @param id internal id of the provider
	 * @param name arbitrary name of the provider
	 */
	ProviderInformation( const QString & id = QString::null,
	                     const QString & name = QString::null );

	/**
	 * Creates a new ProviderInformation object
	 * @param id internal id of the provider
	 * @param name arbitrary name of the provider
	 * @param icon optional icon of the provider
	 */
	ProviderInformation( const QString & id,
	                     const QString & name,
	                     const QIcon & icon );

	/**
	 * Copy Constructor
	 * @param provider information to copy into the new object
	 */
	ProviderInformation( const Blokkal::Ui::ProviderInformation & provider );

	/**
	 * Destructor
	 */
	~ProviderInformation( void );

	/**
	 * Returns the id of this provider. The "Custom"-configuration
	 * has always the id QString::null.
	 * @return the id of this provider
	 */
	const QString & id( void ) const;

	/**
	 * Returns the name of this provider.
	 * @return the name of this provider
	 */
	const QString & name( void ) const;

	/**
	 * Returns the icon of this provider.
	 * @return the icon of this provider
	 */
	const QIcon & icon( void ) const;

	/**
	 * Creates a copy of the contents of @a provider.
	 * @param provider the new content of this object
	 */
	Blokkal::Ui::ProviderInformation & operator=( const Blokkal::Ui::ProviderInformation & provider );

private:
	class Private;
	const Private * d;
};
}

}

#endif
