/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "gentry.h"
#include "gentry.moc"

#include "gblog.h"
#include "gaccount.h"
#include "gio/jobs.h"
#include "gui/entryextensionwidget1.h"

#include <blokkalutils.h>

#include <kdebug.h>
#include <kicon.h>
#include <syndication/atom/entry.h>
#include <syndication/atom/link.h>
#include <syndication/atom/content.h>

#include <QVariant>
#include <QDomElement>

#include <math.h>

class G::Entry::Private {
public:
	Private( void ) {}

	QDomDocument doc; //document contains atom entry as first child

	static QString extractPostId( const QDomElement & entryElement ) {
		Syndication::Atom::Entry entry( entryElement );
		QRegExp idExp( "post-\\d+" );
		int idIndex = idExp.indexIn( entry.id() );
		if( idIndex == -1 ) {
			kError() << "failed to extract post id" << endl;
			return QString::null;
		}
		return entry.id().mid( idIndex + 5, idExp.matchedLength() - 5 );
	}

	static KUrl extractPostUrl( const QDomElement & entryElement ) {
		Syndication::Atom::Entry entry( entryElement );
		QList<Syndication::Atom::Link> links = entry.links();
		for( int i = 0; i < links.size(); ++i ) {
			if( links[i].rel() == "alternate" ) {
				return links[i].href();
			}
		}
		//drafts do not have a "alternate" url
		//kWarning() << "could not find a suitable url" << endl;
		return KUrl();
	}
};

G::Entry::Entry( G::Blog * parentBlog ):
Blokkal::Entry( parentBlog ),
d( new Private() )
{
}

G::Entry::Entry( G::Blog * parentBlog, const QDomElement & entryElement ):
Blokkal::Entry( parentBlog,
                Private::extractPostId( entryElement ),
                Private::extractPostUrl( entryElement ) ),
d( new Private() )
{
	//NOTE links do not have a "alternate" url
	Syndication::Atom::Entry entry( entryElement );
	setSubject( entry.title() );
	setDate( QDateTime::fromTime_t( entry.published() ) );
	setText( entry.content().asString() );

	QList<Syndication::Atom::Link> links = entry.links();
	for( int i = 0; i < links.size(); ++i ) {
		if( links[i].rel() == "edit" ) {
			writeEntry( "editUrl", links[i].href() );
			break;
		}
	}

	QList<QDomElement> elements = entry.unhandledElements();
	for( int i = 0; i < elements.size(); ++i ) {
		QDomElement e = elements[i];
		if( e.nodeName() != "app:control" ) {
			continue;
		}
		
		QDomNodeList list = e.childNodes();
		for( int j = 0; j < list.size(); ++j ) {
			QDomElement el = list.at(i).toElement();
			if( !el.isNull()
					&& el.nodeName() == "app:draft"
					&& el.firstChild().toText().data() == "yes" )
			{
				setDraft( TRUE );
				break;
			}
		}
	}

	d->doc.createElement( "entry" ); //init document
	d->doc.appendChild( entryElement );

	d->doc.firstChild().removeChild( d->doc.firstChild().namedItem( "content" ) );
}

G::Entry::~Entry( void )
{
	delete d;
}

Blokkal::Io::EntryJob * G::Entry::createJob( Blokkal::Entry::JobType type )
{
	switch( type ) {
	case PostJob:
		{
			Blokkal::Io::EntryJob * job = new Blokkal::Io::EntryJob( this );
			job->addSubjob( new G::Io::PostEntryJob( this ) );
			return job;
		}
		break;
	case DeleteJob:
		{
			Blokkal::Io::EntryJob * job = new Blokkal::Io::EntryJob( this );
			job->addSubjob( new G::Io::DeleteEntryJob( this ) );
			return job;
		}		
		break;
	case UserJob:
		break;
	}
	
	return 0;
}

QByteArray G::Entry::toXmlByteArray( void ) const
{
	QDomNode n;
	if( d->doc.isNull() ) {
		//create
		d->doc.appendChild( d->doc.createElementNS( "http://www.w3.org/2005/Atom", "entry" ) );
	}

	//update
	//update date
	n = d->doc.firstChild().namedItem( "published" );
	if( n.isNull() ) {
		n = d->doc.createElement( "published" );
		d->doc.firstChild().appendChild( n );
	}
	if( n.hasChildNodes() ) {
		n.removeChild( n.firstChild() );
	}
	n.appendChild( d->doc.createTextNode( atomTimeStamp() ) );

	//update subject
	n = d->doc.firstChild().namedItem( "title" );
	if( n.isNull() ) {
		n = d->doc.createElement( "title" );
		d->doc.firstChild().appendChild( n );
		n.toElement().setAttribute( "type", "text" );
		
	}
	if( n.hasChildNodes() ) {
		n.removeChild( n.firstChild() );
	}
	n.appendChild( d->doc.createTextNode( subject() ) );


	//update content
	n = d->doc.firstChild().namedItem( "content" );
	if( n.isNull() ) {
		n = d->doc.createElement( "content" );
		n.toElement().setAttribute( "type", "html" );
		d->doc.firstChild().appendChild( n );
	}
	if( n.hasChildNodes() ) {
		n.removeChild( n.firstChild() );
	}
	n.appendChild( d->doc.createTextNode( text() ) );

	//update draft flag
	//1st: remove all draft elements
	QDomNodeList list = d->doc.firstChild().childNodes();
	QList<QDomElement> controlElements;
	QList<QDomElement> draftElements;
	for( int i = 0; i < list.size(); ++i ) {
		n = list.at( i );
		if( n.nodeName() != "app:control" ) {
			continue;
		}

		kDebug() << "found app:control element" << endl;
		controlElements.append( n.toElement() );
		QDomNodeList list2 = n.childNodes();
		for( int j = 0; j < list2.size(); ++j ) {
			if( list2.at(j).nodeName() != "app:draft" ) {
				continue;
			}
			kDebug() << "found app:draft element" << endl;
			draftElements.append( list2.at(j).toElement() );
		}
	}
	for( int i = 0; i < draftElements.size(); ++i ) {
		kDebug() << "removing draft element" << endl;
		draftElements[i].parentNode().removeChild( draftElements[i] );
	}
	//2nd: if this entry is a draft, add draft element again
	if( isDraft() ) {
		if( controlElements.isEmpty() ) {
			n = d->doc.createElementNS( "http://purl.org/atom/app#", "app:control" );
			d->doc.firstChild().appendChild( n );
		}
		else {
			n = controlElements.first();
		}

		n.appendChild( d->doc.createElement( "app:draft" ) );
		n.firstChild().appendChild( d->doc.createTextNode( QString::fromUtf8( "yes" ) ) );
	}
	//done with drafts

	//kDebug() << "generated xml: " << d->doc.toByteArray() << endl;
	return d->doc.toByteArray();
}

QString G::Entry::atomTimeStamp( void ) const
{
	QString dateString = date().toString( Qt::ISODate );
	const QDateTime compTime( date().date(), date().time(), Qt::UTC );
	const int secs = date().secsTo( compTime );
	int mins = floor( secs / 60 );
	mins = mins < 0 ? -mins : mins;
	const int hours = floor( mins / 60 );
	const int hourMins =  mins - ( hours * 60 );
	QString offset = QString::number( hours );
	if( offset.length() == 1 ) {
		offset.prepend( '0' );
	}
	if( secs < 0 ) {
		offset.prepend( '-' );
	}
	else {
		offset.prepend( '+' );
	}
	offset.append( ':' );
	QString minuteString = QString::number( hourMins );
	if( minuteString.length() == 1 ) {
		minuteString.prepend( '0' );
	}
	offset.append( minuteString );
	dateString.append( offset );
	return dateString;
}

KUrl G::Entry::editUrl( void ) const
{
	return readEntry( "editUrl" );
}

void G::Entry::saveProperties( void )
{
	writeComplex( "atomXml", QString( toXmlByteArray() ) );
}

void G::Entry::readProperties( void )
{
	QString xml = readComplex( "atomXml" );
	d->doc.setContent( xml );
	//kDebug() << "atom xml set to:" << d->doc.toString() << endl;
}

Blokkal::Ui::EntryExtensionWidget * G::Entry::createExtensionWidget( Blokkal::Ui::EntryExtensionWidget::ExtensionWidget extension, Blokkal::Ui::EditEntryWidget * parent )
{
	if( extension != Blokkal::Ui::EntryExtensionWidget::ExtensionWidget1 ) {
		return 0;
	}
	
	return new G::Ui::EntryExtensionWidget1( this, parent );
}

bool G::Entry::isDraft( void ) const
{
	//return readBoolEntry( "draft", FALSE );
	return !readBoolEntry( "publish", TRUE );
}

void G::Entry::setDraft( bool draft )
{
	//writeEntry( "draft", draft );
	writeEntry( "publish", !draft );
}

KIcon G::Entry::icon( void ) const
{
	if( !isDraft() ) {
		return Blokkal::Entry::icon();
	}

	return KIcon( Blokkal::Utils::fadePixmap( Blokkal::Entry::icon().pixmap( 32, 32 ) ) );
}
