/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "jobs.h"
#include "jobs.moc"

#include <config.h>

#include "loginjob.h"

#include "../gaccount.h"
#include "../gentry.h"
#include "../gblog.h"

#include <kio/job.h>
#include <kdebug.h>
#include <syndication/atom/document.h>
#include <syndication/atom/entry.h>
#include <syndication/elementwrapper.h>

#include <qdom.h>

//-------------------------
//Job (abstract base class)
//-------------------------
class G::Io::Job::Private {
public:
	Private( G::Account * account ):
	account( account ),
	loginJob( 0 ),
	workerJob( 0 ){}

	G::Account * const account;
	LoginJob * loginJob;
	KIO::TransferJob * workerJob;
	QByteArray responseBuffer;
};

G::Io::Job::Job( G::Account * account ):
Blokkal::Io::Job( account ),
d( new Private( account ) )
{
}

G::Io::Job::~Job( void )
{
	delete d;
}

QByteArray G::Io::Job::payload( void ) const
{
	return QByteArray();
}

void G::Io::Job::start( void )
{
	//if( d->account->auth().isEmpty() ) {
		d->loginJob = new LoginJob( d->account );
		addSubjob( d->loginJob );
	/*}
	else {
		createWorkerJob();
	}*/
	
	if( !subjobs().isEmpty() ) {
		subjobs().first()->start();
	}
	else {
		Blokkal::Io::Job::start();
	}
}

void G::Io::Job::subjobFinished( KJob * job )
{
	//TODO disconnect if the login job fails
	if( error() ) {
		return;
	}
	
	if( job == d->loginJob ) {
		createWorkerJob();
		d->loginJob = 0;
	}
	else if( job == d->workerJob ) {
		//TODO check if the request has failed with an auth error
		if( d->workerJob->isErrorPage() ) {
			kDebug() << "retrieved an error page" << endl;
			setError( Blokkal::Io::UnknownError );
			kDebug() << d->responseBuffer << endl;
			d->workerJob = 0;
			return;
		}
		d->workerJob = 0;
		ioJobFinished();
	}

	if( !subjobs().isEmpty() ) {
		subjobs().first()->start();
	}	
}

void G::Io::Job::createWorkerJob( void )
{
	KIO::TransferJob * job = 0;
	QString customHeaders = QString::fromUtf8( "Authorization: GoogleLogin auth=" + d->account->auth() );
	switch( requestType() ) {
	case HttpGet:
		job = KIO::get( url(),
		                KIO::NoReload,
		                KIO::HideProgressInfo );
		break;
	case HttpPost:
		job = KIO::http_post( url(),
		                      payload(),
		                      KIO::HideProgressInfo );
		break;
	case HttpPut:
		job = KIO::http_post( url(),
		                      payload(),
		                      KIO::HideProgressInfo );
		customHeaders.append( QString::fromUtf8( "\r\nX-HTTP-Method-Override: PUT" ) );
		break;
	case HttpDelete:
		job = KIO::http_post( url(),
		                      payload(),
		                      KIO::HideProgressInfo );
		customHeaders.append( QString::fromUtf8( "\r\nX-HTTP-Method-Override: DELETE" ) );
		break;
	}

	if( requestType() != HttpGet ) {
		job->addMetaData( "content-type", "Content-Type: application/atom+xml; charset=utf-8" );
	}
	
	job->addMetaData( "UserAgent",
	                  QString::fromUtf8( "KDE-Blokkal/%1 GData Plugin").arg( VERSION ) );
	job->addMetaData( "ConnectTimeout", "300" );
	job->addMetaData( "customHTTPHeader", customHeaders );

	connect( job,
	         SIGNAL( data( KIO::Job *, const QByteArray & ) ),
	         SLOT( slotDataArrived( KIO::Job *, const QByteArray & ) ) );
	d->responseBuffer = QByteArray();
	d->workerJob = job;
	addSubjob( job );
}

void G::Io::Job::slotDataArrived( KIO::Job * job, const QByteArray & data )
{
	Q_UNUSED( job );
	d->responseBuffer.append( data );
}

void G::Io::Job::ioJobFinished( void )
{
	/*kDebug() << "iojob finished" << endl;
	kDebug() << d->responseBuffer << endl;*/
}

const QByteArray & G::Io::Job::responseBuffer( void ) const
{
	return d->responseBuffer;
}

G::Account * G::Io::Job::account( void ) const
{
	return d->account;
}

//-----------
//GetBlogsJob
//-----------
class G::Io::GetBlogsJob::Private {
public:
	Private( void ) :
	listsBuilt( FALSE ){}

	bool listsBuilt;
	QList< Syndication::Atom::Entry > blogList;
};

G::Io::GetBlogsJob::GetBlogsJob( G::Account * account ):
Job( account ),
d( new Private() )
{
}

G::Io::GetBlogsJob::~GetBlogsJob( void )
{
	delete d;
}

KUrl G::Io::GetBlogsJob::url( void ) const
{
	return account()->connectAddress();
}

G::Io::Job::RequestType G::Io::GetBlogsJob::requestType( void ) const
{
	return HttpGet;
}

const QList< Syndication::Atom::Entry > & G::Io::GetBlogsJob::blogList( void ) const
{
	if( !d->listsBuilt ) {
		QDomDocument doc;
		doc.setContent( responseBuffer(), TRUE );
		for( QDomNode currentNode = doc.firstChild();
		     !currentNode.isNull();
		     currentNode = currentNode.nextSibling() )
		{
			if( currentNode.isElement() && currentNode.nodeName() == "feed" ) {
				Syndication::Atom::FeedDocument adoc( currentNode.toElement() );
				d->blogList = adoc.entries();
				break;
			}
		}
		d->listsBuilt = TRUE;
	}
	
	return d->blogList;
}

//------------
//PostEntryJob
//------------
class G::Io::PostEntryJob::Private {
public:
	Private( const G::Entry * entry ):
	entry( entry ) {}

	const G::Entry * const entry;
};

G::Io::PostEntryJob::PostEntryJob( const G::Entry * entry ):
G::Io::Job( static_cast<G::Account*>( entry->blog()->account() ) ),
d( new Private( entry ) )
{
}

G::Io::PostEntryJob::~PostEntryJob( void )
{
	delete d;
}

KUrl G::Io::PostEntryJob::url( void ) const
{
	if( d->entry->id().isNull() ) {
		return ( static_cast<G::Blog*>( d->entry->blog() ) )->postUrl();
	}

	return d->entry->editUrl();
}

G::Io::Job::RequestType G::Io::PostEntryJob::requestType( void ) const
{
	if( d->entry->id().isNull() ) {
		return HttpPost;
	}

	return HttpPut;
}

QByteArray G::Io::PostEntryJob::payload( void ) const
{
	return d->entry->toXmlByteArray();
}

//-----------
//GetPostsJob
//-----------
class G::Io::GetPostsJob::Private {
public:
	Private( G::Blog * blog ):
	blog( blog ),
	url( KUrl() ),
	listsBuilt( FALSE ){}

	G::Blog * const blog;
	KUrl url;
	bool listsBuilt;
	QList< Blokkal::Entry* > entryList;
};

G::Io::GetPostsJob::GetPostsJob( G::Blog * blog, const QString & id ):
G::Io::Job( static_cast<G::Account*>( blog->account() ) ),
d( new Private( blog ) )
{
	d->url = blog->postUrl();
	d->url.addPath( id );
}

G::Io::GetPostsJob::GetPostsJob( G::Blog * blog, unsigned int n ):
G::Io::Job( static_cast<G::Account*>( blog->account() ) ),
d( new Private( blog ) )
{
	d->url = blog->postUrl();
	d->url.addQueryItem( QString::fromUtf8( "max-results" ), QString::number( n ) );
}

G::Io::GetPostsJob::GetPostsJob( G::Blog * blog, const QString & query, QueryType type ):
G::Io::Job( static_cast<G::Account*>( blog->account() ) ),
d( new Private( blog ) )
{
	d->url = blog->postUrl();
	switch( type ) {
	case QueryText:
		d->url.addQueryItem( QString::fromUtf8( "q" ), query );
		break;
	case QueryAuthor:
		d->url.addQueryItem( QString::fromUtf8( "author" ), query );
		break;
	}
}

G::Io::GetPostsJob::~GetPostsJob( void )
{
	delete d;
}

KUrl G::Io::GetPostsJob::url( void ) const
{
	return d->url;
}

G::Io::Job::RequestType G::Io::GetPostsJob::requestType( void ) const
{
	return HttpGet;
}

const QList< Blokkal::Entry* > & G::Io::GetPostsJob::entryList( void ) const
{
	if( !d->listsBuilt ) {
		QDomDocument doc;
		doc.setContent( responseBuffer(), TRUE );
		Syndication::ElementWrapper wrapper;
		for( QDomNode currentNode = doc.firstChild();
		     !currentNode.isNull();
		     currentNode = currentNode.nextSibling() )
		{
			if( currentNode.isElement() && currentNode.nodeName() == "feed" ) {
				wrapper = Syndication::ElementWrapper( currentNode.toElement() );
				break;
			}
		}

		QList< QDomElement > entryElements = wrapper.elementsByTagNameNS( "http://www.w3.org/2005/Atom", "entry" );
		for( int i = 0; i < entryElements.size(); ++i ) {
			d->entryList.append( new G::Entry( d->blog, entryElements[i] ) );
		}
		d->listsBuilt = TRUE;
	}
	
	return d->entryList;
}

//--------------
//DeleteEntryJob
//--------------
G::Io::DeleteEntryJob::DeleteEntryJob( const G::Entry * entry ):
PostEntryJob( entry )
{
}

G::Io::DeleteEntryJob::~DeleteEntryJob( void )
{
}

G::Io::Job::RequestType G::Io::DeleteEntryJob::requestType( void ) const
{
	return HttpDelete;
}
