/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef G_IOJOBS_H
#define G_IOJOBS_H

#include <blokkalio/jobs.h>

#include <kurl.h>

#include <QList>

template <class T> class QList;
namespace Syndication {
namespace Atom {
class Entry;
}
}

namespace KIO {
class Job;
}

namespace G {
class Account;
class Blog;
class Entry;
namespace Io {

/**
 * @short Jobs base class
 *
 * Jobs base class.
 * @author Martin Mueller <orvio@orvio.de>
*/
class Job : public Blokkal::Io::Job
{
Q_OBJECT
public:
	/**
	 * Creates a new LoginJob.
	 * @param account connect to the address provided by this account
	 */
	Job( G::Account * account );
	
	/**
	 * Destructor
	 */
	virtual ~Job( void );

	enum RequestType {
		HttpGet,
		HttpPost,
		HttpPut,
		HttpDelete
	};

	/**
	 * Overridden to start one job at a time
	 */
	virtual void start( void );

	/**
	 * @return the account this job operates on
	 */
	G::Account * account( void ) const;

protected:
	/**
	 * This method is called when a subjob has finished.
	 */
	virtual void subjobFinished( KJob * job );

	/**
	 * This method is called when worker io job has finished.
	 * This method will not be called if an error occured.
	 * You can access the response bufer with responseBuffer()
	 */
	virtual void ioJobFinished( void );

	/**
	 * @return the data received by the worker io job
	 */
	const QByteArray & responseBuffer( void ) const;

	/**
	 * @return the url for the worker KIO job.
	 */
	virtual KUrl url( void ) const = 0;

	/**
	 * @return request type of the worker job
	 */
	virtual G::Io::Job::RequestType requestType( void ) const = 0;

	/**
	 * The default implementation returns QByteArray()
	 * @return the payload for HttpPut and HttpPost jobs
	 */
	virtual QByteArray payload( void ) const;

private Q_SLOTS:
	void slotDataArrived( KIO::Job *job, const QByteArray & data );

private:
	void createWorkerJob( void );
	class Private;
	Private * const d;
};

/**
 * @short Fetches the blog list
 *
 * Fetches the blog list.
 * @author Martin Mueller <orvio@orvio.de>
*/
class GetBlogsJob : public G::Io::Job
{
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param account act for this account
	 */
	GetBlogsJob( G::Account * account );
	virtual ~GetBlogsJob( void );

	/**
	 * @return the url for the worker KIO job.
	 */
	virtual KUrl url( void ) const;

	/**
	 * @return request type of the worker job
	 */
	virtual G::Io::Job::RequestType requestType( void ) const;

	/**
	 * @return the received blogs
	 */
	const QList<Syndication::Atom::Entry> & blogList( void ) const;

private:
	class Private;
	Private * const d;
};

/**
 * @short Posts or edits an entry on the server
 *
 * Posts or edits an entry on the server.
 * @author Martin Mueller <orvio@orvio.de>
 */
class PostEntryJob : public G::Io::Job
{
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param entry post/edit this entry
	 */
	PostEntryJob( const G::Entry * entry );
	virtual ~PostEntryJob( void );

	/**
	 * @return the url for the worker KIO job.
	 */
	virtual KUrl url( void ) const;

	/**
	 * @return request type of the worker job
	 */
	virtual G::Io::Job::RequestType requestType( void ) const;

	/**
	 * @return the payload for HttpPut and HttpPost jobs
	 */
	virtual QByteArray payload( void ) const;

private:
	class Private;
	Private * const d;
};

/**
 * @short Removes an entry from the server
 *
 * Removes an entry from the server.
 * @author Martin Mueller <orvio@orvio.de>
 */
class DeleteEntryJob : public G::Io::PostEntryJob
{
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param entry post/edit this entry
	 */
	DeleteEntryJob( const G::Entry * entry );
	virtual ~DeleteEntryJob( void );

	/**
	 * @return request type of the worker job
	 */
	virtual G::Io::Job::RequestType requestType( void ) const;
};

/**
 * @short Fetches entries from the server
 *
 * Fetches entries from the server.
 * @author Martin Mueller <orvio@orvio.de>
 */
class GetPostsJob : public G::Io::Job
{
Q_OBJECT
public:
	/**
	 * Enumeration for query types
	 */
	enum QueryType {
		QueryText = 1, ///< Full text search
		QueryAuthor = 2, ///< Search for author
	};
		
	/**
	 * Constructor
	 * @param blog fetch from this blog
	 * @param id fetch the entry with this id
	 */
	GetPostsJob( G::Blog * blog, const QString & id );
	
	/**
	 * Constructor
	 * @param blog fetch from this blog
	 * @param in fetch the last n entries
	 */
	GetPostsJob( G::Blog * blog, unsigned int n );

	/**
	 * Constructor
	 * @param blog fetch from this blog
	 * @param query use this query
	 * @param type the query type
	 */
	GetPostsJob( G::Blog * blog, const QString & query, QueryType type );
	virtual ~GetPostsJob( void );

	/**
	 * @return the url for the worker KIO job.
	 */
	virtual KUrl url( void ) const;

	/**
	 * @return request type of the worker job
	 */
	virtual G::Io::Job::RequestType requestType( void ) const;

	/**
	 * @return the received entries
	 */
	const QList<Blokkal::Entry*> & entryList( void ) const;

private:
	class Private;
	Private * const d;
};

}

}

#endif
