/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mller                            *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "livejournalaccount.h"
#include "livejournalaccount.moc"

#include "livejournalprotocol.h"
#include "livejournalblog.h"

#include "ljio/loginjob.h"
#include "ljio/friendgroupsmanager.h"
#include "ljio/moodmanager.h"
#include "ljio/userpicturemanager.h"
#include "ljio/checkfriendsjob.h"
#include "ljio/getusertagsjob.h"
#include "ljio/friendinfomanager.h"

#include "ui/ljfriendsmanager.h"

#include <blokkalaccountconfig.h>
#include <blokkalwalletmanager.h>
#include <blokkalui/globalsettings.h>
#include <blokkalbehavioursettings.h>
#include <blokkalutils.h>
#include <blokkalblogconfig.h>

#include <kdebug.h>
#include <kcodecs.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kaction.h>
#include <kstandardaction.h>
#include <knotification.h>

#include <QMap>
#include <QTimer>

class LiveJournalAccount::Private {
public:
	Private( void ) :
	useFastServer( FALSE ),
	loginJob( 0 ) ,
	checkFriendsJob( 0 ),
	lastUpdate( QString::null ){}

	bool useFastServer;

	QByteArray userMD5Password;
	
	LJ::IO::LoginJob * loginJob;

	LJ::IO::CheckFriendsJob * checkFriendsJob;
	QString lastUpdate;
	QTimer * checkFriendsTimer;

	LJ::IO::FriendGroupsManager * friendGroupsManager;
	LJ::IO::MoodManager * moodManager;
	LJ::IO::UserPictureManager * userPictureManager;
	LJ::IO::FriendInfoManager * friendInfoManager;

	QList<QAction*> actionList;
	QAction * manageFriendsAction;
	QAction * openFriendsPageAction;
};

LiveJournalAccount::LiveJournalAccount( LiveJournalProtocol * protocol,
                                        const QString & id ):
PasswordedAccount( protocol, id ),
d( new Private() )
{
	d->friendGroupsManager = new LJ::IO::FriendGroupsManager( this );
	d->moodManager = new LJ::IO::MoodManager( this );
	d->userPictureManager = new LJ::IO::UserPictureManager( this );
	d->friendInfoManager = new LJ::IO::FriendInfoManager( this );
	d->checkFriendsTimer = new QTimer( this );
	connect( d->checkFriendsTimer,
	         SIGNAL( timeout( void ) ),
	         SLOT( checkFriendsPage( void ) ) );

	d->manageFriendsAction = KStandardAction::preferences( this,
		SLOT( manageFriends( void ) ),
		this );
	d->manageFriendsAction->setShortcut( 0 );
	d->manageFriendsAction->setText( i18n( "&Manage Friends" ) );
	d->actionList.append( d->manageFriendsAction );

	d->openFriendsPageAction = KStandardAction::open( this,
		SLOT( openFriendsPage( void ) ),
		this );
	d->openFriendsPageAction->setShortcut( 0 );
	d->openFriendsPageAction->setText( i18n( "&Open Friends Page" ) );
	d->actionList.append( d->openFriendsPageAction );
}

LiveJournalAccount::~LiveJournalAccount( void )
{
	if( isDisconnected() ) {
		disconnectAccount( UserDisconnect );
	}
	delete d;
}

bool LiveJournalAccount::useFastServer( void ) const
{
	return d->useFastServer;
}

const QByteArray & LiveJournalAccount::userMD5Password( void ) const
{
	return d->userMD5Password;
}

bool LiveJournalAccount::useChallengeResponseAuth( void ) const
{
	return config()->readBoolEntry( "usechallenge", TRUE );
}

void LiveJournalAccount::connectInner( void )
{
	if( d->loginJob ) {
		kDebug() << "waiting for pending login job" << endl;
		return;
	}

	d->userMD5Password = KMD5( password().toUtf8() ).hexDigest();
	d->loginJob = new LJ::IO::LoginJob( this, this );
	connect( d->loginJob,
	         SIGNAL( result( KJob * ) ),
	         SLOT( slotLoginJobFinished( KJob * )  ) );
	d->loginJob->start();	
}

void LiveJournalAccount::slotLoginJobFinished( KJob * job )
{
	if( !(job == d->loginJob ) ) {
		kWarning() << "not called by login job!"<< endl;
		return;
	}

	if( !d->loginJob->error() ) {
		QStringList accessList = d->loginJob->accessList();

		//remove old journals first		
		QList<Blokkal::Blog *> blogList = blogs();
		for( QList<Blokkal::Blog *>::ConstIterator it = blogList.begin();
		     it != blogList.end();
		     ++it )
		{
			if( !accessList.contains( (*it)->id() ) ) {
				( *it )->removeBlog();
			}
		}
		blogList = QList<Blokkal::Blog *>();
		
		//update existing blogs and add new ones as necessary
		Blokkal::Blog * blog;
		LiveJournalBlog * journal;
		LiveJournalBlog::JournalType journalType;
		for( QStringList::ConstIterator it = accessList.begin();
		     it != accessList.end();
		     ++it )
		{
			journalType = it == accessList.begin() ? LiveJournalBlog::UserJournal : LiveJournalBlog::CommunityJournal;
			blog = Account::blog( *it );
			if( blog ) {
				//journal exists
				journal = dynamic_cast< LiveJournalBlog * >( blog );
				if( journal ) {
					//does not care about journal name for communities
					journal->setBlogName( d->loginJob->name() );
				}
				else {
					kWarning() << "failed to cast to LiveJournalBlog * " << endl;
				}
			}
			else {
				//new journal
				//constructor does not care about journal name for communities
				journal = new LiveJournalBlog( this, *it, journalType, d->loginJob->name() );
				registerBlog( journal );
			}
		}
		setConnectionStatus( Blokkal::Account::Connected );

		QString message = d->loginJob->message();
		if( !message.isEmpty() ) {
			KMessageBox::information( Blokkal::Ui::GlobalSettings::self()->mainWidget(), message );
		}

		//start check friends
		d->lastUpdate = QString::null;
		checkFriendsPage();
	}
	else if( d->loginJob->error() > 0 ) { //IO Errors
		disconnectAccount( Blokkal::Account::IOErrorDisconnect,
		                   d->loginJob->errorString() );
	}
	else if( d->loginJob->error() < 0 ) { //LJ Errors
		//LJ does not report any meaningfull error codes,
		//just a string. If that string mentions something
		//about the password we guess, that the password
		//was problably wrong :)
		//If we didn't do that here we would supply a wrong
		//password forever and ever, thus locking the user
		//out of the account
		if( d->loginJob->errorString().contains( "password", Qt::CaseInsensitive ) ) {
			d->userMD5Password = QByteArray();
			disconnectAccount( Blokkal::Account::AuthenticationDisconnect );
		}
		else {
			//LJ errors are english only, so some more explanation is added
			disconnectAccount( Blokkal::Account::IOErrorDisconnect,
			                   i18n( "You have been disconnected.\nThe server reported the following error:\n\"%1\"", d->loginJob->errorString() ) );
			
		}
	}

	d->loginJob = 0;
}

LJ::IO::FriendGroupsManager * LiveJournalAccount::friendGroupsManager( void ) const
{
	return d->friendGroupsManager;
}

LJ::IO::MoodManager * LiveJournalAccount::moodManager( void ) const
{
	return d->moodManager;
}

LJ::IO::UserPictureManager * LiveJournalAccount::userPictureManager( void ) const
{
	return d->userPictureManager;
}

KUrl LiveJournalAccount::friendsPage( void ) const
{
	return KUrl( QString::fromLatin1( "http://" )
	             + userName()
	             + '.'
	             + serverName() ).url( KUrl::AddTrailingSlash )
		+ QString::fromLatin1( "friends" );
}

void LiveJournalAccount::openFriendsPage( void )
{
	Blokkal::Utils::openWebpage( friendsPage() );
}

bool LiveJournalAccount::isCheckFriends( void ) const
{
	return config()->readBoolEntry( "monitor-friends-page", FALSE );
}

void LiveJournalAccount::setCheckFriends( bool check )
{
	config()->writeEntry( "monitor-friends-page", check );
}

int LiveJournalAccount::checkFriendsInterval( void ) const
{
	return config()->readEntry( "friends-check-timeout", QString::number( 1 ) ).toInt();
}

void LiveJournalAccount::setCheckFriendsInterval( int interval )
{
	config()->writeEntry( "friends-check-timeout", QString::number( interval ) );
}

void LiveJournalAccount::checkFriendsPage( void )
{
	if( !isCheckFriends() ) {
		return;
	}
	
	//kDebug() << k_funcinfo << endl;
	d->checkFriendsTimer->stop();
	if( d->checkFriendsJob ) {
		d->checkFriendsJob->kill();
		d->checkFriendsJob = 0;
	}

	d->checkFriendsJob = new LJ::IO::CheckFriendsJob( this, d->lastUpdate, 1, this );
	connect( d->checkFriendsJob,
	         SIGNAL( result( KJob * ) ),
	         SLOT( notifyOfFriendsPageUpdate( KJob * ) ) );
	d->checkFriendsJob->start();
}

void LiveJournalAccount::notifyOfFriendsPageUpdate( KJob * job )
{
	if( job != d->checkFriendsJob ) {
		kWarning() << "not called by own check friends job!"<< endl;
		return;
	}

	if( d->checkFriendsJob->error() > 0 ) {
		disconnectAccount( Blokkal::Account::IOErrorDisconnect,
		                   d->checkFriendsJob->errorString() );
		return;
	} //ignore LJ errors

	//kDebug() << k_funcinfo << d->checkFriendsJob->lastUpdate() << ", " << d->checkFriendsJob->hasUpdates() << ", " << d->checkFriendsJob->interval() << endl;
	
	bool wasFirstCheck = d->lastUpdate.isNull();
	d->lastUpdate = d->checkFriendsJob->lastUpdate();

	if( d->checkFriendsJob->hasUpdates() ) {
		KNotification * notification = new KNotification( "livejournal_friendsupdate", Blokkal::Ui::GlobalSettings::self()->mainWidget() );
		notification->setText( i18n( "<b>%1</b> : Your friends page has updates.", id() ) );
		QStringList actionList;
		actionList.append( i18n( "Open" ) );
		actionList.append( i18n( "Ignore" ) );
		notification->setActions( actionList );
		notification->setFlags( KNotification::Persistent );
		notification->sendEvent();
		//notification is deleted when acknowledged
		connect( notification,
		         SIGNAL( action1Activated( void ) ),
		         SLOT( openFriendsPage( void ) ) );
		//stop polling until notification has been acknowledged
		connect( notification,
		         SIGNAL( closed( void ) ),
		         SLOT( startCheckFriendsTimer( void ) ) );
	}
	else if( wasFirstCheck ) {
		//first check => this one is always false, check again
		checkFriendsPage();
		//d->checkFriendsJob is now something else => return;
		return;
	}
	else {
		//not first check and no updates => restart timer
		startCheckFriendsTimer( d->checkFriendsJob->interval() );
	}

	d->checkFriendsJob = 0;
}

void LiveJournalAccount::startCheckFriendsTimer( long s )
{
	//user interval is configured in minutes
	const long userInterval = checkFriendsInterval() * 60 * 1000;
	//user interval is returned in seconds
	const long serverInterval = s * 1000;
	
	if( s < 0 ) {
		d->checkFriendsTimer->start( userInterval );
	}
	else if( userInterval < serverInterval ) {
		d->checkFriendsTimer->start( serverInterval );
	}
	else {
		d->checkFriendsTimer->start( userInterval );
	}
}

void LiveJournalAccount::disconnectAccount( Blokkal::Account::DisconnectReason reason, const QString & reasonString )
{
	d->checkFriendsTimer->stop();
	
	if( d->loginJob ) {
		d->loginJob->kill();
		d->loginJob = 0;
	}
	
	if( d->checkFriendsJob ) {
		d->checkFriendsJob->kill();
		d->checkFriendsJob = 0;
	}
	
	PasswordedAccount::disconnectAccount( reason, reasonString );
}

Blokkal::Blog * LiveJournalAccount::restoreBlog( Blokkal::BlogConfig * config )
{
	if( !config ) {
		return 0;
	}

	if( config->readIntEntry( "type", -1 ) < 0 ) {
		return 0;
	}

	LiveJournalBlog::JournalType type = ( LiveJournalBlog::JournalType) config->readIntEntry( "type", -1 );

	LiveJournalBlog * journal = new LiveJournalBlog( this,
	                                                 config->id(),
	                                                 type,
	                                                 config->title() );
	return journal;
}

LJ::IO::FriendInfoManager * LiveJournalAccount::friendInfoManager( void ) const
{
	return d->friendInfoManager;
}

void LiveJournalAccount::manageFriends( void )
{
	( new LJ::UI::ManageFriendsDialog( this, Blokkal::Ui::GlobalSettings::self()->mainWidget() ) )->show();
}

QList< QAction * > LiveJournalAccount::actionList( void )
{
	d->manageFriendsAction->setEnabled( isConnected() );
	d->openFriendsPageAction->setEnabled( Blokkal::BehaviourSettings::self()->webPageLoadingPolicy() != Blokkal::BehaviourSettings::EnumWebPageLoadingPolicy::LoadDisabled );
	return d->actionList;
}
