/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIVEJOURNALACCOUNT_H
#define LIVEJOURNALACCOUNT_H

#include <blokkalaccount.h>

class LiveJournalProtocol;
class KJob;

namespace LJ{
namespace IO {
class FriendGroupsManager;
class MoodManager;
class UserPictureManager;
class FriendInfoManager;
}
}

/**
 * @short Implementation of a LiveJournal account.
 *
 * Implementation of a LiveJournal account.
 * @author Martin Mller <orvio@orvio.de>
 */
class LiveJournalAccount : public Blokkal::PasswordedAccount
{
Q_OBJECT
public:
	/**
	 * Creates a new LiveJournalAccount
	 * @param protocol the owning protocol
	 * @param accountName the account name
	 */
	LiveJournalAccount( LiveJournalProtocol * protocol, const QString& accountName );
	virtual ~LiveJournalAccount( void );

	/**
	 * Returns whether to use challenge/response authentication or not.
	 * The key in the config is "usechallenge".
	 * @return TRUE if challenge/response authentication shall be used
	 */
	bool useChallengeResponseAuth( void ) const;

	/**
	 * Returns whether the fast servers may be accessed or not.
	 * @return TRUE if the fast server may be accessed
	 */
	bool useFastServer( void ) const;

	/**
	 * Returns the MD5 hash of the password.
	 * @return the MD5 hash of the password
	 */
	const QByteArray & userMD5Password( void ) const;

	/**
	 * Returns the friend groups manager of this account.
	 * @return the friend groups manager
	 */
	LJ::IO::FriendGroupsManager * friendGroupsManager( void ) const;

	/**
	 * Returns the mood manager of this account.
	 * @return the mood manager
	 */
	LJ::IO::MoodManager * moodManager( void ) const;

	/**
	 * Returns the user picture manager of this account.
	 * @return the user picture manager
	 */
	LJ::IO::UserPictureManager * userPictureManager( void ) const;

	/**
	 * Returns the friend info manager of this account.
	 * @return the friend info manager
	 */
	LJ::IO::FriendInfoManager * friendInfoManager( void ) const;
	
	/**
	 * Returns the URL of the friends page.
	 * @return the URL of the friends page
	 */
	KUrl friendsPage( void ) const;

	/**
	 * Returns a list of custom actions for context menus.
	 * The default implementation returns an empty list.
	 * @return a list of custom actions
	 */
	virtual QList<QAction*> actionList( void );
	
	/**
	 * Returns whether the friends page of this account shall be monitored
	 * @return TRUE if the friends page shall be monitored.
	 */
	bool isCheckFriends( void ) const;

	/**
	 * Enables/Disables friends page check
	 * @param check TRUE to enable friends page check
	 */
	void setCheckFriends( bool check );

	/**
	 * Returns the check friends timeout.
	 * @return the check friends timeout
	 */
	int checkFriendsInterval( void ) const;

	/**
	 * Sets the check friends timeout.
	 * @param interval the timeout in minutes
	 */
	void setCheckFriendsInterval( int interval = 1 );
	
public Q_SLOTS:
	/**
	 * Disconnect.
	 * @param reason optional disconnect reason
	 * @param reasonString optional reason description
	 */
	virtual void disconnectAccount( Blokkal::Account::DisconnectReason reason = UserDisconnect,
	                                const QString & reasonString = 0 );

	/**
	 * Opens the friends page.
	 */
	void openFriendsPage( void );

	/**
	 * Opens the friends dialog.
	 */
	void manageFriends( void );

protected Q_SLOTS:
	/**
	 * Queries the server for updates to the friends page.
	 * This will result in notifyOfFriendsPageUpdate() beeing called.
	 * @see notifyOfFriendsPageUpdate()
	 */
	void checkFriendsPage( void );

	/**
	 * This slot is called when the job querying the server for updates
	 * to the friends page returns.
	 * If there are updates, the user will be notified.
	 * If there are no updates the query will be repeated after a timeout.
	 * That timeout is either the user specified one or the timeout that
	 * was returned by the server if the user specified timeout is shorter.
	 * @param job the returning job, this slot will only react on jobs created by checkFriendsPage()
	 */
	void notifyOfFriendsPageUpdate( KJob * job );

	/**
	 * This slot starts the timer after which checkFriendsPage() will be called
	 * @param s the timeout in seconds, if < 0 the user specified timeout will be used
	 */
	void startCheckFriendsTimer( long s = -1 );

protected:
	/**
	 * Restores the journal according to the information in @a config.
	 * @param config restore the journal with this configuration
	 * @return the restored journal or 0
	 */
	virtual Blokkal::Blog * restoreBlog( Blokkal::BlogConfig * config );

protected Q_SLOTS:
	virtual void connectInner( void );
	
private Q_SLOTS:
	void slotLoginJobFinished( KJob * job );

private:
	class Private;
	Private * const d;
};

#endif
