/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJLJENTRY_H
#define LJLJENTRY_H

#include <blokkalentry.h>

class LiveJournalBlog;
namespace LJ {

/**
 * @short This class represents a LiveJournal event.
 *
 * This class represents a LiveJournal event.
 * @author Martin Mueller <orvio@orvio.de>
*/
class Entry : public Blokkal::Entry
{
Q_OBJECT
public:
	/**
	 * Creates a new Entry.
	 * @param parentBlog the blog the new entry belongs to
	 * @param poster the username of the user who posted this entry
	 * @param id the id of this entry
	 */
	Entry( LiveJournalBlog * parentBlog,
	       const QString & poster = QString::null,
	       unsigned int id = 0,
	       const KUrl & url = KUrl() );
	
	virtual ~Entry( void );

	/**
	 * This enumeration identifies the comment settings
	 * for a livejournal entry.
	 */
	enum CommentSettings {
		CommentsDefault = 0, ///< Use account defaults
		CommentsEnabled = 1, ///< Enable comments
		CommentsNoEmail = 2, ///< Enable comments, but do not send notification emails
		CommentsDisabled = 3 ///< Disable comments
	};

	/**
	 * This enumeration identifies the screening settings
	 * for a livejournal entry.
	 */
	enum ScreeningSettings {
		ScreeningDefault = 0, ///< Use account defaults
		ScreenNone = 1, ///< Screen no comments
		ScreenAnonymous = 2, ///< Screen anonymous comments
		ScreenNonFriends = 3,///< Screen comments from user not on the friendslist
		ScreenAll = 4, ///< Screen all comments
	};

	/**
	 * This enumeration is used to identify the currently
	 * selected security level.
	 */
	enum SecurityLevel {
		PublicLevel = 0, ///< for public entries
		FriendsOnlyLevel = 1, ///< for entries visible to friends only
		PrivateLevel = 2, ///< for private entries
	 	CustomLevel = 3 ///< for entries with custom security
	};

	/**
	 * This enumeration is used to identify the adult content flag
	 * for an entry
	 */
	enum AdultContent {
		AdultDefault = 0, ///< Journal default
		AdultNone = 1, ///< No adult content
		AdultConcepts = 2, ///< Adult concepts
		AdultExplicit = 3, ///< Explicit adult content
	};

	/**
	 * Returns the keyword of the user picture or QString::null
	 * if the default user icon is used.
	 * @return the user icon keyword
	 */
	QString userIcon( void ) const;

	/**
	 * Sets the user icon of this entry. If you pass QString::null
	 * either the default icon or no icon (depending on the account
	 * setup on the server) will be assiciated to this entry
	 * when it is posted to the server.
	 * @param keyword the user icon keyword
	 */
	void setUserIcon( const QString & keyword = 0 );

	/**
	 * Returns the "current_music" property of this entry.
	 * @return the "current_music" property
	 */
	QString currentMusic( void ) const;

	/**
	 * Sets the "current_music" property of this entry.
	 * @param currentMusic a string with the current music
	 */
	void setCurrentMusic( const QString & currentMusic = 0 );

	/**
	 * Returns the "current_mood" property of this entry.
	 * @return a string describing the current mood
	 * @see currentMoodID()
	 */
	QString currentMood( void ) const;

	/**
	 * Sets the "current_mood" property of this entry.
	 * @param mood a string describing the curren mood
	 * @see setCurrentMoodID()
	 */
	void setCurrentMood( const QString & mood = 0 );

	/**
	 * Returns the "current_moodid" property of this entry. Mood IDs
	 * refer to predefined moods on the server. They have an icon and a
	 * predefined text associated with them. The text can be overidden
	 * with setCurrentMood().
	 * @return the mood id associated with this entry
	 * @see currentMood()
	 * @see LiveJournalAccount::moodManager()
	 */
	unsigned int currentMoodID( void ) const;

	/**
	 * Sets the "current_moodid" property of this entry.
	 * @param id the new mood id
	 * @see currentMoodID()
	 * @see setCurrentMood()
	 * @see LiveJournalAccount::moodManager()
	 */
	void setCurrentMoodID( unsigned int id );

	/**
	 * Return the "current_location" property of this entry.
	 * @return the current location
	 */
	QString currentLocation( void ) const;

	/**
	 * Sets the "current_location" property of this entry.
	 * @param location the current location
	 */
	void setCurrentLocation( const QString & location = 0 );

	/**
	 * Returns the current security level
	 * @return the security level for this entry
	 */
	LJ::Entry::SecurityLevel security( void ) const;
	
	/**
	 * Sets the security level of this entry. The allow mask
	 * will not be affected by this, but may not be considered
	 * when the security level is set to a value that does
	 * not necessarily need the allow mask.
	 * @param security the new security level
	 */
	void setSecurity( LJ::Entry::SecurityLevel security );

	/**
	 * Returns the allow mask of this entry.
	 * @return the allow mask
	 */
	unsigned int allowMask( void ) const;

	/**
	 * Sets the allow mask of this entry.
	 * @param allowMask the new allow mask
	 */
	void setAllowMask( unsigned int allowMask = 0 );

	/**
	 * Returns the id number of this entry as unsigned int.
	 * 0 is an invalid id.
	 * @return the item id of this entry
	 */
	unsigned int uintId( void ) const;

	/**
	 * Returns the "opt_preformatted" property of this entry.
	 * @return opt_preformatted
	 */
	bool preformatted( void ) const;

	/**
	 * Sets the "opt_preformatted" of this entry
	 * @param preformatted new value for "opt_preformatted"
	 */
	void setPreformatted( bool preformatted = FALSE );

	/**
	 * Returns the "opt_backdated" property of this entry.
	 * @return opt_backdated
	 */
	bool backdated( void ) const;

	/**
	 * Sets the "opt_backdated" of this entry
	 * @param backdated new value for "opt_backdated"
	 */
	void setBackdated( bool backdated );

	/**
	 * Returns the comment settings for this entry
	 * @return the comment settings
	 */
	LJ::Entry::CommentSettings commentSettings( void ) const;

	/**
	 * Sets the comment settings for this entry.
	 * @param commentSettings the new comment settings
	 */
	void setCommentSettings( LJ::Entry::CommentSettings commentSettings = LJ::Entry::CommentsDefault );

	/**
	 * Returns the "opt_screening" property of this entry
	 * @return the "opt_screening" property
	 */
	LJ::Entry::ScreeningSettings screening( void ) const;

	/**
	 * Sets the "opt_screening"  property of this entry
	 * @param screeningSettings the new screening settings
	 */
	void setScreening( LJ::Entry::ScreeningSettings screeningSettings = LJ::Entry::ScreeningDefault );

	/**
	 * Returns the "revnum" property of this entry.
	 * @return the "revnum" property
	 */
	unsigned long revnum( void ) const;

	/**
	 * Sets the "revnum" property of this entry.
	 * @param revnum the new "revnum" value
	 */
	void setRevnum( unsigned long revnum = 0 );

	/**
	 * Returns the "revtime" property of this entry. This
	 * value the date of the last modification of this entry.
	 * @return the "revtime" property
	 */
	unsigned long revtime( void ) const;

	/**
	 * Sets the "revtime" property of this entry.
	 * @param revtime the new "revtime" value
	 */	
	void setRevtime( unsigned long revtime = 0 );

	/**
	 * Returns the poster of this entry.
	 * @return the poster of this entry.
	 */
	QString poster( void ) const;

	/**
	 * Returns the anum of this entry.
	 * @return the anum of this entry
	 */
	unsigned int anum( void ) const;

	/**
	 * Sets the anum of this entry.
	 * @param anum the anum of this entry.
	 */
	void setAnum( unsigned int anum );

	/**
	 * This method returns the icon of the entry.
	 * The icon represents the security level of the entry.
	 * @return the entry icon
	 */
	virtual KIcon icon( void ) const;

	/**
	 * Returns the widget associated with @a extension.
	 * @param extension the identifier for the widget to create
	 * @param parent the parent widget
	 * @return the widget associated with @a extension.
	 */
	virtual Blokkal::Ui::EntryExtensionWidget * createExtensionWidget( Blokkal::Ui::EntryExtensionWidget::ExtensionWidget extension, Blokkal::Ui::EditEntryWidget * parent );

	/**
	 * Merges the properties of @entry into this entry.
	 * @param entry source entry
	 */
	virtual void merge( const Blokkal::Entry * entry );

	/**
	 * Creates and returns a job of @a type
	 * @return a job of @a type
	 */
	virtual Blokkal::Io::EntryJob * createJob( Blokkal::Entry::JobType type );

	/**
	 * @return the adult content flag for this account
	 */
	LJ::Entry::AdultContent adultContent( void ) const;

	/**
	 * Sets the adult content flag for this entry
	 * @param content the adult content flag
	 */
 	void setAdultContent( LJ::Entry::AdultContent content );

protected:
	/**
	 * Sets the poster of this entry.
	 * @param poster the username of the user who posted this entry
	 */
	void setPoster( const QString & poster );
};

}

#endif
