/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_IOBLOGBROWSINGDRIVER_H
#define LJ_IOBLOGBROWSINGDRIVER_H

#include <blokkalio/blogbrowsingdriver.h>
#include <blokkalio/jobs.h>

class LiveJournalBlog;

namespace LJ {
namespace IO {

class BlogBrowsingDriver : public Blokkal::Io::BlogBrowsingDriver {
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param parent parent blog
	 */
	BlogBrowsingDriver( LiveJournalBlog * parent );
	virtual ~BlogBrowsingDriver( void );
	
	/**
	 * If your protocol does only support a limited
	 * number of entries to be loaded by this mode, overridde this method to
	 * return the limit.
	 * @return 50
	 */
	virtual unsigned int recentLimit( void ) const;
	
	/**
	 * This method returns an or'ed list of BlogOperations values.
	 * @return returns an or'ed list of BlogOperations values
	 */
	virtual unsigned int supportedOperations( void ) const;

	/**
	 * Creates and returns a job that loads all entries in @a idList.
	 * @return a job that loads all entries in @a idList
	 */
	virtual Blokkal::Io::LoadEntryJob * createLoadEntryJob( const QStringList & idList );
	
	/**
	 * Creates and returns a job that loads all entries posted on @a date.
	 * @return a job that loads all entries posted on @a date
	 */
	virtual Blokkal::Io::LoadEntryJob * createLoadEntryJob( const QDate & date );

	/**
	 * Creates and returns a job that loads the last @a n entries in the blog.
	 * @see recentLimit()
	 * @return a job that loads the last @a n entries in the blog
	 */
	virtual Blokkal::Io::LoadEntryJob * createLoadEntryJob( unsigned int n );

	/**
	 * Creates and returns a job that loads the index of the blog between
	 * @a from and @a to.
	 * If @a from is invalid, the index starts with the date the blog was created on.
	 * If @a to is invalid, the index ends with the current date.
	 * @return job that loads the index of the blog between @a from and @a to
	 */
	virtual Blokkal::Io::LoadIndexJob * createLoadIndexJob( const QDate & from = QDate(), const QDate & to = QDate() );
	
};

class LoadEntryJob : public Blokkal::Io::LoadEntryJob {
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param blog parent blog
	 */
	LoadEntryJob( Blokkal::Blog * blog );
	virtual ~LoadEntryJob( void );
	
	/**
	 * @return the loaded entries
	 */
	virtual QList<Blokkal::Entry *> entries( void ) const;
	
protected:
	/**
	 * This method is called when a subjob has finished.
	 * Overridden to get the entry.
	 */
	virtual void subjobFinished( KJob * job );

private:
	class Private;
	Private * const d;
	
};

class LoadIndexJob : public Blokkal::Io::LoadIndexJob {
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param blog parent blog
	 */
	LoadIndexJob( Blokkal::Blog * blog );
	virtual ~LoadIndexJob( void );
	
	/**
	 * @return the loaded index
	 */
	virtual QMap<QDate, QStringList> index( void ) const;
	
protected:
	/**
	 * This method is called when a subjob has finished.
	 * Overridden to get the index.
	 */
	virtual void subjobFinished( KJob * job );

private:
	class Private;
	Private * const d;
	
};

}

}

#endif
