/***************************************************************************
 *   Copyright (C) 2006 by Martin Mller                                   *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "friendgroupsmanager.h"
#include "friendgroupsmanager.moc"

#include "../livejournalaccount.h"

#include <kdebug.h>

#include <qdom.h>
#include <QMap>
#include <QFile>

#include <iostream>

/**
 * @short This class is an implementation detail of @ref LJ::IO::FriendGroupsManager.
 *
 * This class is an implementation detail of @ref LJ::IO::FriendGroupsManager.
 * @author Martin Mller <orvio@orvio.de>
 */
class LJ::IO::FriendGroupsManager::Private {
public:
	/**
	 * Creates a new object.
	 */
	Private( void ) :
	friendGroupsDocument( QDomDocument( "LJFriendGroups" ) ),
	groupIDGroupMap( QMap< unsigned short, FriendGroup * >() )
	{
		friendGroupsDocument.appendChild( friendGroupsDocument.createElement( "friendgrouplist" ) );
	}

	/**
	 * Returns a node representing the given node id.
	 * If no such node exists one will be created.
	 * @param groupID
	 * the group id
	 */
	QDomElement groupNode( unsigned short groupID ) {
		QDomNode currentNode = friendGroupsDocument.firstChild().namedItem( "friendgroup" );
		QDomElement currentElement;
		
		while( !currentNode.isNull() ) {
			if( currentNode.isElement() && currentNode.nodeName() == "friendgroup" ) {
				currentElement = currentNode.toElement();
				if( currentElement.attribute( "id" ).toUShort() == groupID ) {
					return currentElement;
				}
			}

			currentNode = currentNode.nextSibling();
		}

		currentElement = friendGroupsDocument.createElement( "friendgroup" );
		currentElement.setAttribute( "id", groupID );
		friendGroupsDocument.firstChild().appendChild( currentElement );
		return currentElement;
	}

	/**
	 * the internal docuemnt
	 */
	QDomDocument friendGroupsDocument;

	/**
	 * Maps group IDs to the FriendGroup objects
	 */
	QMap< unsigned short, FriendGroup * > groupIDGroupMap;
	
	/**
	 * friend groups file name
	 */
	QString friendGroupsFileName;
};

LJ::IO::FriendGroupsManager::FriendGroupsManager( LiveJournalAccount * parentAccount ):
QObject( parentAccount ),
d( new Private() )
{
	d->friendGroupsFileName = parentAccount->dataDirectory() + QString::fromLatin1( "friendgroups.xml" );
	
	//loading and parsing file
	QFile friendGroupsFile( d->friendGroupsFileName );
	if( friendGroupsFile.open( QIODevice::ReadOnly ) ) {
		if( !d->friendGroupsDocument.setContent( &friendGroupsFile ) ) {
			kError() << "error parsing friend groups file " << friendGroupsFile.fileName() << endl;
		}
		friendGroupsFile.close();
	}

	//build group map
	QDomNode currentNode = d->friendGroupsDocument.firstChild().namedItem( "friendgroup" );
	QDomElement currentElement;
	unsigned short groupID;
		
	while( !currentNode.isNull() ) {
		if( currentNode.isElement() && currentNode.nodeName() == "friendgroup" ) {
			currentElement = currentNode.toElement();
			groupID = currentElement.attribute( "id" ).toUShort();
			d->groupIDGroupMap.insert( groupID, new FriendGroup( this, groupID ) );
		}
		
		currentNode = currentNode.nextSibling();
	}
}

LJ::IO::FriendGroupsManager::~FriendGroupsManager( void )
{
	QFile friendGroupsFile( d->friendGroupsFileName );
	if( !friendGroupsFile.open( QIODevice::WriteOnly ) ) {
		kError() << "error opening friend groups file " << friendGroupsFile.fileName() << endl;
	}

   QTextStream fileStream( &friendGroupsFile );
	d->friendGroupsDocument.save( fileStream, 3 );
 
   friendGroupsFile.close();
	
	delete d;
}

void LJ::IO::FriendGroupsManager::setValidGroupIDs( QList< unsigned short > validIDs )
{
	if( d->groupIDGroupMap.isEmpty() ) {
		return;
	}
	
	QList<unsigned short> currentIDs = d->groupIDGroupMap.keys();
	FriendGroup * removedGroup;

	for( QList<unsigned short>::ConstIterator it = currentIDs.begin();
	     it != currentIDs.end();
	     ++it )
	{
		if( !validIDs.contains( *it ) ) {
			removedGroup = d->groupIDGroupMap[ *it ];
			d->groupIDGroupMap.remove( *it );
			emit groupRemoved( removedGroup );
			removedGroup->deleteLater();

			d->friendGroupsDocument.firstChild().removeChild( d->groupNode( *it ) );
		}
	}
}


QList< LJ::IO::FriendGroup * > LJ::IO::FriendGroupsManager::groups( void )
{
	return d->groupIDGroupMap.values();
}


LJ::IO::FriendGroup * LJ::IO::FriendGroupsManager::group( unsigned short groupID )
{
	if( d->groupIDGroupMap.contains( groupID ) ) {
		return d->groupIDGroupMap[ groupID ];
	}

	FriendGroup * newGroup = new FriendGroup( this, groupID );
	d->groupIDGroupMap.insert( groupID, newGroup );
	emit groupAdded( newGroup );
	return newGroup;
}

/**
 * @short This class is an implementation detail of @ref LJ::IO::FriendGroup.
 *
 * This class is an implementation detail of @ref LJ::IO::FriendGroup.
 * @author Martin Mller <orvio@orvio.de>
 */
class LJ::IO::FriendGroup::Private
{
public:
	/**
	 * Creates a new object
	 */
	Private( void ) {}

	/**
	 * the node of this friend group
	 */
	QDomElement groupNode;
};

LJ::IO::FriendGroup::FriendGroup( LJ::IO::FriendGroupsManager * manager, unsigned short groupID, const QString & groupName, unsigned short sortorder ) :
QObject( manager ),
d( new Private() )
{
	d->groupNode = manager->d->groupNode( groupID );

	if( !groupName.isEmpty() ) {
		setGroupName( groupName );
		setSortorder( sortorder );
	}
}

LJ::IO::FriendGroup::~FriendGroup( void )
{
	delete d;
}

QString LJ::IO::FriendGroup::groupName( void )
{
	return d->groupNode.attribute( "name" );
}

void LJ::IO::FriendGroup::setGroupName( const QString & groupName )
{
	if( groupName == this->groupName() ) {
		return;
	}
	
	d->groupNode.setAttribute( "name", groupName );
	emit groupModified( this );
}

unsigned short LJ::IO::FriendGroup::sortorder( void )
{
	return d->groupNode.attribute( "sortorder" ).toUShort();
}

void LJ::IO::FriendGroup::setSortorder( unsigned short sortorder )
{
	if( sortorder == this->sortorder() ) {
		return;
	}
	
	d->groupNode.setAttribute( "sortorder", QString::number( sortorder ) );
	emit groupModified( this );
}

unsigned short LJ::IO::FriendGroup::groupID( void )
{
	return d->groupNode.attribute( "id" ).toUShort();
}

bool LJ::IO::FriendGroup::isPublic( void )
{
	return d->groupNode.attribute( "public" ) == QString::fromLatin1( "true" );
}

void LJ::IO::FriendGroup::setPublic( bool isPublic )
{
	if( isPublic == this->isPublic() ) {
		return;
	}

	d->groupNode.setAttribute( "public", isPublic ? QString::fromLatin1( "true" ) : QString::fromLatin1( "false" ) );
	emit groupModified( this );
}


