/***************************************************************************
 *   Copyright (C) 2006 by Martin Mller                                   *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_IOFRIENDGROUPSMANAGER_H
#define LJ_IOFRIENDGROUPSMANAGER_H

#include <QObject>
#include <QList>

class LiveJournalAccount;
namespace LJ {
namespace IO {

class FriendGroup;

/**
 * @short This class manages friend groups of a single livejournal account.
 *
 * This class manages friendgroups of a single livejournal account.
 * @author Martin Mller <orvio@orvio.de>
*/
class FriendGroupsManager : public QObject
{
Q_OBJECT
public:
	/**
	 * Creates a new FriendGroupsManager.
	 * @param parentAccount the account this manager belongs to
	 */
	FriendGroupsManager( LiveJournalAccount * parentAccount );
	~FriendGroupsManager( void );

	/**
	 * Removes all groups with group ids not in <i>validIDs</i>.
	 * FriendGroup objects held by other objects are not affected by this.
	 * @param validIDs a list of group IDs currently known to belong to actual friend groups
	 */
	void setValidGroupIDs( QList< unsigned short > validIDs );

	/**
	 * Returns a list of current friend groups.
	 * @return the friend groups
	 */
	QList< LJ::IO::FriendGroup * > groups( void );

	/**
	 * Returns the group with the given group ID.
	 * If no such group exists, a new one will be created.
	 * @param groupID fetch the group with this ID
	 */
	LJ::IO::FriendGroup * group( unsigned short groupID );

signals:
	/**
	 * This signal is emitted, when a friend group is removed
	 */
	void groupRemoved( LJ::IO::FriendGroup * group );

	/**
	 * This signal is emitted, when a friend group is added
	 */
	void groupAdded( LJ::IO::FriendGroup * group );
	
private:
	class Private;
	Private * d;
	friend class FriendGroup;
};

/**
 * @short This class represents a single friend group.
 *
 * This class represents a single friend group. This representation is for
 * local use only and any changes will not be send to the server. The data of this
 * class is shared. Changing one object will change all other objects representing the
 * same group.
 * Creating a group object affects what the group manager will return if asked for the current
 * friend groups.
 * @author Martin Mller <orvio@orvio.de>
 */
class FriendGroup : public QObject
{
Q_OBJECT
public:
	~FriendGroup( void );

	/**
	 * Returns the name of the group.
	 * @return the group name
	 */
	QString groupName( void );

	/**
	 * Sets the name of the group.
	 * @param groupName the new group name
	 */
	void setGroupName( const QString & groupName );

	/**
	 * Returns the sortorder of the group
	 * @return the sort order
	 */
	unsigned short sortorder( void );

	/**
	 * Sets the sortorder of the group.
	 * @param sortorder the new sortorder
	 */
	void setSortorder( unsigned short sortorder );

	/**
	 * Returns the group ID if this group.
	 * @return the group ID
	 */
	unsigned short groupID( void );

	/**
	 * Returns whether this group is public or not.
	 * @return TRUE if public
	 */
	bool isPublic( void );

	/**
	 * Sets the public property of this group
	 * @param isPublic the new status
	 */
	void setPublic( bool isPublic );

signals:
	/**
	 * This signal is emitted, when this group is modified.
	 */
	void groupModified( LJ::IO::FriendGroup * friendGroup );
	
protected:
	/**
	 * Creates a new FriendGroup object.
	 * @param manager the manager of this object,
	 * @param groupID the group id
	 * @param groupName the name of the group
	 * @param sortorder the sortorder for this group
	 */
	FriendGroup( LJ::IO::FriendGroupsManager * manager, unsigned short groupID, const QString & groupName = 0, unsigned short sortorder = 50 );
	
private:
	class Private;
	Private * d;
	friend class FriendGroupsManager;
};

}
}

#endif
