/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "friendinfomanager.h"
#include "friendinfomanager.moc"

#include "../livejournalaccount.h"

#include <kdebug.h>

#include <qdom.h>
#include <QMap>
#include <QFile>

#include <iostream>

class LJ::IO::FriendInfoManager::Private
{
public:
	Private( void ) :
	friendInfoDocument( QDomDocument( "LJFriendInfo" ) ),
	friendMap( QMap< QString, FriendInfo * >() ),
	friendOfMap( QMap< QString, FriendOfInfo * >() )
	{
		friendInfoDocument.appendChild( friendInfoDocument.createElement( "friendlist" ) );
	}

	QDomDocument friendInfoDocument;
	QMap< QString, FriendInfo * > friendMap;
	QMap< QString, FriendOfInfo * > friendOfMap;
	QString friendsFileName;

	/**
	 * Returns a node representing the given username.
	 * If no such node exists one will be created.
	 * @param userName the user name
	 */
	QDomElement friendNode( const QString & userName ) {
		QDomNode currentNode = friendInfoDocument.firstChild().namedItem( "friend" );
		QDomElement currentElement;
		
		while( !currentNode.isNull() ) {
			if( currentNode.isElement() && currentNode.nodeName() == "friend" ) {
				currentElement = currentNode.toElement();
				if( currentElement.attribute( "user" ) == userName ) {
					return currentElement;
				}
			}

			currentNode = currentNode.nextSibling();
		}

		currentElement = friendInfoDocument.createElement( "friend" );
		currentElement.setAttribute( "user", userName );
		friendInfoDocument.firstChild().appendChild( currentElement );
		return currentElement;
	}

	/**
	 * Returns a node representing the given username.
	 * If no such node exists one will be created.
	 * @param userName the user name
	 */
	QDomElement friendOfNode( const QString & userName ) {
		QDomNode currentNode = friendInfoDocument.firstChild().namedItem( "friendof" );
		QDomElement currentElement;
		
		while( !currentNode.isNull() ) {
			if( currentNode.isElement() && currentNode.nodeName() == "friendof" ) {
				currentElement = currentNode.toElement();
				if( currentElement.attribute( "user" ) == userName ) {
					return currentElement;
				}
			}

			currentNode = currentNode.nextSibling();
		}

		currentElement = friendInfoDocument.createElement( "friendof" );
		currentElement.setAttribute( "user", userName );
		friendInfoDocument.firstChild().appendChild( currentElement );
		return currentElement;
	}
};

LJ::IO::FriendInfoManager::FriendInfoManager( LiveJournalAccount * parentAccount ):
QObject( parentAccount ),
d( new Private() )
{
	d->friendsFileName = parentAccount->dataDirectory() + QString::fromLatin1( "friendinfo.xml" );

	//loading and parsing file
	QFile friendInfoFile( d->friendsFileName );
	if( friendInfoFile.open( QIODevice::ReadOnly ) ) {
		if( !d->friendInfoDocument.setContent( &friendInfoFile ) ) {
			kError() << k_funcinfo << "error parsing friend info file " << friendInfoFile.fileName() << endl;
		}
		friendInfoFile.close();
	}

	//build friend map
	QDomNode currentNode = d->friendInfoDocument.firstChild().namedItem( "friend" );
	QDomElement currentElement;
	QString userName;
		
	while( !currentNode.isNull() ) {
		if( currentNode.isElement() && currentNode.nodeName() == "friend" ) {
			currentElement = currentNode.toElement();
			userName = currentElement.attribute( "user" );
			d->friendMap.insert( userName, new FriendInfo( this, userName ) );
		}
		
		currentNode = currentNode.nextSibling();
	}

	//build friendof map
	currentNode = d->friendInfoDocument.firstChild().namedItem( "friendof" );

	while( !currentNode.isNull() ) {
		if( currentNode.isElement() && currentNode.nodeName() == "friendof" ) {
			currentElement = currentNode.toElement();
			userName = currentElement.attribute( "user" );
			d->friendOfMap.insert( userName, new FriendOfInfo( this, userName ) );
		}
		
		currentNode = currentNode.nextSibling();
	}
}

LJ::IO::FriendInfoManager::~FriendInfoManager( void )
{
	QFile friendInfoFile( d->friendsFileName );
	if( !friendInfoFile.open( QIODevice::WriteOnly ) ) {
		kError() << k_funcinfo << "error opening friend info file " << friendInfoFile.fileName() << endl;
	}

   QTextStream fileStream( &friendInfoFile );
	d->friendInfoDocument.save( fileStream, 3 );
 
   friendInfoFile.close();
	
	delete d;
}

void LJ::IO::FriendInfoManager::setValidFriends( QStringList friendUsers )
{
	if( d->friendMap.isEmpty() ) {
		return;
	}
	
	QStringList currentUsers = d->friendMap.keys();
	FriendInfo * removedFriend;

	for( QStringList::ConstIterator it = currentUsers.begin();
	     it != currentUsers.end();
	     ++it )
	{
		if( !friendUsers.contains( *it ) ) {
			removedFriend = d->friendMap[ *it ];
			d->friendMap.remove( *it );
			emit friendRemoved( removedFriend );
			removedFriend->deleteLater();

			d->friendInfoDocument.firstChild().removeChild( d->friendNode( *it ) );
		}
	}
}

void LJ::IO::FriendInfoManager::setValidFriendOfs( QStringList friendOfUsers )
{
	if( d->friendOfMap.isEmpty() ) {
		return;
	}
	
	QStringList currentUsers = d->friendOfMap.keys();
	FriendOfInfo * removedFriend;

	for( QStringList::ConstIterator it = currentUsers.begin();
	     it != currentUsers.end();
	     ++it )
	{
		if( !friendOfUsers.contains( *it ) ) {
			removedFriend = d->friendOfMap[ *it ];
			d->friendOfMap.remove( *it );
			emit friendOfRemoved( removedFriend );
			removedFriend->deleteLater();

			d->friendInfoDocument.firstChild().removeChild( d->friendOfNode( *it ) );
		}
	}
}

QList< LJ::IO::FriendInfo * > LJ::IO::FriendInfoManager::friends( void ) const
{
	return d->friendMap.values();
}

LJ::IO::FriendInfo * LJ::IO::FriendInfoManager::friendInfo( const QString & userName )
{
	if( d->friendMap.contains( userName ) ) {
		return d->friendMap[ userName ];
	}

	FriendInfo * newFriend = new FriendInfo( this, userName );
	d->friendMap.insert( userName, newFriend );
	emit friendAdded( newFriend );
	return newFriend;
}

QList< LJ::IO::FriendOfInfo * > LJ::IO::FriendInfoManager::friendOfs( void ) const
{
	return d->friendOfMap.values();
}

LJ::IO::FriendOfInfo * LJ::IO::FriendInfoManager::friendOfInfo( const QString & userName )
{
	if( d->friendOfMap.contains( userName ) ) {
		return d->friendOfMap[ userName ];
	}

	FriendOfInfo * newFriendOf = new FriendOfInfo( this, userName );
	d->friendOfMap.insert( userName, newFriendOf );
	emit friendOfAdded( newFriendOf );
	return newFriendOf;
}

//--------------
//FriendInfoBase
//--------------

class LJ::IO::FriendInfoBase::Private
{
public:
	Private( void ) {}

	QDomElement node;
};

LJ::IO::FriendInfoBase::FriendInfoBase( LJ::IO::FriendInfoManager * manager, const QString & userName, Type type ) :
QObject( manager ),
d( new Private() )
{
	switch( type ) {
	case Friend:
		d->node = manager->d->friendNode( userName );
		break;
	case FriendOf:
		d->node = manager->d->friendOfNode( userName );
		break;
	}
}

LJ::IO::FriendInfoBase::~FriendInfoBase( void )
{
	delete d;
}

void LJ::IO::FriendInfoBase::setAttribute( const QString & name, const QString & value )
{
	if( attribute( name) == value ) {
		return;
	}
	
	d->node.setAttribute( name, value );
	emit modified( this );
}

QString LJ::IO::FriendInfoBase::attribute( const QString & name, const QString & defaultValue ) const
{
	return d->node.attribute( name, defaultValue );
}

QString LJ::IO::FriendInfoBase::userName( void ) const
{
	return attribute( "user" );
}

QString LJ::IO::FriendInfoBase::realName( void ) const
{
	return attribute( "realname" );
}

void LJ::IO::FriendInfoBase::setRealName( const QString & realName )
{
	setAttribute( "realname", realName );
}

QColor LJ::IO::FriendInfoBase::background( void ) const
{
	return QColor( attribute( "bg" ) );
}

void LJ::IO::FriendInfoBase::setBackground( const QColor & color )
{
	setAttribute( "bg", color.name() );
}

QColor LJ::IO::FriendInfoBase::foreground( void ) const
{
	return QColor( attribute( "fg" ) );
}

void LJ::IO::FriendInfoBase::setForeground( const QColor & color )
{
	setAttribute( "fg", color.name() );
}

LJ::IO::FriendInfoBase::FriendType LJ::IO::FriendInfoBase::type( void ) const
{
	QString value = attribute( "type" );

	if( value == "community" ) {
		return TypeCommunity;
	}
	else if( value == "syndicated" ) {
		return TypeSyndicated;
	}

	return TypeUser;
}

void LJ::IO::FriendInfoBase::setType( FriendType type )
{
	QString value;

	switch( type ) {
	case TypeUser:
		value = "user";
		break;
	case TypeCommunity:
		value = "community";
		break;
	case TypeSyndicated:
		value = "syndicated";
		break;
	}

	setAttribute( "type", value );
}

LJ::IO::FriendInfoBase::FriendStatus LJ::IO::FriendInfoBase::status( void ) const
{
	QString value = attribute( "status" );

	if( value == "deleted" ) {
		return StatusDeleted;
	}
	else if( value == "suspended" ) {
		return StatusSuspended;
	}
	else if( value == "purged" ) {
		return StatusPurged;
	}	

	return StatusNormal;
}

void LJ::IO::FriendInfoBase::setStatus( FriendStatus status )
{
	QString value;

	switch( status ) {
	case StatusNormal:
		value = "normal";
		break;
	case StatusDeleted:
		value = "deleted";
		break;
	case StatusPurged:
		value = "purged";
		break;
	case StatusSuspended:
		value = "suspended";
		break;
	}

	setAttribute( "status", value );
}

//------------
//FriendOfInfo
//------------
LJ::IO::FriendOfInfo::FriendOfInfo( LJ::IO::FriendInfoManager * manager, const QString & userName ) :
FriendInfoBase( manager, userName, FriendOf )
{
}

LJ::IO::FriendOfInfo::~FriendOfInfo( void )
{
}

//----------
//FriendInfo
//----------
LJ::IO::FriendInfo::FriendInfo( LJ::IO::FriendInfoManager * manager, const QString & userName ) :
FriendInfoBase( manager, userName, Friend )
{
}

LJ::IO::FriendInfo::~FriendInfo( void )
{
}

QDate LJ::IO::FriendInfo::birthday( void ) const
{
	return QDate::fromString( attribute( "birthday"), Qt::ISODate );
}

void LJ::IO::FriendInfo::setBirthday( QDate birthday )
{
	setAttribute( "birthday", birthday.toString( Qt::ISODate ) );
}

unsigned long LJ::IO::FriendInfo::groupmask( void ) const
{
	return attribute( "groupmask" ).toULong( );
}

void LJ::IO::FriendInfo::setGroupmask( unsigned long mask )
{
	setAttribute( "groupmask", QString::number( mask ) );
}
