/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_IOFRIENDINFOMANAGER_H
#define LJ_IOFRIENDINFOMANAGER_H

#include <QObject>
#include <QList>
#include <QColor>
#include <QDateTime>

class LiveJournalAccount;
namespace LJ {
namespace IO {

class FriendInfo;
class FriendOfInfo;

/**
 * @short Manages friend and friendof info for a single LiveJournal account.
 *
 * Manages friend and friendof info for a single LiveJournal account.
 * @author Martin Mueller <orvio@orvio.de>
*/
class FriendInfoManager : public QObject
{
Q_OBJECT
public:
	/**
	 * Creates a new FriendInfoManager.
	 * @param parentAccount the account this manager belongs to
	 */
	FriendInfoManager( LiveJournalAccount * parentAccount = 0 );
	virtual ~FriendInfoManager( void );

	/**
	 * Removes all friends whose usernames are not in @a friendUsers.
	 * @param friendUsers a list of usernames currently known to belong to actual friends
	 */
	void setValidFriends( QStringList friendUsers );

	/**
	 * Removes all friendOfs whose usernames are not in @a friendOfUsers.
	 * @param friendOfUsers a list of usernames currently known to belong to actual friendOfs
	 */
	void setValidFriendOfs( QStringList friendOfUsers );

	/**
	 * Returns a list of current friends.
	 * @return the friend groups
	 */
	QList< LJ::IO::FriendInfo * > friends( void ) const;

	/**
	 * Returns the friend with the given user name.
	 * If no such friend exists, a new one will be created.
	 * @param userName fetch the friend with this user name
	 */
	LJ::IO::FriendInfo * friendInfo( const QString & userName );

	/**
	 * Returns a list of current friendOfs.
	 * @return the friend groups
	 */
	QList< LJ::IO::FriendOfInfo * > friendOfs( void ) const;

	/**
	 * Returns the friendOf with the given user name.
	 * If no such friendOf exists, a new one will be created.
	 * @param userName fetch the friendOf with this user name
	 */
	LJ::IO::FriendOfInfo * friendOfInfo( const QString & userName );

signals:
	/**
	 * This signal is emitted when a new friend is added
	 * @param friendInfo the new friend
	 */
	void friendAdded( LJ::IO::FriendInfo * friendInfo );

	/**
	 * This signal is emitted when a friend is removed
	 * @param friendInfo the removed friend
	 */
	void friendRemoved( LJ::IO::FriendInfo * friendInfo );

	/**
	 * This signal is emitted when a new friendOf is added
	 * @param friendInfo the new friendOf
	 */
	void friendOfAdded( LJ::IO::FriendOfInfo * friendInfo );

	/**
	 * This signal is emitted when a friendOf is removed
	 * @param friendInfo the removed friendOf
	 */
	void friendOfRemoved( LJ::IO::FriendOfInfo * friendInfo );

private:
	class Private;
	Private * d;
	friend class FriendInfoBase;
};

/**
 * @short Base class for FriendInfo and FriendInfo.
 *
 * This class is an interface to the internal storage of friend information.
 * It also provides access to attributes common to friends and friendOfs.
 * @author Martin Mueller <orvio@orvio.de>
 */
class FriendInfoBase : public QObject
{
Q_OBJECT
public:
	/**
	 * Identifies the type of the friend
	 */
	enum FriendType {
		TypeUser = 0, ///< Normal user
		TypeCommunity = 1, ///< Community
		TypeSyndicated = 2 ///< RSS Feed
	};

	/**
	 * Identifies the status of the friend.
	 * StatusNormal is a normal, active account. The protocol documentation
	 * does not explain the differences between the other three states.
	 */
	enum FriendStatus {
		StatusNormal = 0,
		StatusDeleted = 1,
		StatusSuspended = 2,
		StatusPurged = 3,
	};

	/**
	 * Destructor
	 */
	virtual ~FriendInfoBase( void );

	/**
	 * Return the user name
	 * @return the user name
	 */
	QString userName( void ) const;

	/**
	 * Returns the real name
	 * @return the real name
	 */
	QString realName( void ) const;

	/**
	 * Sets the real name
	 * @param realName the new real name
	 */
	void setRealName( const QString & realName );

	/**
	 * Returns the background color
	 * @param the background color
	 */
	QColor background( void ) const;

	/**
	 * Set the background color
	 * @param color the color
	 */
	void setBackground( const QColor & color );

	/**
	 * Returns the foreground color
	 * @param the foreground color
	 */
	QColor foreground( void ) const;

	/**
	 * Set the foreground color
	 * @param color the color
	 */
	void setForeground( const QColor & color );

	/**
	 * Returns the type of the friend
	 * @return the type of the friend
	 */
	FriendType type( void ) const;

	/**
	 * Sets the type of the friend
	 * @param type the new type
	 */
	void setType( FriendType type );

	/**
	 * Returns the status of the friend
	 * @return the status
	 */
	FriendStatus status( void ) const;

	/**
	 * Sets the status of the friend
	 * @param status the new status
	 */
	void setStatus( FriendStatus status );

signals:
	/**
	 * This signal is emitted when this object is modified
	 * @param friendInfo this
	 */
	void modified( LJ::IO::FriendInfoBase * friendInfo );
	
protected:

	/**
	 * Identiefies the node type to use for the derived class
	 */
	enum Type {
		Friend,
		FriendOf
	};

	/**
	 * Creates a new FriendInfoBase
	 * @param manager the parent FriendInfoManager
	 * @param userName user name of the friend or friendOf
	 * @param type the node type to use
	 */
	FriendInfoBase( LJ::IO::FriendInfoManager * manager, const QString & userName, Type type );

	/**
	 * Sets a property
	 * @param name property name
	 * @param value the new value
	 */
	void setAttribute( const QString & name, const QString & value );

	/**
	 * Returns a property. If no property with @a name can be found @a defaultValue
	 * is returned
	 * @param name property name
	 * @param defaultValue the default value if the property cannot be found
	 */
	QString attribute( const QString & name, const QString & defaultValue = QString::null ) const;

private:
	class Private;
	Private * d;
};

class FriendOfInfo : public FriendInfoBase
{
Q_OBJECT
public:
	FriendOfInfo( LJ::IO::FriendInfoManager * manager, const QString & userName );
	virtual ~FriendOfInfo( void );
};

class FriendInfo : public FriendInfoBase
{
Q_OBJECT
public:
	FriendInfo( LJ::IO::FriendInfoManager * manager, const QString & userName);
	virtual ~FriendInfo( void );

	/**
	 * Returns the birthday of the friend
	 * @return the birthday
	 */
	QDate birthday( void ) const;

	/**
	 * Sets the birthday of the friend
	 * @param birthday the birthday
	 */
	void setBirthday( QDate birthday );

	/**
	 * Returns the groupmask of this friend
	 * @return the groupmask
	 */
	unsigned long groupmask( void ) const;

	/**
	 * Sets the groupmask of this friend.
	 * @param mask the group mask
	 */
	void setGroupmask( unsigned long mask );
};

}

}

#endif
