/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "geteventsjob.h"
#include "geteventsjob.moc"

#include "../livejournalblog.h"
#include "../livejournalaccount.h"
#include "../ljentry.h"

#include <blokkalio/util.h>

#include <kdebug.h>

#include <QDateTime>

class LJ::IO::GetEventsJob::Private {
public:
	Private( LiveJournalBlog * blog, SelectType type ):
	blog( blog ),
	type( type ),
	eventsCount( 0 ){}

	LiveJournalBlog * const blog;
	const SelectType type;

	QDate date;
	QString id;
	unsigned int n;

	unsigned int eventsCount;
	//maps index in response to properties
	QMap<int, QMap<QByteArray,QByteArray> > propertyMap;

	//maps indexids to metaproperties
	QMap<unsigned int, QMap<QByteArray,QByteArray> > metaPropertyMap;
	
};

LJ::IO::GetEventsJob::GetEventsJob( LiveJournalBlog * blog, const QDate & date, QObject * parent ):
Job( dynamic_cast<LiveJournalAccount*>( blog->account() ), parent ),
d( new Private( blog, TypeDay ) )
{
	d->date = date;
}

LJ::IO::GetEventsJob::GetEventsJob( LiveJournalBlog * blog, const QString & id, QObject * parent ):
Job( dynamic_cast<LiveJournalAccount*>( blog->account() ), parent ),
d( new Private( blog, TypeOne ) )
{
	d->id = id;
}

LJ::IO::GetEventsJob::GetEventsJob( LiveJournalBlog * blog, unsigned int n, QObject * parent ):
Job( dynamic_cast<LiveJournalAccount*>( blog->account() ), parent ),
d( new Private( blog, TypeLastN ) )
{
	d->n = n;
}

LJ::IO::GetEventsJob::~GetEventsJob( void )
{
	delete d;
}

QByteArray LJ::IO::GetEventsJob::postData( void ) const
{
	QByteArray postData = QString::fromLatin1( "mode=getevents&lineendings=unix&usejournal=%1" ).arg( d->blog->id() ).toAscii();
	switch( d->type ) {
	case TypeDay:
		postData += QString::fromLatin1( "&selecttype=day&year=%1&month=%2&day=%3" ).arg( d->date.year() ).arg(d->date.month() ).arg( d->date.day() ).toAscii();
		break;
	case TypeOne:
		postData += "&selecttype=one&itemid=" + QByteArray::number( d->id.toUInt() );
		break;
	case TypeLastN:
		postData += "&selecttype=lastn&howmany=" + QByteArray::number( d->n );
	};
	return postData;
}

QList< LJ::Entry * > LJ::IO::GetEventsJob::entries( void ) const
{
	if( d->eventsCount == 0 ) {
		buildPropertyMaps();
	}
	
	QList< LJ::Entry *> entries;
	LJ::Entry * entry;
	for( unsigned int i = 1; i <= d->eventsCount; ++i ) {
		entry = readEntry( i );
		if( entry ) {
			entries.append( entry );
		}
		else {
			kError() << "could not read entry with index " << i << endl;
		}
	}

	return entries;
}

LJ::Entry * LJ::IO::GetEventsJob::readEntry( int index ) const
{
	if( d->propertyMap[index]["itemid"].isEmpty() ) {
		kError() << "no itemid for entry with index " << index << endl;
		return 0;
	}
	const unsigned int id = d->propertyMap[index]["itemid"].toUInt();
	if( !id > 0 ) {
		kError() << "invalid itemid (" << id << ") for entry with index " << index << endl;
		return 0;
	}

	LJ::Entry * const entry = new LJ::Entry( d->blog,
	                                         d->propertyMap[index]["poster"],
	                                         id,
	                                         QString( d->propertyMap[index]["url"] ) );
	QByteArray & value = d->propertyMap[index]["eventtime"];
	entry->setDate( QDateTime( QDate( value.mid( 0,4).toUInt(),
	                                  value.mid( 5,2 ).toUInt(),
	                                  value.mid( 8,2 ).toUInt() ),
	                           QTime( value.mid( 11,2 ).toUInt(),
	                                  value.mid( 14,2 ).toUInt(),
	                                  value.mid( 17, 2 ).toUInt() ) ) );
	entry->setText( Blokkal::Io::Util::decodeUtf8Data( d->propertyMap[index]["event"], TRUE ) );

	//security
	value = d->propertyMap[index]["security"];
	if( value == "private" ) {
		entry->setSecurity( LJ::Entry::PrivateLevel );
	}
	else if( value == "usemask" ) {
		unsigned long allowMask = d->propertyMap[index]["allowmask"].toULong();
		entry->setAllowMask( allowMask );
		if( allowMask == 1 ) {
			entry->setSecurity( LJ::Entry::FriendsOnlyLevel );
		}
		else {
			entry->setSecurity( LJ::Entry::CustomLevel );
		}
	}

	entry->setSubject( Blokkal::Io::Util::decodeUtf8Data( d->propertyMap[index]["subject"], TRUE ) );
	
	entry->setAnum( d->propertyMap[index]["anum"].toUInt() );

	//meta properties
	for( QMap<QByteArray,QByteArray>::ConstIterator it = d->metaPropertyMap[id].begin();
	     it != d->metaPropertyMap[id].end();
	     ++it )
	{
		value = it.value();
		if( it.key() == "current_music" ) {
			entry->setCurrentMusic( Blokkal::Io::Util::decodeUtf8Data( value, TRUE ) );
		}
		else if( it.key() ==  "current_mood" ) {
			entry->setCurrentMood( Blokkal::Io::Util::decodeUtf8Data( value, TRUE ) );
		}
		else if( it.key() == "current_moodid" ) {
			entry->setCurrentMoodID( value.toUInt() );
		}
		else if( it.key() == "picture_keyword" ) {
			entry->setUserIcon( Blokkal::Io::Util::decodeUtf8Data( value, TRUE ) );
		}
		else if( it.key() == "current_location" ) {
			entry->setCurrentLocation( Blokkal::Io::Util::decodeUtf8Data( value, TRUE ) );
		}
		else if( it.key() == "opt_backdated" ) {
			entry->setBackdated( value.toInt() > 0 );
		}
		else if( it.key() == "opt_preformatted" ) {
			entry->setPreformatted( value.toInt() > 0 );
		}
		else if( it.key() == "opt_noemail" && value.toInt() > 0 ) {
			if( entry->commentSettings() != LJ::Entry::CommentsDisabled ) {
				entry->setCommentSettings( LJ::Entry::CommentsNoEmail );
			}
		}
		else if( it.key() == "opt_nocomments") {
			if( value.toInt() > 0 ) {
				entry->setCommentSettings( LJ::Entry::CommentsDisabled );
			}
			else if( entry->commentSettings() != LJ::Entry::CommentsNoEmail ) {
				entry->setCommentSettings( LJ::Entry::CommentsEnabled );
			}
		}
		else if( it.key() == "adult_content" ) {
			if( value == "none") {
				entry->setAdultContent( Entry::AdultNone );
			}
			else if( value == "concepts" ) {
				entry->setAdultContent( Entry::AdultConcepts );
			}
			else if( value == "explicit" ) {
				entry->setAdultContent( Entry::AdultExplicit );
			}
		}
		else if( it.key() == "opt_screening" || it.key() == "opt_whoscreened") {
			switch( value.at( 0 ) ) {
			case 'A':
				entry->setScreening( LJ::Entry::ScreenAll );
				break;
			case 'R':
				entry->setScreening( LJ::Entry::ScreenAnonymous );
				break;
			case 'F':
				entry->setScreening( LJ::Entry::ScreenNonFriends );
				break;
			case 'N':
				entry->setScreening( LJ::Entry::ScreenNone );
				break;
			}
		}
		else if( it.key() == "revnum" ) {
			entry->setRevnum( value.toUInt() );
		}
		else if( it.key() == "revtime" ) {
			entry->setRevtime( value.toInt() );
		}
		else if( it.key() == "taglist" ) {
			const QString tags = Blokkal::Io::Util::decodeUtf8Data( value, TRUE );
			entry->setCategories( tags.split( ',', QString::SkipEmptyParts) );
		}
	}

	return entry;
}

void LJ::IO::GetEventsJob::buildPropertyMaps( void ) const
{
	QMap<int,unsigned int> metaIndexItemIdMap;
	QMap<int,QByteArray> metaIndexKeyMap;
	QMap<int,QByteArray> metaIndexValueMap;
	
	int index = 0;
	int oldIndex = 0;
	unsigned int propCount = 0;
	while( ( index = responseBuffer().indexOf( '\n', oldIndex ) ) > -1 )
	{
		const QByteArray key = responseBuffer().mid( oldIndex, index-oldIndex );
		++index;	
		oldIndex = index;
		index = responseBuffer().indexOf( '\n', oldIndex /*+ key.length() - 2*/ );
		if( KDE_ISUNLIKELY( index < 0 ) ) {
			index = responseBuffer().length()-1;
		}
		const QByteArray value = responseBuffer().mid( oldIndex, index-oldIndex );
		++index;
		oldIndex = index;

		if( key.startsWith( "events_" ) ) {
			const int secondDash = key.indexOf( '_', 7 );
			if( KDE_ISUNLIKELY( secondDash < 0 ) ) {
				if( KDE_ISLIKELY( key == "events_count" ) ) {
					//this must be "events_count"
					d->eventsCount = value.toUInt();
				}
				continue;
			}
			d->propertyMap[key.mid( 7, secondDash - 7 ).toInt()][key.mid( secondDash + 1 )] = value;
		}
		else if( key.startsWith( "prop_" ) ) {
			const int secondDash = key.indexOf( '_', 6 );
			if( KDE_ISUNLIKELY( secondDash < 0  ) ) {
				if( KDE_ISLIKELY( key == "prop_count" ) ) {
					//this must be "prop_count"
					propCount = value.toUInt();
				}
				continue;
			}
			const QByteArray subkey = key.mid( secondDash + 1 );
			if( subkey == "itemid") {
				metaIndexItemIdMap[key.mid( 5, secondDash - 5 ).toInt()] = value.toUInt();
			}
			else if( subkey == "name" ) {
				metaIndexKeyMap[key.mid( 5, secondDash - 5 ).toInt()] = value;
			}
			else if( subkey == "value" ) {
				metaIndexValueMap[key.mid( 5, secondDash - 5 ).toInt()] = value;
			}
			else {
				kError() << "invalid subkey:" << subkey << endl;
				continue;
			}
		}
	}

	//build meta propertymap
	for( unsigned int i = 1; i <= propCount; ++i ) {
		d->metaPropertyMap[metaIndexItemIdMap[i]][metaIndexKeyMap[i]] = metaIndexValueMap[i];
	}
}
