/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_IOGETEVENTSJOB_H
#define LJ_IOGETEVENTSJOB_H

#include "job.h"

#include <QtCore/QList>

class LiveJournalBlog;
class QDate;

namespace LJ {
class Entry;
namespace IO {

/**
 * @short Retrieves entries from the server.
 *
 * This class queries the server with the "getevents" mode to retrieve
 * one or more entries from the server, depending on the way it was constructed.
 * @author Martin Mueller <orvio@orvio.de>
*/
class GetEventsJob : public LJ::IO::Job
{
Q_OBJECT
public:
	/**
	 * Creates a new GetEventsJob that retrieves all entries of a specific @a date.
	 * @param blog get an entry of this blog
	 * @param date get all entries posted on this day
	 * @param parent parent object
	 */
	GetEventsJob( LiveJournalBlog * blog, const QDate & date, QObject * parent = 0 );

	/**
	 * Creates a new GetEventsJob that retrieves an entries with a specific @a id.
	 * @param blog get an entry of this blog
	 * @param id get the entry with this id
	 * @param parent parent object
	 */
	GetEventsJob( LiveJournalBlog * blog, const QString & id, QObject * parent = 0 );
	
	/**
	 * Creates a new GetEventsJob that retrieves the last @a n entries
	 * @param blog get an entry of this blog
	 * @param n how many entries to fetch
	 * @param parent parent object
	 */
	GetEventsJob( LiveJournalBlog * blog, unsigned int n, QObject * parent = 0 );
	virtual ~GetEventsJob( void );

	/**
	 * Returns all entries received by this job.
	 * You are responsible for deleting the objects.
	 * @note Each call of this method will result in a new set of objects
	 * @return all entries received by this job
	 */
	QList< LJ::Entry * > entries( void ) const;

protected:
	/**
	 * This enumeration is used to identify the mode this job should work with.
	 * The names corresponds to the legal values of the "selecttype" setting
	 * described in the protocol documentation.
	 */
	enum SelectType {
		TypeDay, ///< Get all entries of a specific day
		TypeOne, ///< Get just on entry with a specific id
		TypeLastN, ///< Get the last n entries
	};

	/**
	 * Returns the job specific postdata.
	 * @return the job specific postdata
	 */
	virtual QByteArray postData( void ) const;

	/**
	 * Reads the entry with index @a index and returns it.
	 * The smallest legal index is 1. You can get the number of
	 * received entries by reading the "events_count" key.
	 * If some error occured, 0 is returned.
	 * @note Each call of this method results in a new object.
	 * @param index read the entry with this index
	 * @return the entry, or 0
	 */
	LJ::Entry * readEntry( int index ) const;

private:
	void buildPropertyMaps( void ) const;
	class Private;
	Private * const d;
};

}

}

#endif
