/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mller                            *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "job.h"
#include "job.moc"

#include <config.h>

#include "../livejournalaccount.h"

#include <blokkalio/util.h>

#include <kdebug.h>
#include <kcodecs.h>
#include <kio/job.h>
#include <kurl.h>
#include <klocale.h>

#include <QRegExp>

class LJ::IO::Job::Private {
public:
	Private( const LiveJournalAccount * account ) :
	account( account ){}

	QString challenge;
	QByteArray responseBuffer;
	const LiveJournalAccount * const account;
};

LJ::IO::Job::Job( const LiveJournalAccount * account, QObject * parent ) :
Blokkal::Io::Job( parent ),
d( new Private( account ) )
{
	if( d->account->useChallengeResponseAuth() ) {
		createIoJob( QByteArray( "mode=getchallenge" )
		             + "&clientversion=KDE-Blokkal/" + VERSION
		             + "&ver=1&user=" + QUrl::toPercentEncoding( d->account->userName() ) );
	}
}

LJ::IO::Job::~Job( void )
{
	delete d;
}

void LJ::IO::Job::start( void )
{
	if( !d->account->useChallengeResponseAuth() ) {
		createIoJob( postData()
		             + "&user=" + QUrl::toPercentEncoding( d->account->userName() )
		             + "&hpassword=" + d->account->userMD5Password()
		             + "&clientversion=KDE-Blokkal/" + VERSION + "&ver=1" );
	}

	if( !subjobs().isEmpty() ) {
		subjobs().first()->start();
	}
	else {
		Blokkal::Io::Job::start();
	}
}

QString LJ::IO::Job::getValueOfKey( const QString & key ) const
{
	QString buffer = QString::fromAscii( responseBuffer() );
	int keyOffset = buffer.indexOf( QRegExp( "(\n|^)" + key + "[\r]?\n" ) );
	if( keyOffset < 0 ) {
		return 0;
	}
	
	//Usually we have to add +2 for the new line characters
	//but if the key is in the first line returened
	int newLineCharAdd = 2;
	if( responseBuffer().at( keyOffset ) != '\n' ) {
		newLineCharAdd = 1;
	}
	int valueLength = responseBuffer().indexOf('\n', keyOffset + key.length() + newLineCharAdd ) - keyOffset-key.length() - newLineCharAdd;
	QByteArray value = responseBuffer().mid( keyOffset + key.length() + newLineCharAdd, valueLength );
	return Blokkal::Io::Util::decodeUtf8Data( value, TRUE );
}

void LJ::IO::Job::subjobFinished( KJob * job )
{
	Q_UNUSED( job );
	
	if( !success() && !error() ) {
		setError( -1 ); //TODO maybe invent some error codes
		if( getValueOfKey( "errmsg" ).isEmpty() ) {
			if( responseBuffer().isEmpty() ) {
				setErrorText( i18n( "The server responded with an empty message" ) );
			}
			else {
				setErrorText( i18n( "The server responded with a malformed message." ) );
				kDebug() << responseBuffer() << endl;
			}
		}
		else {
			setErrorText( getValueOfKey( "errmsg" ) );
		}
	}
	
	if( !error() && d->account->useChallengeResponseAuth() ) {
		if( d->challenge.isEmpty() ) {
			//this must be the get challenge job
			setPercent( 0 );
			d->challenge = getValueOfKey( "challenge" );
			//create the real io job
			const QString authResponse = KMD5( d->challenge.toAscii() + d->account->userMD5Password() ).hexDigest();
			createIoJob( postData() +
			             "&user=" + QUrl::toPercentEncoding( d->account->userName() )
			             + "&auth_method=challenge" + "&auth_challenge=" + QUrl::toPercentEncoding( d->challenge )
			             + "&auth_response=" + QUrl::toPercentEncoding( authResponse )
			             + "&clientversion=KDE-Blokkal/" + VERSION + "&ver=1" );
		}
		else if( !d->challenge.isEmpty() ) {
			//this must be the real io job
			ioJobFinished();
		}
	}

	if( !subjobs().isEmpty() ) {
		d->responseBuffer = QByteArray();
	}

	if( !error() && !subjobs().isEmpty() ) {
		subjobs().first()->start();
	}

}

const QByteArray & LJ::IO::Job::responseBuffer( void ) const
{
	return d->responseBuffer;
}

void LJ::IO::Job::slotDataArrived( KIO::Job * job, const QByteArray & data )
{
	if ( job && data.size() > 0 ) {
		d->responseBuffer.append( data );
	}
}

bool LJ::IO::Job::success( void ) const
{
	QString success = getValueOfKey( "success" );
	return ( !success.isEmpty() && success == "OK" );
}

const LiveJournalAccount * LJ::IO::Job::account( void ) const
{
	return d->account;
}

void LJ::IO::Job::createIoJob( const QByteArray & postData )
{
	KIO::TransferJob * job = KIO::http_post( d->account->connectAddress(),
	                                         postData,
	                                         KIO::HideProgressInfo );

	job->addMetaData( "UserAgent", QString::fromLatin1( "KDE-Blokkal/%1 LiveJournal Plugin").arg( VERSION ) );
	job->addMetaData( "content-type", "Content-Type: application/x-www-form-urlencoded" );
	job->addMetaData( "ConnectTimeout", "300" );
	if( d->account->useFastServer() ) {
		job->addMetaData( QString( "Cookie" ), QString( "Cookie: ljfastserver=1" ) );
	}
	
	connect( job,
	         SIGNAL( data( KIO::Job *, const QByteArray & ) ),
	         SLOT( slotDataArrived( KIO::Job *, const QByteArray & ) ) );
	addSubjob( job );
}

void LJ::IO::Job::ioJobFinished( void )
{
}
