/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mller                            *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_IOJOB_H
#define LJ_IOJOB_H

#include <blokkalio/jobs.h>

namespace KIO {
class Job;
}

class LiveJournalAccount;
namespace LJ{
namespace IO {
/**
 * @short Base class for all operations on the server.
 * 
 * Base class for all operations on the server. It provides some
 * functions needed by all subclasses like getValueOfKey().
 * This class handles the request authentication.
 *
 * @author Martin Mller <orvio@orvio.de>
*/
class Job : public Blokkal::Io::Job
{
Q_OBJECT
public:
	/**
	 * Creates a new Job.
	 * @param account use the authentication information from this account
	 * @param parent the owning object
	 */
	Job( const LiveJournalAccount * account, QObject * parent = 0 );

	/**
	 * Deletes the object.
	 */
	virtual ~Job( void );

	/**
	 * Starts the first subjob.
	 */
	virtual void start( void );

	/**
	 * Searches for a string in the return data and returns the corresponding value.
	 * Returns an empty string if the key was not found.
	 * @param key search for this key
	 * @return the value of the first key or QString::null
	 */
	QString getValueOfKey( const QString &key ) const;

	/**
	* @return TRUE if the key "success" was found in the returned data and has the value "OK".
	*/
	bool success( void ) const;

	/**
	 * Returns the account this job works for.
	 * Jobs get the required authentication information
	 * from the account.
	 * @return the context account
	 */
	const LiveJournalAccount * account( void ) const;

protected:
	/**
	 * This method is called when a subjob has finished.
	 * Reimplemented to start the next subjob.
	 */
	virtual void subjobFinished( KJob * job );

	/**
	 * This method is called when the io job has finished without io
	 * error and the success value indicates a successfull query
	 * The default implementation does nothing.
	 */
	virtual void ioJobFinished( void );
	
	/**
	 * Override this method in subclasses. You don't need to include any client identification,
	 * version or authentication data. Just stick to the operation related data. This class
	 * handles averything else. Example return string: "mode=getfriendgroups".
	 * The returned string must be url form encoded.
	 */
	virtual QByteArray postData( void ) const = 0;

	/**
	 * Returns the response buffer. This is the raw data as received from the server.
	 * Expect UTF-8 encoded form data. If you want to extract a single value,
	 * use getValueOfKey() instead. That method converts the data as well.
	 * You can use decodeUTF8FormData() to decode the raw data in the response
	 * buffer.
	 * @return the response buffer
	 * @see getValueOfKey()
	 * @see decodeUTF8FormData()
	 */
	const QByteArray & responseBuffer( void ) const;

protected slots:

	/**
	 * Stores the returned data in the response buffer.
	 * You should not need to override this method.
	 * @see responseBuffer()
	 * @param job the worker job
	 * @param data the new data
	*/
	virtual void slotDataArrived( KIO::Job *job, const QByteArray & data );

private:
	void createIoJob( const QByteArray & postData );
	class Private;
	Private * const d;
};

}
}
#endif
