/***************************************************************************
 *   Copyright (C) 2006 by Martin Mller                                   *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "moodmanager.h"
#include "moodmanager.moc"

#include "../livejournalaccount.h"

#include <kdebug.h>
#include <kstandarddirs.h>

#include <qdom.h>
#include <qfile.h>
#include <qdir.h>
#include <qmap.h>
#include <qtimer.h>

#include <iostream>

/**
 * @short This class is an implementation detail of @ref LJ::IO::MoodManager.
 *
 * This class is an implementation detail of @ref LJ::IO::MoodManager.
 * @author Martin Mller <orvio@orvio.de>
 */
class LJ::IO::MoodManager::Private
{
public:
	/**
	 * Constructs a new object.
	 */
	Private( void ) :
	moodsDocument( QDomDocument( "LJMoods" ) ),
	moodMap( QMap< unsigned int, Mood * >() )
	{
		moodListElement = moodsDocument.createElement( "moodlist" );
		moodListElement.setAttribute( "maxid", (int) 0 );
		moodsDocument.appendChild( moodListElement );
	}

	/**
	 * Returns the document element for the given mood.
	 * @param moodID the mood id
	 */
	QDomElement moodNode( unsigned int moodID ) {
		QDomElement currentElement;
		for( QDomNode currentNode = moodListElement.firstChild();
		     !currentNode.isNull();
		     currentNode = currentNode.nextSibling() )
		{
			if( currentNode.isElement() && currentNode.nodeName() == "mood" ) {
				currentElement = currentNode.toElement();
				if( currentElement.attribute( "id" ).toUInt() == moodID ) {
					return currentElement;
				}
			}
		}

		currentElement = moodsDocument.createElement( "mood" );
		currentElement.setAttribute( "id", moodID );
		moodListElement.appendChild( currentElement );
		return currentElement;
	}

	QDomDocument moodsDocument;
	QDomElement moodListElement;
	QString moodFileName;
	QMap< unsigned int, Mood * > moodMap;
};

LJ::IO::MoodManager::MoodManager( LiveJournalAccount * parentAccount ) :
QObject( parentAccount ),
d( new Private() )
{
	d->moodFileName = parentAccount->dataDirectory() + QString::fromLatin1( "moods.xml" );

	//loading and parsing file
	QFile moodsFile( d->moodFileName );
	if( moodsFile.open( QIODevice::ReadOnly ) ) {
		if( !d->moodsDocument.setContent( &moodsFile ) ) {
			kError() << "error parsing moods file " << moodsFile.fileName() << endl;
		}
		d->moodListElement = d->moodsDocument.firstChild().toElement();

		if( d->moodListElement.isNull() ) {
			d->moodListElement = d->moodsDocument.createElement( "moodlist" );
			d->moodListElement.setAttribute( "maxid", (int) 0 );
			d->moodsDocument.appendChild( d->moodListElement );
		}
		moodsFile.close();
	}

	QDomElement currentElement;
	unsigned int currentMoodID;
	for( QDomNode currentNode = d->moodListElement.firstChild();
	     !currentNode.isNull();
	     currentNode = currentNode.nextSibling() )
	{
		if( currentNode.isElement() && currentNode.nodeName() == "mood" ) {
			currentElement = currentNode.toElement();
			currentMoodID = currentElement.attribute( "id" ).toUInt();
			d->moodMap.insert( currentMoodID, new Mood( this, currentMoodID ) );
		}
	}
	
}

LJ::IO::MoodManager::~MoodManager( void )
{
	QFile moodsFile( d->moodFileName );
	if( !moodsFile.open( QIODevice::WriteOnly ) ) {
		kError() << "error opening moods file " << moodsFile.fileName() << endl;
	}

   QTextStream fileStream( &moodsFile );
	d->moodsDocument.save( fileStream, 3 );
 
   moodsFile.close();
	
	delete d;
}

unsigned int LJ::IO::MoodManager::maxMoodID( void )
{
	return d->moodListElement.attribute( "maxid" ).toUInt();
}

QList< LJ::IO::Mood * > LJ::IO::MoodManager::moods( void )
{
	return d->moodMap.values();
}

LJ::IO::Mood * LJ::IO::MoodManager::mood( unsigned int moodID )
{
	if( d->moodMap.contains( moodID ) ) {
		return d->moodMap[ moodID ];
	}

	return 0;
}

void LJ::IO::MoodManager::addMood( unsigned short moodID, unsigned short parentMood, const QString & moodName )
{
	Mood * newMood = new Mood( this, moodID, parentMood, moodName );
	d->moodMap.insert( moodID, newMood );
	if( d->moodListElement.attribute( "maxid" ).toUInt() < moodID ) {
		d->moodListElement.setAttribute( "maxid", moodID );
	}
	
	emit moodAdded( newMood );
}

/**
 * @short This class is an implementation detail of @ref LJ::IO::Mood.
 *
 * This class is an implementation detail of @ref LJ::IO::Mood.
 * @author Martin Mller <orvio@orvio.de>
 */
class LJ::IO::Mood::Private
{
public:
	/**
	 * Creates a new object.
	 */
	Private( void ) {}

	/**
	 * The mood node element
	 */
	QDomElement moodElement;
};

LJ::IO::Mood::Mood( MoodManager * manager, unsigned int moodID, unsigned int parentMood, const QString & moodName ) :
QObject( manager ),
d( new Private() )
{
	d->moodElement = manager->d->moodNode( moodID );
	d->moodElement.setAttribute( "parent", parentMood );
	d->moodElement.setAttribute( "name", moodName );
}

LJ::IO::Mood::Mood( MoodManager * manager, unsigned int moodID ) :
QObject( manager ),
d( new Private() )
{
	d->moodElement = manager->d->moodNode( moodID );
}

LJ::IO::Mood::~Mood( void )
{
	delete d;
}

unsigned int LJ::IO::Mood::moodID( void ) const
{
	return d->moodElement.attribute( "id" ).toUInt();
}

unsigned int LJ::IO::Mood::parentMood( void ) const
{
	return d->moodElement.attribute( "parent" ).toUInt();
}

QString LJ::IO::Mood::moodName( void ) const
{
	return d->moodElement.attribute( "name" );
}


