/***************************************************************************
 *   Copyright (C) 2006 by Martin Mller                                   *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_IOMOODMANAGER_H
#define LJ_IOMOODMANAGER_H

#include <qobject.h>
#include <qimage.h>

class LiveJournalAccount;
namespace LJ {
namespace IO {
class Mood;
/**
 * @short This class manages moods.
 *
 * This class manages moods as they are returned by the livejournal server.
 * @author Martin Mller <orvio@orvio.de>
*/
class MoodManager : public QObject
{
Q_OBJECT
public:
	/**
	 * Creates a new MoodManager.
	 * @param parentAccount the account this manager belongs to
	 */
	MoodManager( LiveJournalAccount * parentAccount );
	~MoodManager( void );

	/**
	 * Returns the highest ID of the currently known moods.
	 * @return the max mood ID
	 */
	unsigned int maxMoodID( void );

	/**
	 * Returns a list of currently known moods.
	 * @return current mood list
	 */
	QList< LJ::IO::Mood * > moods( void );

	/**
	 * Returns the mood with the given id or 0 if no such mood exists
	 * @return the reqested mood or 0
	 */
	LJ::IO::Mood * mood( unsigned int moodID );

	/**
	 * This method creates a new mood and adds it to the list of currently known moods.
	 * @param moodID the ID of the new mood
	 * @param parentMood the ID of the parent mood
	 * @param moodName the name of the new mood
	 */
	void addMood( unsigned short moodID, unsigned short parentMood, const QString & moodName );

signals:
	/**
	 * This signal is emitted when a mood was added to the list.
	 */
	void moodAdded( LJ::IO::Mood * newMood );

private:
	class Private;
	Private * d;
	friend class Mood;
};

/**
 * @short This class represents a single mood.
 *
 * This class represents a single mood. As the mood list my only grow and moods
 * are never changed once they were received, there is no way to change
 * any properties.
 * @author Martin Mller <orvio@orvio.de>
 */
class Mood : public QObject
{
Q_OBJECT
public:
	~Mood( void );

	/**
	 * Returns the ID of this mood.
	 * @return the mood ID
	 */
	unsigned int moodID( void ) const;

	/**
	 * Returns the ID of the parent mood.
	 * @return the parent's ID
	 */
	unsigned int parentMood( void ) const;

	/**
	 * Returns the name of this mood.
	 * @return the name of this mood
	 */
	QString moodName( void ) const;

protected:
	/**
	 * Constructs a new Mood object.
	 * @param manager parent object
	 * @param moodID mood id
	 * @param parentMood the id of the parent mood
	 * @param moodName name of the mood
	 */
	Mood( MoodManager * manager, unsigned int moodID, unsigned int parentMood, const QString & moodName );

	/**
	 * Constructs a new Mood object, using the data of the mood with id <i>moodID</i>.
	 * @param manager the parent manager
	 * @param moodID use the data of this mood
	 */
	Mood( MoodManager * manager, unsigned int moodID );

private:
	class Private;
	Private * d;
	friend class MoodManager;
};

}
}

#endif
