/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "userpicturemanager.h"
#include "userpicturemanager.moc"

#include "../livejournalaccount.h"
#include <blokkalimagefetcher.h>

#include <kdebug.h>

#include <QFile>
#include <QDir>
#include <QDomElement>

class LJ::IO::UserPictureManager::Private {
public:
	Private( void ) :
	userPictureDocument( QDomDocument( "LJUserPictures" ) ),
	userPictureMap( QMap< QString, UserPicture * >() ),
	defaultPicture( 0 )
	{
		userPictureDocument.appendChild( userPictureDocument.createElement( "userpictures" ) );
	}

	/**
	 * Returns a node representing the given node id.
	 * If no such node exists one will be created.
	 * @param keyword the userpicture keyword
	 */
	QDomElement pictureNode( const QString & keyword ) {
		QDomNode currentNode = userPictureDocument.firstChild().firstChild();
		QDomElement currentElement;
		
		while( !currentNode.isNull() ) {
			if( currentNode.isElement() && currentNode.nodeName() == "keywordpicture" ) {
				currentElement = currentNode.toElement();
				if( currentElement.attribute( "keyword" ) == keyword ) {
					return currentElement;
				}
			}

			currentNode = currentNode.nextSibling();
		}

		currentElement = userPictureDocument.createElement( "keywordpicture" );
		currentElement.setAttribute( "keyword", keyword );
		userPictureDocument.firstChild().appendChild( currentElement );

		return currentElement;
	}

	QDomDocument userPictureDocument;
	QMap< QString, UserPicture * > userPictureMap;
	QString userPictureFileName;
	QString userPictureFilesDir;
	UserPicture * defaultPicture;
};

LJ::IO::UserPictureManager::UserPictureManager( LiveJournalAccount * parentAccount ) :
QObject( parentAccount ),
d( new Private() )
{
	d->userPictureFileName = parentAccount->dataDirectory() + QString::fromLatin1( "userpictures.xml" );
	d->userPictureFilesDir = parentAccount->dataDirectory() + QString::fromLatin1( "userpictures/" );

	//loading and parsing file
	QFile userPictureFile( d->userPictureFileName );
	if( userPictureFile.open( QIODevice::ReadOnly ) ) {
		if( !d->userPictureDocument.setContent( &userPictureFile ) ) {
			kError() << "error parsing user picture file " << userPictureFile.fileName() << endl;
		}
		userPictureFile.close();
	}

	//build map
	QDomNode currentNode = d->userPictureDocument.firstChild().namedItem( "keywordpicture" );
	QDomElement currentElement;
	QString keyword;
		
	while( !currentNode.isNull() ) {
		if( currentNode.isElement() && currentNode.nodeName() == "keywordpicture" ) {
			currentElement = currentNode.toElement();
			keyword = currentElement.attribute( "keyword" );
			d->userPictureMap.insert( keyword, new UserPicture( this, keyword ) );
		}
		
		currentNode = currentNode.nextSibling();
	}
	
	currentNode = d->userPictureDocument.firstChild().namedItem( "defaultpicture" );
	if( !currentNode.isNull() ) {
		d->defaultPicture = new UserPicture( this, 0 );
	}
}

LJ::IO::UserPictureManager::~UserPictureManager( void )
{
	QFile userPictureFile( d->userPictureFileName );
	if( !userPictureFile.open( QIODevice::WriteOnly ) ) {
		kError() << "error opening friend groups file " << userPictureFile.fileName() << endl;
	}

   QTextStream fileStream( &userPictureFile );
	d->userPictureDocument.save( fileStream, 3 );
 
   userPictureFile.close();
	delete d;
}

void LJ::IO::UserPictureManager::setValidKeywords( QList< QString > validKeywords )
{
	QList<QString> currentKeywords = d->userPictureMap.keys();
	UserPicture * removedPicture;
	
	for( QList<QString>::ConstIterator it = currentKeywords.begin();
	     it != currentKeywords.end();
	     ++it )
	{
		if( !validKeywords.contains( *it ) ) {
			removedPicture = d->userPictureMap[ *it ];
			d->userPictureMap.remove( *it );
			emit pictureRemoved( d->userPictureMap[ *it ] );

			d->userPictureDocument.firstChild().removeChild( d->pictureNode( *it ) );
			QFile pictureFile( d->userPictureFilesDir + QString::fromLatin1( "kwp_" ) + removedPicture->keyword() );
			pictureFile.remove();

			removedPicture->deleteLater();
		}
	}
}

QList< LJ::IO::UserPicture * > LJ::IO::UserPictureManager::pictures( void ) const
{
	return d->userPictureMap.values();
}

LJ::IO::UserPicture * LJ::IO::UserPictureManager::picture( const QString & keyword )
{
	if( d->userPictureMap.contains( keyword ) ) {
		return d->userPictureMap[ keyword ];
	}

	UserPicture * newPicture = new UserPicture( this, keyword );
	d->userPictureMap.insert( keyword, newPicture );
	emit pictureAdded( newPicture );
	return newPicture;
}

LJ::IO::UserPicture * LJ::IO::UserPictureManager::picture( const QString & keyword ) const
{
	if( d->userPictureMap.contains( keyword ) ) {
		return d->userPictureMap[ keyword ];
	}

	return 0;
}

LJ::IO::UserPicture * LJ::IO::UserPictureManager::defaultPicture( void ) const
{
	return d->defaultPicture;
}

void LJ::IO::UserPictureManager::setDefaultPictureURL( const QString & url )
{
	if( url.isEmpty() ) {
		UserPicture * pic = d->defaultPicture;
		d->defaultPicture = 0;
		emit pictureRemoved( pic );
		delete d->defaultPicture;
		QDomNode defaultPictureNode = d->userPictureDocument.firstChild().namedItem( "defaultpicture" );
		if( defaultPictureNode.isNull() ) {
			d->userPictureDocument.firstChild().removeChild( defaultPictureNode );
		}
		

	}
	else {
		if( d->defaultPicture ) {
			d->defaultPicture->setPictureURL( url );
		}
		else {
			d->defaultPicture = new UserPicture( this, 0, url );
			emit pictureAdded( d->defaultPicture );
		}
	}
}

//-----------
//UserPicture
//-----------
class LJ::IO::UserPicture::Private
{
public:
	Private( void ) {}

	QDomElement pictureNode;
	QString pictureDirectory;
};

LJ::IO::UserPicture::UserPicture( UserPictureManager * manager, const QString & keyword, const QString & picurl ) :
QObject( manager),
d( new Private() )
{
	if( keyword.isEmpty() ) {
		QDomNode defaultPictureNode = manager->d->userPictureDocument.firstChild().namedItem( "defaultpicture" );
		if( defaultPictureNode.isNull() ) {
			d->pictureNode = manager->d->userPictureDocument.createElement( "defaultpicture" );
			manager->d->userPictureDocument.firstChild().appendChild( d->pictureNode );
		}
		else {
			d->pictureNode = defaultPictureNode.toElement();
		}
		 
	}
	else {
		d->pictureNode = manager->d->pictureNode( keyword );
	}

	if( !picurl.isEmpty() ) {
		setPictureURL( picurl );
	}

	d->pictureDirectory = manager->d->userPictureFilesDir;
}

LJ::IO::UserPicture::~UserPicture( void )
{
	delete d;
}

void LJ::IO::UserPicture::setPictureURL( const QString & url )
{
	if( url == d->pictureNode.attribute( "url" ) ) {
		return;
	}

	else {
		d->pictureNode.setAttribute( "url", url );
		Blokkal::ImageFetcher * fetcher = new Blokkal::ImageFetcher( url, this );
		connect( fetcher, SIGNAL( imageReceived( QImage, const QString & ) ),
		         this, SLOT( saveUserPicture( QImage ) ) );
	}
}

QString LJ::IO::UserPicture::keyword( void ) const
{
	return d->pictureNode.attribute( "keyword" );
}

QImage LJ::IO::UserPicture::picture( void ) const
{
	QDir pictureDir( d->pictureDirectory );
	if( !pictureDir.exists() ) {
		pictureDir.cdUp();
		pictureDir.mkdir( d->pictureDirectory );
		pictureDir.cd( d->pictureDirectory );
	}
	
	if( keyword().isEmpty() ) {
		return QImage( d->pictureDirectory + QString::fromLatin1( "default_picture" ) );
	}
	
	return QImage( d->pictureDirectory + QString::fromLatin1( "kwp_" ) + keyword() );
}

void LJ::IO::UserPicture::saveUserPicture( QImage userPicture )
{
	QDir pictureDir( d->pictureDirectory );
	if( !pictureDir.exists() ) {
		pictureDir.cdUp();
		pictureDir.mkdir( d->pictureDirectory );
		pictureDir.cd( d->pictureDirectory );
	}
	
	if( keyword().isEmpty() ) {
		userPicture.save( d->pictureDirectory + QString::fromLatin1( "default_picture" ), "PNG" );
	}
	else {
		userPicture.save( d->pictureDirectory + QString::fromLatin1( "kwp_" ) + keyword(), "PNG" );
	}
}
