/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_IOUSERPICTUREMANAGER_H
#define LJ_IOUSERPICTUREMANAGER_H

#include <QObject>
#include <QList>
#include <QImage>

class LiveJournalAccount;
namespace LJ {
namespace IO {
class UserPicture;

/**
 * @short This class manages user icons.
 *
 * This class manages user icons.
 * @author Martin Mller <orvio@orvio.de>
*/
class UserPictureManager : public QObject
{
Q_OBJECT
public:
	/**
	 * Creates a new UserPictureManager.
	 * @param parentAccount the account this manager belongs to
	 * @param name object name
	 */
	UserPictureManager( LiveJournalAccount * parentAccount );
	~UserPictureManager( void );

	/**
	 * Removes all currently stored picture keywords that are not
	 * in @a validKeywords.
	 * @param validKeywords the new keyword list
	 */
	void setValidKeywords( QList<QString> validKeywords );

	/**
	 * This method returns all currently known user icons but
	 * the default icon.
	 * @return list of user icons
	 */
	QList< LJ::IO::UserPicture * > pictures( void ) const;

	/**
	 * This method returns the user icon with keyword @a keyword.
	 * If no such icon exists, a new object will be created.
	 * @param keyword the keyword of the desired user icon
	 * @return the requested user icon
	 */
	LJ::IO::UserPicture * picture( const QString & keyword );

	/**
	 * This method returns the user icon with keyword @a keyword.
	 * If no such icon exists, 0 is returned.
	 * @param keyword the keyword of the desired user icon
	 * @return the requested user icon
	 */
	LJ::IO::UserPicture * picture( const QString & keyword ) const;

	/**
	 * This method returns the default user icon.
	 * @return the default user icon, 0 id no default exists
	 */
	LJ::IO::UserPicture * defaultPicture( void ) const;

	/**
	 * This method sets the default picture url.
	 * @param url the new picture url, if 0 the default picture is removed.
	 */
	void setDefaultPictureURL( const QString & url );

Q_SIGNALS:
	/**
	 * This signal is emitted when a user picture is removed
	 */
	void pictureRemoved( LJ::IO::UserPicture * picture );

	/**
	 * This signal is emitted when a user picture is added.
	 */
	void pictureAdded( LJ::IO::UserPicture * picture );

private:
	class Private;
	Private * const d;
	friend class UserPicture;

};

/**
 * @short This class represents a user icon.
 *
 * This class represents a user icon.
 * @author Martin Mller <orvio@orvio.de>
 */
class UserPicture : public QObject
{
Q_OBJECT
public:
	virtual ~UserPicture( void );

	/**
	 * Sets the picture url. If the url is not equal to the previous
	 * picture url, the new user icon will be downloaded and saved.
	 * @param url the url of the user icon
	 */
	void setPictureURL( const QString & url );

	/**
	 * This method returns the local copy of the user icon.
	 * @return the user icon
	 */
	QImage picture( void ) const;

	/**
	 * This method returns the keyword of this user picture.
	 * If QString::null is returned this is the default picture
	 * @return the picture keyword
	 */
	QString keyword( void ) const;

signals:
	/**
	 * This signal is emitted when a new user picture has been received.
	 */
	void pictureChanged( LJ::IO::UserPicture * picture );
	
protected:
	/**
	 * Constructs a new UserPicture object.
	 * @param manager the manager this object belongs to.
	 * @param keyword the keyword of this user icon
	 * @param picurl the url of the user icon image
	 */
	UserPicture( UserPictureManager * manager, const QString & keyword, const QString & picurl = 0 );

protected slots:
	/**
	 * Saves the newly received user picture.
	 * @param userPicture the new usre picture
	 */
	void saveUserPicture( QImage userPicture );

private:
	class Private;
	Private * const d;
	friend class UserPictureManager;
};

}
}
#endif
