/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljmoodselector.h"
#include "ljmoodselector.moc"

#include "../livejournalaccount.h"
#include "../ljio/moodmanager.h"

#include <kcombobox.h>
#include <klineedit.h>
#include <kdialog.h>
#include <klocale.h>

#include <QLayout>

class LJ::UI::MoodSelector::Private {
public:
	Private ( void ) :
	moodList( QList< LJ::IO::Mood * >() ){}

	QList< LJ::IO::Mood * > moodList;

	/**
	 * Inserts a mood at the correct position of
	 * the mood list
	 * @param newMood mood to insert
	 * @return the position at which the new mood was inserted into the list
	 */
	int insertMood( LJ::IO::Mood * newMood ) {
		if( !newMood ) {
			return -1;
		}
		
		if( moodList.isEmpty() ) {
			moodList.append( newMood );
			return 0;
		}

		int position = 0;
		for( QList< LJ::IO::Mood * >::Iterator it = moodList.begin();
		     it != moodList.end();
		     ++it, ++position )
		{
			//if( newMood->moodName() <= (*it)->moodName() ) {
			if( QString::localeAwareCompare( newMood->moodName(), (*it)->moodName() ) <= 0 ) {
				moodList.insert( it, newMood );
				return position;
			}
		}

		moodList.append( newMood );
		return moodList.count() - 1;
	}
	
	KComboBox * moodComboBox;
	KLineEdit * moodNameLineEdit;
};

LJ::UI::MoodSelector::MoodSelector( LiveJournalAccount * account, QWidget * parent ) :
QWidget( parent ),
d( new Private() )
{
	QHBoxLayout * layout = new QHBoxLayout( this );
	layout->setMargin( 0 );

	d->moodComboBox = new KComboBox( this );
	d->moodComboBox->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
	layout->addWidget( d->moodComboBox );
	d->moodNameLineEdit = new KLineEdit( this );
	d->moodNameLineEdit->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
	layout->addWidget( d->moodNameLineEdit );

	//Make it possible to select no mood at all
	d->moodComboBox->addItem( i18n( "(None)" ) );
	
	QList< LJ::IO::Mood * > unsortedMoods = account->moodManager()->moods();
	if( unsortedMoods.isEmpty() ) {
		return;
	}

	//sort moodlist
	for( QList< LJ::IO::Mood * >::ConstIterator it = unsortedMoods.begin();
	     it != unsortedMoods.end();
	     ++it )
	{
		d->insertMood( *it );
	}

	for( QList< LJ::IO::Mood * >::ConstIterator it = d->moodList.begin();
	     it != d->moodList.end();
	     ++it )
	{
		d->moodComboBox->addItem( (*it)->moodName() );
	}

	connect( account->moodManager(), SIGNAL(moodAdded( LJ::IO::Mood * ) ),
	         this, SLOT( addMood( LJ::IO::Mood * ) ) );

	connect( d->moodComboBox, SIGNAL( activated( int ) ),
	         this, SLOT( emitMoodChanged( void ) ) );
	connect( d->moodNameLineEdit, SIGNAL( textChanged ( const QString & ) ),
	         this, SLOT( emitMoodChanged( void ) ) );
}

LJ::UI::MoodSelector::~MoodSelector( void )
{
	delete d;
}

void LJ::UI::MoodSelector::addMood( LJ::IO::Mood * newMood )
{
	if( !newMood ) {
		return;
	}
	
	int position = d->insertMood( newMood );
	d->moodComboBox->addItem( newMood->moodName(), position );
}

void LJ::UI::MoodSelector::emitMoodChanged( void )
{
	emit moodChanged();
}

unsigned int LJ::UI::MoodSelector::moodID( void ) const
{
	if( d->moodList.isEmpty() ) {
		return 0;
	}

	if( d->moodComboBox->currentIndex() >= d->moodList.count() ) {
		return 0;
	}

	if( d->moodComboBox->currentIndex() <= 0 ) {
		return 0;
	}

	return ( d->moodList.at( d->moodComboBox->currentIndex() - 1 ) )->moodID();
}

void LJ::UI::MoodSelector::setMoodID( unsigned int id )
{
	if( !id ) {
		d->moodComboBox->setCurrentIndex( 0 );
	}
	
	if( d->moodList.isEmpty() ) {
		return;
	}

	int position = 0;
	for( QList< LJ::IO::Mood * >::ConstIterator it = d->moodList.begin();
	     it != d->moodList.end();
	     ++it, ++position )
	{
		if( (*it)->moodID() == id ) {
			d->moodComboBox->setCurrentIndex( position + 1 );
		}
	}
}

QString LJ::UI::MoodSelector::moodName( void ) const
{
	return d->moodNameLineEdit->text();
}

void LJ::UI::MoodSelector::setMoodName( const QString moodName )
{
	d->moodNameLineEdit->setText( moodName ); 
}
