/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljusericonselector.h"
#include "ljusericonselector.moc"

#include "../livejournalaccount.h"
#include "../ljio/userpicturemanager.h"

#include <klocale.h>
#include <kdebug.h>

#include <QAbstractListModel>
#include <QItemDelegate>
#include <QPainter>

//-------------
//IconListModel
//-------------
LJ::UI::IconListModel::IconListModel( const LJ::IO::UserPictureManager * manager,
                                      QObject * parent ) :
QAbstractListModel( parent )
{
	m_icons.append( 0 );
	addUserPicture( manager->defaultPicture() );
	QList< LJ::IO::UserPicture * > icons = manager->pictures();
	for( int i = 0; i < icons.size(); ++i ) {
		addUserPicture( icons[i] );
	}

	connect( manager,
	         SIGNAL( pictureAdded( LJ::IO::UserPicture * ) ),
	         SLOT( addUserPicture( LJ::IO::UserPicture * ) ) );
	connect( manager,
	         SIGNAL( pictureRemoved( LJ::IO::UserPicture * ) ),
	         SLOT( removeUserPicture( LJ::IO::UserPicture * ) ) );
}

LJ::UI::IconListModel::~IconListModel( void )
{}


int LJ::UI::IconListModel::rowCount( const QModelIndex & parent ) const
{
	if( parent.isValid() ) {
		return 0;
	}

	return m_icons.size();
}

QVariant LJ::UI::IconListModel::data( const QModelIndex & index, int role ) const
{
	if( !index.isValid() ) {
		return QVariant();
	}

	const  LJ::IO::UserPicture * const icon = m_icons.at( index.row() );
	if( !icon ) {
		if( index.row() == 0
		    && role == Qt::DisplayRole )
		{
			return i18n( "(Default)" );
		}
		return QVariant();
	}

	if( role == Qt::DisplayRole ) {
		if( icon->keyword().isEmpty() ) {
			return i18n( "(Default)" );
		}
		
		return icon->keyword();
	}
	else  if( role == Qt::DecorationRole ) {
		return QPixmap::fromImage( icon->picture() );
	}
	else if( role == ImageRole ) {
		return icon->picture();
	}

	return QVariant();
}

void LJ::UI::IconListModel::updateUserPicture( LJ::IO::UserPicture * picture )
{
	if( !picture ) {
		return;
	}

	const int index = m_icons.indexOf( picture );
	if( index == -1 ) {
		return;
	}

	const QModelIndex modelIndex = createIndex( 0, 0, picture );
	emit dataChanged( modelIndex, modelIndex );
}

void LJ::UI::IconListModel::addUserPicture( LJ::IO::UserPicture * picture )
{
	//kDebug() << "adding picture: " << picture << endl;
	if( !picture ) {
		return;
	}

	if( picture->keyword().isEmpty() ){ //default picture -> first item
		m_icons[0] = picture;
		const QModelIndex index = createIndex( 0, 0, picture );
		emit dataChanged( index, index );
	}
	else {
		beginInsertRows( QModelIndex(), m_icons.size(), m_icons.size() );
		m_icons.append( picture );
		endInsertRows();
	}

	connect( picture,
	         SIGNAL( pictureChanged( LJ::IO::UserPicture * ) ),
	         SLOT( updateUserPicture( LJ::IO::UserPicture * ) ) );
}

void LJ::UI::IconListModel::removeUserPicture( LJ::IO::UserPicture * picture )
{
	if( !picture ) {
		return;
	}

	if( picture->keyword().isEmpty() ) {
		m_icons[0] = 0;
		const QModelIndex index = createIndex( 0, 0, picture );
		emit dataChanged( index, index );
	}
	else {
		const int index = m_icons.indexOf( picture );
		if( index == -1 ) {
			return;
		}

		beginRemoveRows( QModelIndex(),
		                 index,
		                 index );
		m_icons.removeAll( picture );
		endRemoveRows();
	}
}

//----------------
//UserIconDelegate
//----------------
class LJ::UI::IconDelegate : public QItemDelegate
{
public:
	IconDelegate( QObject * parent = 0 );
	virtual ~IconDelegate( void );
	
    virtual void paint(QPainter * painter,
                       const QStyleOptionViewItem & option,
                       const QModelIndex &index) const;
    virtual QSize sizeHint(const QStyleOptionViewItem & option, 
                           const QModelIndex & index) const;
private:
	static const int sm_margin = 5;
	static const int sm_maxIconSize = 100;
};

LJ::UI::IconDelegate::IconDelegate( QObject * parent ):
QItemDelegate( parent )
{
}

LJ::UI::IconDelegate::~IconDelegate( void )
{}

void LJ::UI::IconDelegate::paint( QPainter * painter,
                          const QStyleOptionViewItem & option,
                          const QModelIndex & index ) const
{
	//background
	painter->save();
	painter->fillRect( option.rect, option.palette.base() );
	painter->restore();

	//focus
	if( option.state & QStyle::State_Selected ) {
		painter->save();
		painter->setPen( option.palette.highlight().color() );
		painter->drawRect(option.rect.adjusted(2, 2, -2, -2));
		painter->restore();
	}

	//image
	QImage image = index.data( IconListModel::ImageRole ).value<QImage>();
	const QRect imageRect( option.rect.x() + sm_margin,
	                       option.rect.y() + sm_margin,
	                       sm_maxIconSize,
	                       sm_maxIconSize );
	if( !image.isNull() ) {
		if( image.height() > sm_maxIconSize
		    || image.width() > sm_margin )
		{
			image = image.scaled( sm_maxIconSize,
			                      sm_maxIconSize,
			                      Qt::KeepAspectRatio,
			                      Qt::SmoothTransformation );
		}
		painter->drawPixmap( imageRect, QPixmap::fromImage( image ) );
	}

	//text
	painter->save();
	const QRect textRect( option.rect.x() + 2 * sm_margin + sm_maxIconSize,
	                      option.rect.y() + sm_margin,
	                      option.rect.right() - imageRect.width(),
	                      imageRect.height() );
	QFont font = painter->font();
	font.setWeight(QFont::Bold);
	painter->setFont(font);
	painter->drawText( textRect,
	                   Qt::AlignLeft|Qt::AlignVCenter|Qt::TextWordWrap,
	                   index.data().toString() );
	painter->restore();
}

QSize LJ::UI::IconDelegate::sizeHint( const QStyleOptionViewItem & option,
                              const QModelIndex & index ) const
{
	Q_UNUSED( option );
	Q_UNUSED( index );
	//width, height
	return QSize( 1, sm_maxIconSize + 2 * sm_margin );
}

//----------------
//UserIconSelector
//----------------
class LJ::UI::UserIconSelector::Private
{
public:
	Private( void ) {}
};

LJ::UI::UserIconSelector::UserIconSelector( LiveJournalAccount * account, QWidget * parent ) :
KComboBox( parent ),
d( new Private() )
{
	setModel( new IconListModel( account->userPictureManager(), this ) );
	setItemDelegate( new IconDelegate( this ) );

	connect( model(),
	         SIGNAL( dataChanged( const QModelIndex &, const QModelIndex & ) ),
	         SLOT( setState( void ) ) );
	connect( model(),
	         SIGNAL( rowsInserted( const QModelIndex &, int, int ) ),
	         SLOT( setState( void ) ) );
	connect( model(),
	         SIGNAL( rowsRemoved( const QModelIndex &, int, int ) ),
	         SLOT( setState( void ) ) );

	setState();
}

LJ::UI::UserIconSelector::~UserIconSelector( void )
{
	delete d;
}

QString LJ::UI::UserIconSelector::keyword( void )
{
	if( currentIndex() ) {
		return currentText();
	}

	return QString::null;
}

void LJ::UI::UserIconSelector::setState( void )
{
	setEnabled( count() > 1 );
}
